Attribute VB_Name = "Module1"
Global Application As Object
Global Workbook As Object
Global DebugOn

Declare Function GetStdHandle Lib "kernel32" (ByVal _
           nStdHandle As Long) As Long
Declare Function AllocConsole Lib "kernel32" () As Long
Declare Function WriteFile Lib "kernel32" _
           (ByVal hConsoleOutput As Long, lpBuffer As Any, ByVal _
           nNumberOfCharsToWrite As Long, lpNumberOfCharsWritten As Long, _
           lpReserved As Any) As Long

Public Const STD_OUTPUT_HANDLE = -11&
Sub Usage()
    ReturnError ("PerfstatGrapher <fullpath to psf_files> [/D] [/W WordTemplatePath] [/E ExcelTemplatePath]")
End Sub

Sub Main()
    
    Dim Result As String
    Dim Args As Variant
    Dim WordTemplatePath As String
    Dim ExcelTemplatePath As String
    Dim PsfFilesPath As String
    
    DebugOn = False
    WordTemplatePath = ""
    ExcelTemplatePath = ""
    
    Args = Split(Command)
    PsfFilesPath = Args(0)
    ' Args[0] is psf_files path
    ' Other Args, if any, include :
    '   /W Word Doc Template Path
    '   /E Excel Template Path
    For i = 0 To UBound(Args)
        If (Args(i) = "/D") Then
            DebugOn = True
        ElseIf (Args(i) = "/h") Then
            Usage
            GoTo EndProgram
        ElseIf (Args(i) = "/W") Then
            WordTemplatePath = Args(i + 1)
        ElseIf (Args(i) = "/E") Then
            ExcelTemplatePath = Args(i + 1)
        End If
    Next i
    
    If DebugOn Then
        ReturnError ("Finished Command Line Parsing")
    End If
    If (PsfFilesPath = "") Then
        Usage
    Else
        Result = PsfBuildCharts(PsfFilesPath, _
                                WordTemplatePath, _
                                ExcelTemplatePath)
        If (Result <> "") Then
            ReturnError (Result)
        End If
    End If
EndProgram:

End Sub
Sub ReturnError(Msg As String)
    Dim Result As Long, sOut As String, cWritten As Long
    
    hConsole = GetStdHandle(STD_OUTPUT_HANDLE)
    If hConsole = 0 Then
        Result = AllocConsole
        If Result = 0 Then MsgBox "Couldn't AllocConsole"
        hConsole = GetStdHandle(STD_OUTPUT_HANDLE)
        If hConsole = 0 Then MsgBox "Couldn't allocate STDOUT"
    End If
    sOut = Msg & vbCrLf
    Result = WriteFile(hConsole, ByVal sOut, Len(sOut), cWritten, _
                           ByVal 0&)
    
End Sub
Function PsfBuildCharts(PsfPath As String, _
                        DocPath As String, _
                        ExcelPath As String) As String
' This procedure automates the creation of the perfstat
' graphs. PsfPath is a pathname to the directory/folder
' containing the output of psf_get_all_data.pl
    Dim ExcelTemplatePath As String
    Dim DocTemplatePath As String
    Dim SaveAsPath As String
    Dim CheckPathsResult As String

    PsfPath = PsfPath & "\"
    If (DocPath = "") Then
        DocTemplatePath = _
            "C:\Program Files\na_psf\doc_templates\NAS_SAN_Perf.doc"
    Else
        DocTemplatePath = DocPath
    End If
    If (ExcelPath = "") Then
        ExcelTemplatePath = _
            "C:\Program Files\na_psf\Excel_templates\Perfstat.xlt"
    Else
        ExcelTemplatePath = ExcelPath
    End If
    CheckPathsResult = CheckPaths(DocTemplatePath, ExcelTemplatePath, _
                                    PsfPath)
    If DebugOn Then
        ReturnError ("Finished CheckPaths")
    End If
    If (CheckPathsResult <> "") Then
        PsfBuildCharts = CheckPathsResult
        Exit Function
    End If
    PsfBuildCharts = ""
    On Error GoTo AppError
    
    Set Application = CreateObject("Excel.Application")
    ' By default Excel will be invisible
    Set Workbook = Application.Workbooks.Add(Template:=ExcelTemplatePath)
    If DebugOn Then
        ReturnError ("Finished Excel Add Workbook")
    End If
    Application.Run "MakeAllSheetsFromPath", PsfPath
    If DebugOn Then
        ReturnError ("Finished MakeAllSheetsFromPath")
    End If
    Application.Run "MakeAllChartsInternal"
    SaveAsPath = PsfPath & "Perfstat.doc"
    Application.Run "NonInteractiveExportToWord", DocTemplatePath, SaveAsPath
    SaveAsPath = PsfPath & "Perfstat.htm"
    Application.Run "NonInteractiveExportToWord", DocTemplatePath, SaveAsPath
    SaveAsPath = PsfPath & "Perfstat.xls"
    If Dir(SaveAsPath) <> "" Then
        On Error Resume Next
        Kill "C:\temp.xls"
        On Error GoTo AppError
        Workbook.SaveAs "C:\temp.xls"
        Workbook.Close savechanges:=False
        Kill SaveAsPath
        Name "C:\temp.xls" As SaveAsPath
    Else
        Workbook.SaveAs SaveAsPath
        Workbook.Close savechanges:=False
    End If
    Set Workbook = Nothing
    Application.Quit
    Exit Function
    
AppError:
    PsfBuildCharts = "Error from: " & Err.Source & ": " & Err.Number
    Workbook.Close
    Set Workbook = Nothing
    Application.Quit
    Exit Function

End Function
Function CheckPaths(DocTemplatePath As String, _
                ExcelTemplatePath As String, _
                PsfPath As String) As String
    CheckPaths = ""
    On Error GoTo BadDocTemplate
    Open DocTemplatePath For Input As #1
    On Error GoTo BadExcelTemplate
    Open ExcelTemplatePath For Input As #2
    On Error GoTo BadPsfPath
    ChDir PsfPath
        
    Close
    Exit Function
BadDocTemplate:
    CheckPaths = "Cannot Open " & DocTemplatePath
    Resume Next
BadExcelTemplate:
    CheckPaths = "Cannot Open " & ExcelTemplatePath
    Resume Next
BadPsfPath:
    CheckPaths = "Directory Does Not Exist " & PsfPath
    Resume Next
End Function

