#!/usr/bin/env perl 

# This script is provided "as is" it is
# not warranted but may be supported on a limited basis
# by Network Appliance.
#
# psf_compute_vol_rt.pl
#
#   PerfStatFilter to compute vol avg RT based on usecs and chain
#   v.01   02-03-2005 : Created.
#

use Getopt::Std;
getopts("dh");

if ($opt_h) { usage(); }

if ($opt_d) {
   $debug = 1;  
} else {
   $debug = 0;
}

$chain_file = $ARGV[0];
$usecs_file = $ARGV[1];

open(INPUTA, '<' . $chain_file) || die 'Unable to open chain file (arg1) .';
open(INPUTB, '<' . $usecs_file) || die 'Unable to open usecs file (arg2) .';

$sample = 0;
$next_instance = 1;
while ($Chain_Line = <INPUTA>) {
	$Chain_Line =~ s/\r//g;     # Remove DOS carraige returns
        chomp $Chain_Line;
        @chain_line = split (/,/, $Chain_Line);
	if ($Chain_Line =~ /^Timestamp/) {
		# Save instances to compare with usecs
		@chain_instances = @chain_line;
	}
	else {
        	$chain_timestamp = $chain_line[0];
	}
	$Usecs_Line = <INPUTB>;
	$Usecs_Line =~ s/\r//g;     # Remove DOS carraige returns
        chomp $Usecs_Line;
        @usecs_line = split (/,/, $Usecs_Line);
	if ($Usecs_Line =~ /^Timestamp/) {
		# Check instances with Chain
		# If OK print the header
		$num_instances = scalar @usecs_line - 1;
		for ($i=0; $i <= $num_instances; $i++) {
			if ( $usecs_line[$i] = $chain_line[$i] ) {
				printf ("%s, ",  $usecs_line[$i]);
			} else {
				printf ("Instances do not match from psf_compute_vol_rt.pl\n");
				exit();
			}
		}
		printf ("\n");
		next;
	}
	else  {
        	$usecs_timestamp = $usecs_line[0];
	} 
	if ($usecs_timestamp ne $chain_timestamp) {
		printf ("Timestamps do not match from psf_compute_vol_rt.pl \n");
		exit();
	}
	printf ("%s, ", $usecs_timestamp);
	for ($i=1; $i <= $num_instances; $i++) {
		printf ("%.1f, ", ($chain_line[$i] * $usecs_line[$i]) / 1000 );	
	}
	printf ("\n");
}

sub usage {
   print <<EOM;
psf_compute_vol_rt.pl [-h] <chain_file>  <usecs_file>

EOM
exit;

}
