#!/usr/bin/env perl 

# This script is provided "as is" it is
# not warranted but may be supported on a limited basis
# by Network Appliance.
#
# psf_compute_xfer_sz.pl
#
#   PerfStatFilter to compute lun avg xfer size based on KB/s and IOPS
#   v.01   02-03-2005 : Created.
#

use Getopt::Std;
getopts("dh");

if ($opt_h) { usage(); }

if ($opt_d) {
   $debug = 1;  
} else {
   $debug = 0;
}

$iops_file = $ARGV[0];
$kbs_file = $ARGV[1];

open(INPUTA, '<' . $iops_file) || die 'Unable to open iops file (arg1) .';
open(INPUTB, '<' . $kbs_file) || die 'Unable to open kbs file (arg2) .';

$sample = 0;
$next_instance = 1;
while ($IOPS_Line = <INPUTA>) {
	$IOPS_Line =~ s/\r//g;     # Remove DOS carraige returns
        chomp $IOPS_Line;
        @iops_line = split (/,/, $IOPS_Line);
	if ($IOPS_Line =~ /^Timestamp/) {
		# Save instances to compare with KBS
		@iops_instances = @iops_line;
	}
	else {
        	$iops_timestamp = $iops_line[0];
	}
	$KBS_Line = <INPUTB>;
	$KBS_Line =~ s/\r//g;     # Remove DOS carraige returns
        chomp $KBS_Line;
        @kbs_line = split (/,/, $KBS_Line);
	if ($KBS_Line =~ /^Timestamp/) {
		# Check instances with IOPS
		# If OK print the header
		$num_instances = @kbs_line - 1;
		for ($i=0; $i <= $num_instances; $i++) {
			if ( $kbs_line[$i] = $iops_line[$i] ) {
				printf ("%s, ",  $kbs_line[$i]);
			} else {
				printf ("Instances do not match from psf_compute_xfer_sz.pl\n");
				exit();
			}
		}
		printf ("\n");
		next;
	}
	else  {
        	$kbs_timestamp = $kbs_line[0];
	} 
	if ($kbs_timestamp ne $iops_timestamp) {
		printf ("Timestamps do not match from psf_compute_xfer_sz.pl \n");
		exit();
	}
	printf ("%s, ", $kbs_timestamp);
	for ($i=1; $i <= $num_instances; $i++) {
		if ($iops_line[$i] > 0 ) {
			printf ("%.1f, ", $kbs_line[$i] / $iops_line[$i] );	
		} else {
			printf ("0.0, ");
		}
	}
	printf ("\n");
}

sub usage {
   print <<EOM;
psf_compute_xfer_sz.pl [-h] <iops_file>  <kbs_file>

EOM
exit;

}
