#!/usr/bin/env perl 

use Getopt::Std;
use File::Spec::Functions;

getopts("dhF:I:O:");

if ($opt_h) { usage(); }

if ($opt_d) {
   $debug = 1;
}

if ($opt_I) {
   $InputDirsFile = $opt_I;
} else {
   usage();
}

if ($opt_O) { 
   $OutputDir = $opt_O;  
} else {
   usage();
}

if ($opt_F) {
   $File = $opt_F;
   @FilesToConsolidate = ($File);
} else {
   @FilesToConsolidate = (
				"cifs_hist_stats.txt",
				"cifs_hist.txt",
				"cp_kinds.txt",
			 	"cp_times.txt",
				"fcp_cm_stats.txt",
				"lun_qlen.txt",
				"lun_riops.txt",
				"lun_rkbs.txt",
				"lun_rsize.txt",
				"lun_rt.txt",
				"lun_wiops.txt",
				"lun_wkbs.txt",
				"lun_wsize.txt",
				"lun_summ_qlen.txt",
				"lun_summ_riops.txt",
				"lun_summ_rkbs.txt",
				"lun_summ_rsize.txt",
				"lun_summ_rt.txt",
				"lun_summ_wiops.txt",
				"lun_summ_wkbs.txt",
				"lun_summ_wsize.txt",
				"lun_summ_prtnr_tmp.txt",
				"nfsv3_hist.txt",
				"nfsv3_hist_stats.txt",
				"statit.txt",
				"vol_cprchain.txt",
				"vol_cpriops.txt",
				"vol_cprkbs.txt",
				"vol_cprusecs.txt",
				"vol_iops.txt",
				"vol_rchain.txt",
				"vol_riops.txt",
				"vol_rkbs.txt",
				"vol_rmsecs.txt",
				"vol_rusecs.txt",
				"vol_util.txt",
				"vol_wchain.txt",
				"vol_wiops.txt",
				"vol_wkbs.txt",
				"vol_wmsecs.txt",
				"vol_wusecs.txt",
				"vol_lun_riops.txt",
				"vol_lun_wiops.txt",
				"vol_lun_rkbs.txt",
				"vol_lun_wkbs.txt",
				"vol_lun_qlen.txt",
				"rg_rkbs.txt",
				"rg_wkbs.txt",
				"wafl.txt",
				"wafl_ops.txt",
				"wafl_susp.txt");

   @FilesToCopy = (		"filer.txt",
				"versions.txt",
				"vol_info.txt"); 
}

mkdir $OutputDir;
open (INPUTDIRS, '<', $InputDirsFile) || die 'Unable to open Input Dir file.';
my @InputDirs = <INPUTDIRS>;
chomp @InputDirs;

if ($debug) {
	printf ("Output Directory : %s\n", $OutputDir);
	foreach $input_dir (@InputDirs) {
		printf ("Input Directory : %s \n", $input_dir);
	}
}

foreach $copy_file (@FilesToCopy) {
	$outfile = catfile ($OutputDir, $copy_file);
	$input_file = catfile ($InputDirs[0], $copy_file);
	$cmd = "cp $input_file $outfile";
	if ($debug) {
		printf ("Cmd : %s \n", $cmd);
	}
	system ($cmd);
}

foreach $cons_file (@FilesToConsolidate) {
	$found_data = 0;
	$outfile = catfile ($OutputDir, $cons_file);
	undef $hdr;
	open (OUTPUTF, '>', $outfile)  || die 'Unable to open output $outfile ';
	foreach $input_dir (@InputDirs) {
		$input_file = catfile ($input_dir, $cons_file); 
		if ($debug) {
			printf ("Processing file : %s \n", $input_file);
		}
		open (INPUTF, '<', $input_file) || die "Unable to open input file $input_file";
		undef @data_lines;
		$num_lines_read = 0;
		while ($Line = <INPUTF>) { 
			$num_lines_read++;
			if ($Line =~ /^\d\d\/\d\d\/\d\d\d\d/) {
				# Data Line 
				# Special filter to trim down wafl_susp.txt 
				if ($cons_file eq "wafl_susp.txt") {
					if ($Line =~ /WAFL_READ|WAFL_WRITE|WAFL_SMB/) {
						push (@data_lines, $Line);
					} else {
						next;
					}
				} else {
					push (@data_lines, $Line);
				}
			} elsif ($Line =~ /No sample/ ) {
				next;
			} else {
				push (@hdr_lines, $Line);	
			}
		}
		if ( scalar (@data_lines) ) {
			$found_data = 1;
		}
		if ($debug) {
			printf ("Num lines read : %d Num lines kept :%d \n", $num_lines_read, scalar (@data_lines));
		}
		if (!$hdr) {
			$hdr = 1;
			foreach $hdr_line (@hdr_lines) {
				printf (OUTPUTF "%s", $hdr_line);
			}
		}  
		foreach $line ( @data_lines ) {
			printf (OUTPUTF "%s", $line);
		}
		close (INPUTF);
		undef @hdr_lines;
	}
	close (OUTPUTF);
	if (!$found_data) {
		unlink $outfile;
	}
}

sub usage {
	print <<EOM;
psf_consolidate.pl -O <output_dir> -I <file_of_input_dir_names> [-F <file>]

	Consolidates psf summary output files from several psf directories.
	If -F <file> is specified, only that file is consolidated
	(e.g. statit_summ.txt)  
EOM
exit;

}
