#!/usr/bin/env perl 

# This script is provided "as is" it is
# not warranted but may be supported on a limited basis
# by Network Appliance.
#
# psf_wafl_cp_kinds.pl
#
#   PerfStatFilter for certain WAFL CP related stats .
#
#   v0.03  11-09-2004 : Removed call to psf_versions.pl since none of values are used.
#   v0.02  03-23-2004 : Added -f <filer> for multi-filer perfstat files.
#                       Only tested against perfstat 5.20 Linux version.#
#   v0.01  02-25-2004 : Initial implementation.
#

# EXPORTED CONSTANTS (Not really, you have to copy them)

use constant _NA_TIMESTAMP                  => 0;
use constant _NA_CP_MI_SECS                 => 1;
use constant _NA_CP_NUM_CPS                 => 2;
use constant _NA_CP_PERCENT_FROM_TIMER      => 3;
use constant _NA_CP_PERCENT_FROM_SNAPSHOT   => 4;
use constant _NA_CP_PERCENT_FROM_LOW_WATER  => 5;
use constant _NA_CP_PERCENT_FROM_HIGH_WATER => 6;
use constant _NA_CP_PERCENT_FROM_LOG_FULL   => 7;
use constant _NA_CP_PERCENT_FROM_CP         => 8;
use constant _NA_CP_PERCENT_FROM_FLUSH      => 9;
use constant _NA_CP_PERCENT_FROM_SYNC       => 10;
use constant _NA_CP_PERCENT_FROM_VBUF       => 11;
use constant _NA_CP_PERCENT_FROM_OTHER      => 12;


use constant FIRST_PERCENT                  => 3;
use constant LAST_PERCENT                   => 12;

# Imported
use constant    _NA_VERS_PERFSTAT_MAJOR => 0;
use constant    _NA_VERS_PERFSTAT_MINOR => 1;
use constant    _NA_VERS_ONTAP_VERSION  => 2;
use constant    _NA_VERS_ONTAP_MAJOR    => 3;
use constant    _NA_VERS_ONTAP_MINOR    => 4;
use constant    _NA_VERS_FILER_MODEL    => 5;
use constant    _NA_VERS_FILER_MAGIC1   => 6;
use constant    _NA_VERS_FILER_MAGIC2   => 7;
use constant    _NA_VERS_PERFSTAT_PLATFORM => 8;


use Getopt::Std;
getopts("dDf:hHnT:");

if ($opt_h) { usage(); }

if ($opt_d) {
   $debug = 1;
} else {
   $debug = 0;
}

if ($opt_D) {
   $data_plus_header = 1;
} else {
   $data_plus_header = 0;
}

if ($opt_f) {
   $filer = $opt_f;
}

if ($opt_H) {
   $header = 1;
} else {
   $header = 0;
}

if ($opt_n) {
   $nice_output = 1;
} else {
   $nice_output = 0;
}

if ($opt_T) {
   $timestamp = $opt_T;
}

$cp_from_timer, $cp_from_snapshot, $cp_from_low_water, $cp_from_high_water, $cp_from_log_full;
$cp_from_cp, $cp_from_flush, $cp_from_sync, $cp_from_vbuf, $cp_from_other;

$data[_NA_CP_PERCENT_FROM_TIMER]      = \$cp_from_timer;
$data[_NA_CP_PERCENT_FROM_SNAPSHOT]   = \$cp_from_snapshot;
$data[_NA_CP_PERCENT_FROM_LOW_WATER]  = \$cp_from_low_water;
$data[_NA_CP_PERCENT_FROM_HIGH_WATER] = \$cp_from_high_water;
$data[_NA_CP_PERCENT_FROM_LOG_FULL]   = \$cp_from_log_full;
$data[_NA_CP_PERCENT_FROM_CP]         = \$cp_from_cp;
$data[_NA_CP_PERCENT_FROM_FLUSH]      = \$cp_from_flush;
$data[_NA_CP_PERCENT_FROM_SYNC]       = \$cp_from_sync;
$data[_NA_CP_PERCENT_FROM_VBUF]       = \$cp_from_vbuf;
$data[_NA_CP_PERCENT_FROM_OTHER]      = \$cp_from_other;

$header_line = "Timestamp,      Seconds, NumCPs, %Timer, %Snapshot, %LowH2O, %HighH2O, %LogFull, %btbCP, %Flush, %Sync, %Vbuf, %Other, ";

if (  !$nice_output && $header ) {
   printf ("%s \n", $header_line);
   if ( !$data_plus_header ) {
      exit;
   }
}

$perfstat_file = $ARGV[0];
open(INPUTA, '<' . $perfstat_file) || die 'Unable to open perfstat file.';

$seconds = 0;
$found_wafl_stats = 0;

if (!$timestamp) {
   if ( $filer ) {
      $timestamp = `psf_filer_timestamp.pl -f $filer $perfstat_file`;
   }
   else {
      $timestamp = `psf_filer_timestamp.pl $perfstat_file`;
   }
}
chop $timestamp;

# Get to POSTSTATs 
while ($Line = <INPUTA>) {
   if ($Line =~ /POSTSTAT/ ) {  
      last ;
   }
}

$done = 0;
while ($Line = <INPUTA>) {
   if ( $done ) {
      last;
   }
   if ( ( $Line =~ /^=-=-/ ) &&
           ( $Line =~ /POSTSTATS/ ) &&
           ( $Line =~ /wafl_susp -w/ ) ) {
      @line = split (/\s+/, $Line);
      $curr_filer = $line[2];
      if ( $debug ) {
         printf ("Current statit filer : %s \n", $curr_filer);
      }
   }
   elsif ( ($Line =~ /WAFL statistics over / )  &&
           ( (!$filer) ||
             ($filer =~ /^$curr_filer/) ) ) {
      if ( $debug ) { print "Found wafl_susp \n"; }
      $found_wafl_stats = 1;
      @line = split (/ +/, $Line);
      $seconds = $line[3];
      $num_cps = 0;
      while ($Line = <INPUTA>) { 
               if ($Line =~ /POSTSTAT/ ) {
                  $done = 1;
                  last;
               }
               if ($Line =~ /^cp_from_timer/ ){
                  @line = split (/ +/, $Line);
                  $cp_from_timer = $line[2];
                  $num_cps += $cp_from_timer;
                  if ($debug) {printf ("cp_from_timer : %d \n", $cp_from_timer);}
                  next;
               }
               elsif ($Line =~ /^cp_from_snapshot/ ){
                  @line = split (/ +/, $Line);
                  $cp_from_snapshot = $line[2];
                  $num_cps += $cp_from_snapshot;
                  if ($debug) {printf ("cp_from_snapshot : %d \n", $cp_from_snapshot);}
                  next;
               }
               elsif ($Line =~ /^cp_from_low_water/ ){
                  @line = split (/ +/, $Line);
                  $cp_from_low_water = $line[2];
                  $num_cps += $cp_from_low_water;
                  if ($debug) {printf ("cp_from_low_water: %d \n", $cp_from_low_water);}
                  next;
               }
               elsif ($Line =~ /^cp_from_high_water/ ){
                  @line = split (/ +/, $Line);
                  $cp_from_high_water = $line[2];
                  $num_cps += $cp_from_high_water;
                  if ($debug) {printf ("cp_from_high_water: %d \n", $cp_from_high_water);}
                  next;
               }
               elsif ($Line =~ /^cp_from_log_full/ ){
                  @line = split (/ +/, $Line);
                  $cp_from_log_full = $line[2];
                  $num_cps += $cp_from_log_full;
                  if ($debug) {printf ("cp_from_log_full: %d \n", $cp_from_log_full);}
                  next;
               }
               elsif ($Line =~ /^cp_from_cp/ ){
                  @line = split (/ +/, $Line);
                  $cp_from_cp = $line[2];
                  $num_cps += $cp_from_cp;
                  if ($debug) {printf ("cp_from_cp : %d \n", $cp_from_cp );}
                  next;
               }
               elsif ($Line =~ /^cp_from_flush/ ){
                  @line = split (/ +/, $Line);
                  $cp_from_flush = $line[2];
                  $num_cps += $cp_from_flush;
                  if ($debug) {printf ("cp_from_flush : %d \n", $cp_from_flush );}
                  next;
               }
               elsif ($Line =~ /^cp_from_sync/ ){
                  @line = split (/ +/, $Line);
                  $cp_from_sync = $line[2];
                  $num_cps += $cp_from_sync;
                  if ($debug) {printf ("cp_from_sync : %d \n", $cp_from_sync );}
                  next;
               }
               elsif ($Line =~ /^cp_from_vbuf/ ){
                  @line = split (/ +/, $Line);
                  $cp_from_vbuf = $line[2];
                  $num_cps += $cp_from_vbuf;
                  if ($debug) {printf ("cp_from_vbuf : %d \n", $cp_from_vbuf );}
                  next;
               }
               elsif ($Line =~ /^cp_from/ ){
                  @line = split (/ +/, $Line);
                  $cp_from_other = $line[2];
                  $num_cps += $line[2];
                  if ($debug) {printf ("cp_from other : %d \n", $line[2] );}
                  next;
               }

      }
   }
}
if ( $found_wafl_stats ) {

   if ( $nice_output ) {
      printf ("Measurement Interval (seconds)          : \t %d \n", $seconds);
      printf ("Number of CPs                           : \t %d \n", $num_cps);
      printf ("Percent from Timer                      : \t %d \n", ($cp_from_timer/$num_cps)*100);
      printf ("Percent from Snapshot                   : \t %d \n", ($cp_from_snapshot/$num_cps)*100);
      printf ("Percent from Low Water                  : \t %d \n", ($cp_from_low_water/$num_cps)*100);
      printf ("Percent from High Water                 : \t %d \n", ($cp_from_high_water/$num_cps)*100);
      printf ("Percent from Log Full                   : \t %d \n", ($cp_from_log_full/$num_cps)*100);
      printf ("Percent from CP                         : \t %d \n", ($cp_from_cp/$num_cps)*100);
      printf ("Percent from Flush                      : \t %d \n", ($cp_from_flush/$num_cps)*100);
      printf ("Percent from Sync                       : \t %d \n", ($cp_from_sync/$num_cps)*100);
      printf ("Percent from Vbuf                       : \t %d \n", ($cp_from_vbuf/$num_cps)*100);
      printf ("Percent from Other                      : \t %d \n", ($cp_from_other/$num_cps)*100);
   }
   else {
      printf ("%s, %d, %d, ", $timestamp, $seconds, $num_cps);
      for ( $i=FIRST_PERCENT; $i <= LAST_PERCENT; $i++ ) {
         printf ("%d,    ", (${$data[$i]}/$num_cps)*100 );
      }
      printf ("\n");
   }
}

sub usage {
   print <<EOM;
psf_cp_kinds.pl [-dDHhn]  [-f <filer>]  <perfstat filename>

Summarizes the types of cps from from perfstat outpt files.

    Arguments and switches -
       -d          Print debug messages.
       -D          Print data as well as a header. Only relevent if -H specified. 
       -f <filer>  For multi-filer perfstats choose this one.
       -H          Print a header line only unless -D specified. 
       -h          This message.
       -n          Print Nice output, a value per line labeled with what it is.
EOM
exit;

}

