#!/usr/bin/env perl 

# This script is provided "as is" it is
# not warranted but may be supported on a limited basis
# by Network Appliance.
#
# psf_extract_iterations.pl
#
#   PerfStatFilter for extracting one iteration from a multi-iteration
#	perfstat file. For use with Performance Analyzer. This is different 
#	from psf_split_iterations.pl, in that it extracts the entirety of the
#	iteration rather than just the performance data.  
#
#   v0.01  10-27-2005 : Initial implementation.
#


use Getopt::Std;
getopts("b:df:I:");

if ($opt_b) {
   $base = $opt_b;
} else {
   $base = 0;
}

if ($opt_h) { usage(); }

if ($opt_f) {
   $file_root = $opt_f;; 
} 

if ($opt_I) {
   $desired_iteration = $opt_I;
}
else {
   printf ("Usage: psf_extract_iteration.pl -I <iteration> <perfstat_file> \n");
   exit();
}

if ($opt_d) {
   $debug = 1; 
} else {
   $debug = 0;
}

$perfstat_file = $ARGV[0];

open(INPUTA, '<' . $perfstat_file) || die 'Unable to open perfstat file (arg1) .';

while ($Line = <INPUTA>) { 
   if ($Line =~ / Perfstat/ ) {    # save header stuff  
      push (@header, $Line);
      while ($Line = <INPUTA>) {
         if ($Line =~ / BEGIN Iteration/) {
	    if ($debug) {
		printf ("BEGIN Iteration found\n");
	    }
            last;
         } 
	 elsif ($Line =~ /^=-=-=-=/ ) {
            last;
         }
	 elsif ($Line =~ /=-=-=-=-=-= PERF / ) {
		# Try to fix lines that lost linefeeds
		$Line =~ s/=-=-=-=-=-= PERF/\n=-=-=-=-=-= PERF/;
		push (@header, $Line);
	 }	
         else {
            push (@header, $Line);
        }
      }
    }
    if ($Line =~ / BEGIN Iteration/) {
	if ($Line =~ / echo/) {
        	next;
      	}
      	close (OUTPUTA);
      	@line = split (/\s+/, $Line);
      	$iteration = $line[3];
      	if ($iteration == $desired_iteration) {
      		$outfile = sprintf ("perfstat.%04d.txt",$iteration);
      		open(OUTPUTA, '>'. $outfile) || die 'Cant open output ';
      		foreach $hdr_line (@header) {
         		printf (OUTPUTA "%s", $hdr_line);
      		}
      	}
      	else {
		next;
      	}

      	# If we get here we have the desired iteration and have finished printing
      	# the perfstat header. 
      	while ($Line = <INPUTA>) {
         	if ($Line =~ /END Iteration/) {
	    		if ($debug) {
				printf ("END Iteration found\n");
	    		}
            		last;
         	}
		elsif (($Line =~ /=-=-=-=-=-= PERF / ) &&
			!($Line =~ /^=-=-=-=-=-= PERF / )) {
			 # Try to fix lines that lost linefeeds
                	$Line =~ s/=-=-=-=-=-= PERF/\n=-=-=-=-=-= PERF/;
		}
         	printf (OUTPUTA "%s",$Line);
      	}
    }
}   
