#!/usr/bin/env perl 

# This script is provided "as is" it is
# not warranted but may be supported on a limited basis
# by Network Appliance.
#
# psf_fcp.pl
#
#   PerfStatFilter for certain stats from fcp stats.
#
#   v0.01  11-04-2003 : Initial implementation.
#

# EXPORTED CONSTANTS (Not really, you have to copy them)

use constant _NA_TIMESTAMP              => 0;
use constant _NA_FCP_QDEPTH             => 1;


use Getopt::Std;
getopts("dh");

if ($opt_h) { usage(); }

if ($opt_d) {
   $debug = 1;        
} else {
   $debug = 0;
}

if ($opt_n) {
   $nice_output = 1;  
   $avg_all_samples = 1; 
} else {
   $nice_output = 0;
}

$perfstat_file = $ARGV[0];
open(INPUTA, '<' . $perfstat_file) || die 'Unable to open perfstat file (arg1) .';

$sample=0;

@qdepth;

if ( !$nice_output && $header ) {
   print $header_line1;
   if ( $extended_stats ) {
      print $ext_header_line2;
   }
   else {
      print $header_line2;
   }
   if (!$data_plus_header) {
     exit;
   }
}
      
while ($Line = <INPUTA>) { 
   if ($Line =~ / Start time:/ ) {
      @line = split (/ +/, $Line);
      $mo = $line[4];
      $day = $line[5];
      $time = $line[6];
      $yr = $line[8];
      next;
   }
   elsif ($Line =~ /Queue Depth/ ) {
      if ( $debug ) { print "Found statit \n"; }
      @line = split (/ +/, $Line);
      $sample++;
      $qdepth[$sample] = @line[2];
      next;
   }
}
if ( $debug ) { printf ("Num samples found : %d \n", $sample); }

if ( $sample > 0 ) { 
       $mo_num = month_to_numeric ($mo);
       $timestamp = sprintf ("%d\/%d\/%d %s", $mo_num, $day, $yr, $time);
       printf ("%s, ", $timestamp);
       for ( $j=1; $j <= $sample; $j++ ) {
           printf ("%3d,", $qdepth[$j]);
       }
       printf ("\n");
}
else {
  print "No Samples found \n";
} 

# END OF MAIN 

sub month_to_numeric {
   local ($month) = @_ ;
   my $result;

   if ($month =~ /Jan/ ) {
        $result = 1;
   }
   elsif ($month =~ /Feb/ ) {
        $result = 2;
   }
   elsif ($month =~ /Mar/ ) {
        $result = 3;
   }
   elsif ($month =~ /Apr/ ) {
        $result = 4;
   }
   elsif ($month =~ /May/ ) {
        $result = 5;
   }
   elsif ($month =~ /Jun/ ) {
        $result = 6;
   }
   elsif ($month =~ /Jul/ ) {
        $result = 7;
   }
   elsif ($month =~ /Aug/ ) {
        $result = 8;
   }
   elsif ($month =~ /Sep/ ) {
        $result = 9;
   }
   elsif ($month =~ /Oct/ ) {
        $result = 10;
   }
   elsif ($month =~ /Nov/ ) {
        $result = 11;
   }
   elsif ($month =~ /Dec/ ) {
        $result = 12;
   }
   else {
        $result = -1;
   }
   return $result;
}

sub usage {
   print <<EOM;
psf_fcp.pl [-h]  <perfstat filename>

Summarizes the statit data from perfstat outpt files. 

    Arguments and switches - 
       -h          This message.
EOM
exit;

}
