#!/usr/bin/env perl 

# Put in option -I to only match files named 
# perfstat.<num>.txt 

use constant    _NA_VERS_PERFSTAT_MAJOR => 0;
use constant    _NA_VERS_PERFSTAT_MINOR => 1;
use constant    _NA_VERS_ONTAP_VERSION  => 2;
use constant    _NA_VERS_ONTAP_MAJOR    => 3;
use constant    _NA_VERS_ONTAP_MINOR    => 4;
use constant    _NA_VERS_FILER_MODEL    => 5;
use constant    _NA_VERS_FILER_MAGIC1   => 6;
use constant    _NA_VERS_FILER_MAGIC2   => 7;
use constant    _NA_VERS_PERFSTAT_PLATFORM => 8;
use constant    _NA_VERS_NUM_SAMPLES    => 9;
use constant    _NA_VERS_SECS_PER_SAMPLE => 10;
use constant    _NA_VERS_SECS_PER_ITERATION => 11;
use constant    _NA_VERS_NUM_FILER_CPUS     => 12;

use Getopt::Std;
use File::Spec::Functions;

getopts("de:hF:Is:S:");

if ($opt_h) { usage(); }

if ($opt_d) {
   $debug = 1;
}

if ($opt_e) {
   $end_time = $opt_e;
} else {
   $end_time = "";
}

if ($opt_s) {
   $start_time = $opt_s;
} else {
   $start_time = "";
}

if ($opt_F) {
   $MaxToFork = $opt_F;
} else {
   $MaxToFork = 1;
}

if ($opt_I) { 
   $iterations_only = 1; 
} else {
   $iterations_only = 0; 
   $tag = $ARGV[0];
   if (!$tag) {
        $tag = "perfstat";
   }
}

if ($opt_S) {
   $file_to_split = $opt_S;
}

$NumForked = 0;

if ($file_to_split) {
	$num_iterations = get_num_iterations ($file_to_split);
	if ($debug) { printf ("num_iterations: %d\n", $num_iterations);}
	if ($num_iterations > 1) {
		system ("psf_split_iterations.pl $file_to_split");
		$have_multiple_perfstat_files = 1;
		$iterations_only = 1;
	} else {
		$have_multiple_perfstat_files = 0;
	} 
}

if ($file_to_split && !$have_multiple_perfstat_files ) { 
	push @filenames, $file_to_split;
} else {
	opendir (DIRHANDLE, ".") || die "Cannot opendir . $!";
	foreach $name (sort readdir(DIRHANDLE)) {
		if ($iterations_only) { 
			if ( ($name =~ /^perfstat\.\d+\.txt/) && 
				(is_in_timerange ($name, $start_time, $end_time)) ) {
				push @filenames, $name;
				if ($debug) {
					printf ("Selected file: %s \n", $name);
				}
			}
	 		else {
	    		if ($debug) {
					printf ("Rejected file: %s \n", $name);
	    		}
	 		}
		} else {   
			if ($name =~ /$tag/) {
				push @filenames, $name;
			}
		}
	}
	closedir (DIRHANDLE);
}
$config_file = $filenames[0];
if ($iterations_only) {
	if ( -f "perfstat.0001.txt") {
		$config_file = "perfstat.0001.txt";
	}
}

@filers = `psf_filers.pl $filenames[0]`;
if ($debug) {
	printf ("Base filename : %s \n", $filenames[0]);
	foreach $filer (@filers) {
		printf ("    %s \n", $filer);
	}
}
chomp @filenames;
chomp @filers;
sort @filers;
$num_filers = scalar @filers;

$num_files = scalar @filenames;
# Get perfstat version
$version_line = `psf_versions.pl -f $filers[0] $config_file`;
@version_data = split (/,/, $version_line);
$perfstat_major = $version_data[_NA_VERS_PERFSTAT_MAJOR];

foreach $filer (@filers) {
	chomp $filer;
	if (($num_filers > 1) || ($perfstat_major > 5)) {
		$f_clause = " -f $filer";
	}
	else {  # Some perfstat files without multiple filers
			# do not have the filer name in the header lines.
			# So if the perfstat file is only for one filer, do not use
			# the -f options
		$f_clause = "";
	}
	if ($debug) {
		printf ("perfstat_major : %s  f_clause : %s \n", 
			$perfstat_major, $f_clause);
	}
	$psf_files = sprintf ("%s%s", $filer , "_psf_files");
	mkdir $psf_files;

   $outfile = catfile ($psf_files , "filer.txt");
   system ("echo $filer  > $outfile ");
   $outfile = catfile ($psf_files , "versions.txt");
   system ("psf_versions.pl $f_clause $config_file  > $outfile ");
   $outfile = catfile ($psf_files , "wafl.txt");
   system ("psf_wafl.pl -H  > $outfile ");
   $outfile = catfile ($psf_files , "wafl_susp.txt");
   system ("psf_wafl_susp.pl -H  > $outfile ");
   $outfile = catfile ($psf_files , "statit.txt");
   system ("psf_statit.pl -H -X  > $outfile ");
   $outfile = catfile ($psf_files , "statit_summ.txt");
   system ("psf_statit.pl -H -X  > $outfile ");
   $outfile = catfile ($psf_files , "sysstat.txt");
   system ("psf_sysstat.pl -H  > $outfile ");
   $outfile = catfile ($psf_files , "vol.txt");
   system ("psf_vol.pl -H -X  > $outfile ");
   $outfile = catfile ($psf_files , "vol_by_rg.txt");
   system ("psf_vol.pl -H -X  > $outfile ");
   $outfile = catfile ($psf_files , "vol_by_rg_tot.txt");
   system ("psf_vol.pl -H -X  > $outfile ");
   $outfile = catfile ($psf_files , "vol_summ.txt");
   system ("psf_vol.pl -H -X  > $outfile ");
   $outfile = catfile ($psf_files , "vol_info.txt");
   system ("psf_vol_info.pl $f_clause $config_file > $outfile");
   $vol_info_file {$filer} = $outfile;
   $outfile = catfile ($psf_files , "parity_disks.txt");
   system ("psf_parity_disks.pl $f_clause $config_file > $outfile");
   $parity_disk_file {$filer} = $outfile;
   $outfile = catfile ($psf_files , "lun_info.txt");
   system ("psf_lun_info.pl -H -D $f_clause $config_file > $outfile");
   $outfile = catfile ($psf_files , "lun.txt");
   system ("psf_lun.pl -H  > $outfile ");
   $outfile = catfile ($psf_files , "lun_detail.txt");
   system ("psf_lun.pl -H  > $outfile ");
   $outfile = catfile ($psf_files , "lun_summ.txt");
   system ("psf_lun.pl -H  > $outfile ");
   $outfile = catfile ($psf_files , "lun_iops_kbs.txt");
   system ("psf_lun_iops_kbs.pl -H  > $outfile ");
   $outfile = catfile ($psf_files , "ldisk.txt");
   system ("psf_ldisk.pl -H  > $outfile ");
   $outfile = catfile ($psf_files , "cp_times.txt");
   system ("psf_cp_times.pl -H  > $outfile ");
   $outfile = catfile ($psf_files , "cp_kinds.txt");
   system ("psf_cp_kinds.pl -H  > $outfile ");
   $outfile = catfile ($psf_files , "lun_hist.txt");
   system ("psf_hist.pl -H  > $outfile ");
   $outfile = catfile ($psf_files , "lun_hist_stats.txt");
   system ("psf_hist.pl -H -S > $outfile ");
   $outfile = catfile ($psf_files , "lun_rt_ge_16.txt");
   system ("psf_hist.pl -H -S > $outfile ");
   $outfile = catfile ($psf_files , "lun_rt_ge_128.txt");
   system ("psf_hist.pl -H -S > $outfile ");
   $outfile = catfile ($psf_files , "lun_rt_ge_1024.txt");
   system ("psf_hist.pl -H -S > $outfile ");
   $outfile = catfile ($psf_files , "nfsv3_hist.txt");
   system ("psf_hist.pl -H  > $outfile ");
   $outfile = catfile ($psf_files , "nfsv3_hist_stats.txt");
   system ("psf_hist.pl -H -S > $outfile ");
   $outfile = catfile ($psf_files , "cifs_hist.txt");
   system ("psf_hist.pl -H  > $outfile ");
   $outfile = catfile ($psf_files , "cifs_hist_stats.txt");
   system ("psf_hist.pl -H -S > $outfile ");
   $outfile = catfile ($psf_files , "wafl_q_time_hist.txt");
   system ("psf_hist.pl -H > $outfile ");
   $outfile = catfile ($psf_files , "wafl_q_time_hist_stats.txt");
   system ("psf_hist.pl -H -S > $outfile ");
   $outfile = catfile ($psf_files , "wafl_q_time_ge_16.txt");
   system ("psf_hist.pl -H -S > $outfile ");
}

$start_time = time();
$curr_file = 1;
foreach $file (@filenames) {
	chomp $file;
	foreach $filer (@filers) {
		chomp $filer;
		if (($num_filers > 1) || ($perfstat_major > 5)) {
			$f_clause = " -f $filer";
		}
		else {  # Some perfstat files without multiple filers
           	# do not have the filer name in the header lines.
           	# So if the perfstat file is only for one filer, do not use
           	# the -f options
         	$f_clause = "";
		}
		if ($debug) {
       		printf ("perfstat_major : %s  f_clause : %s \n", 
                	$perfstat_major, $f_clause);
		}

		$psf_files = sprintf ("%s%s", $filer , "_psf_files");
		$timestamp = `psf_filer_timestamp.pl $f_clause $file`;
		chomp $timestamp;
		$T_clause = " -T \"$timestamp\"";
 
	   $outfile = catfile ($psf_files , "wafl.txt");
	   fork_psf_proc ("psf_wafl.pl $f_clause $file >> $outfile") ;
	   $outfile = catfile ($psf_files , "wafl_susp.txt");
	   fork_psf_proc ("psf_wafl_susp.pl $f_clause $file >> $outfile") ;
	   $outfile = catfile ($psf_files , "statit.txt");
	   fork_psf_proc ("psf_statit.pl -X $f_clause $file >> $outfile ");
	   if ($debug) {
			printf ("doing psf_statit.pl on %s for filer %s \n", 
				$file, $f_clause);
	   }
	   $outfile = catfile ($psf_files , "statit_summ.txt");
	   system ("psf_statit.pl -S -X $f_clause $file >> $outfile ");
	   $outfile = catfile ($psf_files , "sysstat.txt");
	   fork_psf_proc ("psf_sysstat.pl $f_clause $file >> $outfile ");
	   # $vol_info = $vol_info_file {$filer};
	   if ($vol_info) {
			$vinfo_clause = "-i $vol_info";
	   } else {
			$vinfo_clause = "";
	   }
	   $parity_disks = $parity_disk_file {$filer};
	   if ($parity_disks) {
			$p_clause = "-p $parity_disks";
	   } else {
			$p_clause = "";
	   }
	   $outfile = catfile ($psf_files , "vol.txt");
	   system ("psf_vol.pl -X -t $f_clause $vinfo_clause $p_clause $file >> $outfile ");
	   $outfile = catfile ($psf_files , "vol_by_rg.txt");
	   fork_psf_proc ("psf_vol.pl -X -r -R $f_clause $vinfo_clause $p_clause $file >> $outfile ");
	   $outfile = catfile ($psf_files , "vol_by_rg_tot.txt");
	   fork_psf_proc ("psf_vol.pl -X -t -r -R $f_clause $vinfo_clause $p_clause $file >> $outfile ");
	   $outfile = catfile ($psf_files , "vol_summ.txt");
	   system ("psf_vol.pl -S -X -t $f_clause $vinfo_clause $p_clause $file >> $outfile ");
	   $outfile = catfile ($psf_files , "lun.txt");
	   fork_psf_proc ("psf_lun.pl $f_clause $file >> $outfile ");
	   $outfile = catfile ($psf_files , "lun_detail.txt");
	   fork_psf_proc ("psf_lun.pl -L $f_clause $file >> $outfile ");
	   $outfile = catfile ($psf_files , "lun_summ.txt");
	   system ("psf_lun.pl -S  $f_clause $file >> $outfile ");
	   $outfile = catfile ($psf_files , "lun_iops_kbs.txt");
	   fork_psf_proc ("psf_lun_iops_kbs.pl $f_clause $file >> $outfile ");
	   $outfile = catfile ($psf_files , "ldisk.txt");
	   fork_psf_proc ("psf_ldisk.pl -A $file >> $outfile ");
	   $outfile = catfile ($psf_files , "cp_times.txt");
	   fork_psf_proc ("psf_cp_times.pl $f_clause $file >> $outfile ");
	   $outfile = catfile ($psf_files , "cp_kinds.txt");
	   fork_psf_proc ("psf_cp_kinds.pl $f_clause $file >> $outfile ");
	   $outfile = catfile ($psf_files , "lun_hist.txt");
	   fork_psf_proc ("psf_hist.pl -N scsi $f_clause $T_clause $file >> $outfile ");
	   $outfile = catfile ($psf_files , "lun_hist_stats.txt");
	   fork_psf_proc ("psf_hist.pl -S -N scsi $f_clause $T_clause $file >> $outfile ");
           $outfile = catfile ($psf_files , "lun_rt_ge_16.txt");
	   fork_psf_proc ("psf_hist.pl -O 16 -N scsi $f_clause $T_clause $file >> $outfile ");
           $outfile = catfile ($psf_files , "lun_rt_ge_128.txt");
	   fork_psf_proc ("psf_hist.pl -O 128 -N scsi $f_clause $T_clause $file >> $outfile ");
           $outfile = catfile ($psf_files , "lun_rt_ge_1024.txt");
	   fork_psf_proc ("psf_hist.pl -O 1024 -N scsi $f_clause $T_clause $file >> $outfile ");
	   $outfile = catfile ($psf_files , "nfsv3_hist.txt");
	   fork_psf_proc ("psf_hist.pl -N v3 $f_clause $T_clause $file >> $outfile ");
	   $outfile = catfile ($psf_files , "nfsv3_hist_stats.txt");
	   fork_psf_proc ("psf_hist.pl -R -S -N v3 $f_clause $T_clause $file >> $outfile ");
	   $outfile = catfile ($psf_files , "cifs_hist.txt");
	   fork_psf_proc ("psf_hist.pl -N CIFS $f_clause $T_clause $file >> $outfile ");
	   $outfile = catfile ($psf_files , "cifs_hist_stats.txt");
	   fork_psf_proc ("psf_hist.pl -S -N CIFS $f_clause $T_clause $file >> $outfile ");
	   $outfile = catfile ($psf_files , "wafl_q_time_hist.txt");
	   fork_psf_proc ("psf_hist.pl -N WAFL $f_clause $T_clause $file >> $outfile ");
	   $outfile = catfile ($psf_files , "wafl_q_time_hist_stats.txt");
	   fork_psf_proc ("psf_hist.pl -S -N WAFL $f_clause $T_clause $file >> $outfile ");
	   $outfile = catfile ($psf_files , "wafl_q_time_ge_16.txt");
	   fork_psf_proc ("psf_hist.pl -O 16 -N WAFL $f_clause $T_clause $file >> $outfile ");

	   wait_for_forks();

           $outfile = catfile ($psf_files , "file_timestamp_map.txt");
           open(OUTPUTA, '>>'. $outfile) || die 'Cant open output ';
           $timestamp = `psf_filer_timestamp.pl $f_clause $file`;
           printf (OUTPUTA "%s, %s, \n", $file, $timestamp);
           close(OUTPUTA);
   }        
   $outfile = "psf_progress.txt";
   open(OUTPUTA, '>'. $outfile) || die 'Cant open output ';
   $curr_time = time();
   $elapsed_secs = $curr_time - $start_time;
   $percent_done = $curr_file / $num_files; 
   $remaining_secs = ($elapsed_secs / $percent_done) - $elapsed_secs; 
   printf (OUTPUTA "%d, %d, %.1f, %d, %d, ", 
		$num_files, $curr_file, 
		($percent_done * 100), 
		$elapsed_secs, $remaining_secs);
   close(OUTPUTA);
   $curr_file++;
}
foreach $filer (@filers) {
	chomp $filer;
	$psf_files = sprintf ("%s%s", $filer , "_psf_files");

	$infile = catfile ($psf_files , "wafl_susp.txt");
        $outfile = catfile ($psf_files , "wafl_susp_sort.txt");
        system ("sort $infile > $outfile ");
        $infile = catfile ($psf_files , "wafl_susp_sort.txt");
        $outfile = catfile ($psf_files , "wafl_ops.txt");
        fork_psf_proc ("psf_transpose.pl -C 3 $infile > $outfile ");

        $infile = catfile ($psf_files , "lun.txt");
        $outfile = catfile ($psf_files , "lun_sort.txt");
        system ("sort $infile > $outfile ");
        $infile = catfile ($psf_files , "lun_sort.txt");
        $outfile = catfile ($psf_files , "vol_lun_riops.txt");
        fork_psf_proc ("psf_transpose.pl -C 2 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_lun_wiops.txt");
        fork_psf_proc  ("psf_transpose.pl -C 3 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_lun_rkbs.txt");
        fork_psf_proc  ("psf_transpose.pl -C 5 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_lun_wkbs.txt");
        fork_psf_proc  ("psf_transpose.pl -C 6 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_lun_qlen.txt");
        fork_psf_proc  ("psf_transpose.pl -C 8 $infile > $outfile ");

# transpose lun_summ.txt

	$infile = catfile ($psf_files , "lun_summ.txt");
        $outfile = catfile ($psf_files , "lun_summ_sort.txt");
        system ("sort $infile > $outfile ");
        $infile = catfile ($psf_files , "lun_summ_sort.txt");
        $outfile = catfile ($psf_files , "lun_summ_riops.txt");
        fork_psf_proc ("psf_transpose.pl -C 2 $infile > $outfile ");
        $outfile = catfile ($psf_files , "lun_summ_wiops.txt");
        fork_psf_proc ("psf_transpose.pl -C 3 $infile > $outfile ");
        $outfile = catfile ($psf_files , "lun_summ_rkbs.txt");
        fork_psf_proc ("psf_transpose.pl -C 5 $infile > $outfile ");
        $outfile = catfile ($psf_files , "lun_summ_wkbs.txt");
        fork_psf_proc ("psf_transpose.pl -C 6 $infile > $outfile ");
        $outfile = catfile ($psf_files , "lun_summ_rt.txt");
        fork_psf_proc ("psf_transpose.pl -C 7 $infile > $outfile ");
        $outfile = catfile ($psf_files , "lun_summ_qlen.txt");
        fork_psf_proc ("psf_transpose.pl -C 8 $infile > $outfile ");

	wait_for_forks();

        # Compute rsize and wsize files
        $iops_file = catfile ($psf_files , "lun_summ_riops.txt");
        $kbs_file = catfile ($psf_files , "lun_summ_rkbs.txt");
        $outfile = catfile ($psf_files , "lun_summ_rsize.txt");
        fork_psf_proc ("psf_compute_xfer_sz.pl $iops_file $kbs_file > $outfile ");
        $iops_file = catfile ($psf_files , "lun_summ_wiops.txt");
        $kbs_file = catfile ($psf_files , "lun_summ_wkbs.txt");
        $outfile = catfile ($psf_files , "lun_summ_wsize.txt");
        fork_psf_proc ("psf_compute_xfer_sz.pl $iops_file $kbs_file > $outfile ");

# transpose lun_detail.txt

        $infile = catfile ($psf_files , "lun_detail.txt");
        $outfile = catfile ($psf_files , "lun_detail_sort.txt");
	system ("sort $infile > $outfile ");
	$infile = catfile ($psf_files , "lun_detail_sort.txt");
	$outfile = catfile ($psf_files , "lun_riops.txt"); 
	fork_psf_proc ("psf_transpose.pl -C 2 $infile > $outfile ");
	$outfile = catfile ($psf_files , "lun_wiops.txt"); 
	fork_psf_proc  ("psf_transpose.pl -C 3 $infile > $outfile ");
	$outfile = catfile ($psf_files , "lun_rkbs.txt"); 
	fork_psf_proc  ("psf_transpose.pl -C 5 $infile > $outfile ");
	$outfile = catfile ($psf_files , "lun_wkbs.txt"); 
	fork_psf_proc  ("psf_transpose.pl -C 6 $infile > $outfile ");
	$outfile = catfile ($psf_files , "lun_rt.txt"); 
	fork_psf_proc  ("psf_transpose.pl -C 7 $infile > $outfile ");
	$outfile = catfile ($psf_files , "lun_qlen.txt"); 
	fork_psf_proc  ("psf_transpose.pl -C 8 $infile > $outfile ");
	
	wait_for_forks();

        # Compute lun_rsize and lun_wsize files 
        $iops_file = catfile ($psf_files , "lun_riops.txt");
        $kbs_file = catfile ($psf_files , "lun_rkbs.txt");
        $outfile = catfile ($psf_files , "lun_rsize.txt");
        fork_psf_proc  ("psf_compute_xfer_sz.pl $iops_file $kbs_file > $outfile ");
        $iops_file = catfile ($psf_files , "lun_wiops.txt");
        $kbs_file = catfile ($psf_files , "lun_wkbs.txt");
        $outfile = catfile ($psf_files , "lun_wsize.txt");
        fork_psf_proc  ("psf_compute_xfer_sz.pl $iops_file $kbs_file > $outfile ");

# transpose vol_summ.txt

        $infile = catfile ($psf_files , "vol_summ.txt");
        $outfile = catfile ($psf_files , "vol_summ_sort.txt");
        system ("sort $infile > $outfile ");
        $infile = catfile ($psf_files , "vol_summ_sort.txt");
        $outfile = catfile ($psf_files , "vol_summ_util.txt");
        fork_psf_proc ("psf_transpose.pl -C 3 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_summ_iops.txt");
        fork_psf_proc ("psf_transpose.pl -C 4 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_summ_riops.txt");
        fork_psf_proc  ("psf_transpose.pl -C 5 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_summ_rchain.txt");
        fork_psf_proc  ("psf_transpose.pl -C 6 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_summ_rusecs.txt");
        fork_psf_proc  ("psf_transpose.pl -C 7 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_summ_wiops.txt");
        fork_psf_proc  ("psf_transpose.pl -C 8 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_summ_wchain.txt");
        fork_psf_proc  ("psf_transpose.pl -C 9 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_summ_wusecs.txt");
        fork_psf_proc  ("psf_transpose.pl -C 10 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_summ_cpriops.txt");
        fork_psf_proc  ("psf_transpose.pl -C 11 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_summ_cprchain.txt");
        fork_psf_proc  ("psf_transpose.pl -C 12 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_summ_cprusecs.txt");
        fork_psf_proc  ("psf_transpose.pl -C 13 $infile > $outfile ");

	wait_for_forks();

        # Compute Response Time files
        $chain_file = catfile ($psf_files , "vol_summ_rchain.txt");
        $usecs_file = catfile ($psf_files , "vol_summ_rusecs.txt");
        $outfile = catfile ($psf_files , "vol_summ_rmsecs.txt");
        fork_psf_proc ("psf_compute_vol_rt.pl $chain_file $usecs_file > $outfile ");
        $chain_file = catfile ($psf_files , "vol_summ_wchain.txt");
        $usecs_file = catfile ($psf_files , "vol_summ_wusecs.txt");
        $outfile = catfile ($psf_files , "vol_summ_wmsecs.txt");
        fork_psf_proc ("psf_compute_vol_rt.pl $chain_file $usecs_file > $outfile ");
        # Compute KBS files
        $chain_file = catfile ($psf_files , "vol_summ_rchain.txt");
        $iops_file = catfile ($psf_files , "vol_summ_riops.txt");
        $outfile = catfile ($psf_files , "vol_summ_rkbs.txt");
        fork_psf_proc ("psf_compute_vol_kbs.pl $chain_file $iops_file > $outfile ");
        $chain_file = catfile ($psf_files , "vol_summ_wchain.txt");
        $iops_file = catfile ($psf_files , "vol_summ_wiops.txt");
        $outfile = catfile ($psf_files , "vol_summ_wkbs.txt");
        fork_psf_proc ("psf_compute_vol_kbs.pl $chain_file $iops_file > $outfile ");
        $chain_file = catfile ($psf_files , "vol_summ_cprchain.txt");
        $iops_file = catfile ($psf_files , "vol_summ_cpriops.txt");
        $outfile = catfile ($psf_files , "vol_summ_cprkbs.txt");
        fork_psf_proc ("psf_compute_vol_kbs.pl $chain_file $iops_file > $outfile ");


# volume transpose files
	$infile = catfile ($psf_files , "vol.txt");
        $outfile = catfile ($psf_files , "vol_sort.txt");
        system ("sort $infile > $outfile ");
        $infile = catfile ($psf_files , "vol_sort.txt");
        $outfile = catfile ($psf_files , "vol_util.txt");
        fork_psf_proc  ("psf_transpose.pl -C 3 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_iops.txt");
        fork_psf_proc  ("psf_transpose.pl -C 4 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_riops.txt");
        fork_psf_proc  ("psf_transpose.pl -C 5 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_rchain.txt");
        fork_psf_proc  ("psf_transpose.pl -C 6 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_rusecs.txt");
        fork_psf_proc  ("psf_transpose.pl -C 7 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_wiops.txt");
        fork_psf_proc  ("psf_transpose.pl -C 8 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_wchain.txt");
        fork_psf_proc  ("psf_transpose.pl -C 9 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_wusecs.txt");
        fork_psf_proc  ("psf_transpose.pl -C 10 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_cpriops.txt");
        fork_psf_proc  ("psf_transpose.pl -C 11 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_cprchain.txt");
        fork_psf_proc  ("psf_transpose.pl -C 12 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_cprusecs.txt");
        fork_psf_proc  ("psf_transpose.pl -C 13 $infile > $outfile ");

	wait_for_forks();

	 # Compute Response Time files
        $chain_file = catfile ($psf_files , "vol_rchain.txt");
        $usecs_file = catfile ($psf_files , "vol_rusecs.txt");
        $outfile = catfile ($psf_files , "vol_rmsecs.txt");
        fork_psf_proc ("psf_compute_vol_rt.pl $chain_file $usecs_file > $outfile ");
        $chain_file = catfile ($psf_files , "vol_wchain.txt");
        $usecs_file = catfile ($psf_files , "vol_wusecs.txt");
        $outfile = catfile ($psf_files , "vol_wmsecs.txt");
        fork_psf_proc ("psf_compute_vol_rt.pl $chain_file $usecs_file > $outfile ");

        # Compute KBS files
        $chain_file = catfile ($psf_files , "vol_rchain.txt");
        $iops_file = catfile ($psf_files , "vol_riops.txt");
        $outfile = catfile ($psf_files , "vol_rkbs.txt");
        fork_psf_proc ("psf_compute_vol_kbs.pl $chain_file $iops_file > $outfile ");
        $chain_file = catfile ($psf_files , "vol_wchain.txt");
        $iops_file = catfile ($psf_files , "vol_wiops.txt");
        $outfile = catfile ($psf_files , "vol_wkbs.txt");
        fork_psf_proc ("psf_compute_vol_kbs.pl $chain_file $iops_file > $outfile ");
        $chain_file = catfile ($psf_files , "vol_cprchain.txt");
        $iops_file = catfile ($psf_files , "vol_cpriops.txt");
        $outfile = catfile ($psf_files , "vol_cprkbs.txt");
        fork_psf_proc ("psf_compute_vol_kbs.pl $chain_file $iops_file > $outfile ");

        # Now do volume raidgroup transpose files
        $infile = catfile ($psf_files , "vol_by_rg_tot.txt");
        $outfile = catfile ($psf_files , "vol_by_rg_tot_sort.txt");
        system ("sort $infile > $outfile ");
        $infile = catfile ($psf_files , "vol_by_rg_tot_sort.txt");
        $outfile = catfile ($psf_files , "rg_util.txt");
        fork_psf_proc  ("psf_transpose.pl -C 3 $infile > $outfile ");
        $outfile = catfile ($psf_files , "rg_iops.txt");
        fork_psf_proc  ("psf_transpose.pl -C 4 $infile > $outfile ");
        $outfile = catfile ($psf_files , "rg_riops.txt");
        fork_psf_proc  ("psf_transpose.pl -C 5 $infile > $outfile ");
        $outfile = catfile ($psf_files , "rg_rchain.txt");
        fork_psf_proc  ("psf_transpose.pl -C 6 $infile > $outfile ");
        $outfile = catfile ($psf_files , "rg_rusecs.txt");
        fork_psf_proc ("psf_transpose.pl -C 7 $infile > $outfile ");
        $outfile = catfile ($psf_files , "rg_wiops.txt");
        fork_psf_proc  ("psf_transpose.pl -C 8 $infile > $outfile ");
        $outfile = catfile ($psf_files , "rg_wchain.txt");
        fork_psf_proc  ("psf_transpose.pl -C 9 $infile > $outfile ");
        $outfile = catfile ($psf_files , "rg_wusecs.txt");
        fork_psf_proc  ("psf_transpose.pl -C 10 $infile > $outfile ");
        $outfile = catfile ($psf_files , "rg_cpriops.txt");
        fork_psf_proc  ("psf_transpose.pl -C 11 $infile > $outfile ");
        $outfile = catfile ($psf_files , "rg_cprchain.txt");
        fork_psf_proc  ("psf_transpose.pl -C 12 $infile > $outfile ");
        $outfile = catfile ($psf_files , "rg_cprusecs.txt");
        fork_psf_proc  ("psf_transpose.pl -C 13 $infile > $outfile ");

	# Now do volume raidgroup transpose files for per disk iops
        $infile = catfile ($psf_files , "vol_by_rg.txt");
        $outfile = catfile ($psf_files , "vol_by_rg_sort.txt");
        system ("sort $infile > $outfile ");
        $infile = catfile ($psf_files , "vol_by_rg_sort.txt");
        $outfile = catfile ($psf_files , "rg_iops_per_disk.txt");
        fork_psf_proc  ("psf_transpose.pl -C 4 $infile > $outfile ");

	wait_for_forks();
        
	# Compute Response Time files
        $chain_file = catfile ($psf_files , "rg_rchain.txt");
        $usecs_file = catfile ($psf_files , "rg_rusecs.txt");
        $outfile = catfile ($psf_files , "rg_rmsecs.txt");
        fork_psf_proc ("psf_compute_vol_rt.pl $chain_file $usecs_file > $outfile ");
        $chain_file = catfile ($psf_files , "rg_wchain.txt");
        $usecs_file = catfile ($psf_files , "rg_wusecs.txt");
        $outfile = catfile ($psf_files , "rg_wmsecs.txt");
        fork_psf_proc ("psf_compute_vol_rt.pl $chain_file $usecs_file > $outfile ");

        # Compute KBS files
        $chain_file = catfile ($psf_files , "rg_rchain.txt");
        $iops_file = catfile ($psf_files , "rg_riops.txt");
        $outfile = catfile ($psf_files , "rg_rkbs.txt");
        fork_psf_proc ("psf_compute_vol_kbs.pl $chain_file $iops_file > $outfile ");
        $chain_file = catfile ($psf_files , "rg_wchain.txt");
        $iops_file = catfile ($psf_files , "rg_wiops.txt");
        $outfile = catfile ($psf_files , "rg_wkbs.txt");
        fork_psf_proc ("psf_compute_vol_kbs.pl $chain_file $iops_file > $outfile ");
        $chain_file = catfile ($psf_files , "rg_cprchain.txt");
        $iops_file = catfile ($psf_files , "rg_cpriops.txt");
        $outfile = catfile ($psf_files , "rg_cprkbs.txt");
        fork_psf_proc ("psf_compute_vol_kbs.pl $chain_file $iops_file > $outfile ");


	# Check the size of the sysstat.txt file, splitting it
	# if it is too large for Excel. 
	$outfile = catfile ($psf_files , "sysstat.txt");
	$sysstat_lines = `psf_count_lines.pl $outfile`;
	if ($sysstat_lines > 32766) {
		$outbase =  catfile ($psf_files , "sysstat");
		system ("psf_split_sysstat.pl $outfile $outbase");
		$new_sysstat_name = catfile ($psf_files , "sysstat_too_big.txt");
		rename $outfile, $new_sysstat_name;
	}
	wait_for_forks();
}

sub fork_psf_proc {
	local ($command_str) = @_;

	if ($NumForked >= $MaxToFork) {
		wait();  # For one to finish 
		$NumForked--;
	}
	$pid = fork();
	if ( !$pid ) {
		system ($command_str);
		exit;
	}
	$NumForked++;
}

sub wait_for_forks {
        while ($NumForked) {
              wait();
              $NumForked--;
        }
}

sub get_num_iterations {
	local ($file) = @_;
	my $Line, @line, $iterations, $len;

	$iterations = 1;
	open(LOCINPUTA, '<' . $file) || die 'Unable to open perfstat file';
	while ($Line = <LOCINPUTA>) {
		if ($Line =~ /^=-=-=-=-=-/) {
			last;
		}
		elsif ($Line =~ /^ITERATIONS/) {
			$Line =~ s/\r//g;     # Remove DOS carraige returns
			@line = split (/\s+/, $Line);	
			$iterations = $line[2];
			# Remove quotes 
			$len = length $iterations;
			if ($iterations =~ /\"/) {
				$iterations = substr ($iterations, 1, $len-1);
            			$len--;
			}
			if ($iterations =~ /\"/) {
            			$iterations = substr ($iterations, 0, $len-1);
			}
			if ($debug) { printf ("iterations: %d \n", $iterations); }
			last;
		}
		else {
			next;
		}
	}
	return $iterations;
}

sub is_in_timerange {
	local ($file, $start_time, $end_time) = @_;

	if (($start_time eq "") && ($end_time eq "")) {
		return (1);
	} else {
		my $lower_bound_comparison = 0;
		my $upper_bound_comparison = 0;
		my $timestamp = `psf_filer_timestamp.pl $file`;
		if ($start_time) {
			$lower_bound_comparison = `psf_compare_timestamps.pl '$timestamp' '$start_time'`;
		}
		if ($end_time) {
			$upper_bound_comparison = `psf_compare_timestamps.pl '$timestamp' '$end_time'`;
		}
		if ( ($lower_bound_comparison == -1) ||
		     ($upper_bound_comparison == 1) ) {
			return (0);
		} else {
			return (1);
		}
	}
}

sub usage {
   print <<EOM;
psf_get_all_data.pl [-I] [-S <filename>]  [<perfstat-filename-substr>]

Processes a set of perfstat files (or a single multi-iteration perfstat file) for 
use by the Perfstat Grapher. The result of executing this script is a set of 
directories having names of the form <filer-name>_psf_files, with each 
such directory containing the input files to the Perfstat Grapher for that 
filer. 

   Arguments and switches -
       -I              Only process files with names of the form :
                       "perfstat.NNNN.txt" 
       -S <filename>   Treats <filename> as a multi-iteration perfstat file, 
                       splitting out the iterations into separate files named 
                       "perfstat.NNNN.txt" where "NNNN" is the iteration number.
                       After the iterations have been split out they are 
                       processed. 
       <perfstat-filename-substr> 
                       If supplied, all files in the current working directory 
                       matching the regular expression *<perfstat-filename-substr>*
                       are processed. By default this pattern matching string is 
                       "perfstat".
   Normal Usage - 
	
	psf_get_all_data.pl -S <multi-iteration-perfstat-filename> 
	psf_graph.pl <filername>_psf_files
EOM
exit;

}

