#!/usr/bin/env perl 

use File::Spec::Functions;
use Getopt::Std;
getopts("dsS:");

if ($opt_d) {
   $debug = 1;
} else {
   $debug = 0;
}

if ($opt_S) {
   $symbol = $opt_S;
} 

if ($opt_s) {
   $ps_s_clause = "-s ";
} 

$selected_output = $ARGV[0];
$tag = $ARGV[1];
  
if (!$tag) {
   $tag = "perfstat";
}
opendir (DIRHANDLE, ".") || die "Cannot opendir . $!";
foreach $name (sort readdir(DIRHANDLE)) {
   if ($name =~ /$tag/) {
      push @filenames, $name;
   }
}
closedir (DIRHANDLE);

@filers = `psf_filers.pl $filenames[0]`;
chomp @filenames;
chomp @filers;
$num_filers = scalar @filers;
foreach $filer (@filers) {
   chomp $filer;
   if ($num_filers > 1) {
      $f_clause = " -f $filer";
   }
   else {  # Some perfstat files without multiple filers
           # do not have the filer name in the header lines.
           # So if the perfstat file is only for one filer, do not use
           # the -f options
      $f_clause = "";
   }
   $psf_files = sprintf ("%s%s", $filer , "_psf_files");
   mkdir $psf_files;
   
   $vinfo_file =  catfile ($psf_files , "vol_info.txt");
   if ( -f $vinfo_file ) { 
	$vinfo_clause = "-i $vinfo_file";
   } else {
        $vinfo_clause = "";
   }

   $outfile = catfile ($psf_files , "$selected_output");
   if ($selected_output =~ /^filer\.txt/) {
      system ("echo $filer  > $outfile "); }
   elsif ($selected_output =~ /^versions\.txt/) {
      system ("psf_versions.pl $f_clause $filenames[0]  > $outfile "); }
   elsif ($selected_output =~ /^wafl\.txt/) {
      system ("psf_wafl.pl -H  > $outfile "); }
   elsif ($selected_output =~ /^wafl_susp\.txt/) {
      system ("psf_wafl_susp.pl -H  > $outfile "); }
   elsif ($selected_output =~ /^statit\.txt/) {
      system ("psf_statit.pl -H -X  > $outfile "); }
   elsif ($selected_output =~ /^statit_first\.txt/) {
      system ("psf_statit.pl -H -X  > $outfile "); }
   elsif ($selected_output =~ /^statit_summ\.txt/) {
      system ("psf_statit.pl -H -X  > $outfile "); }
   elsif ($selected_output =~ /^sysstat\.txt/) {
      system ("psf_sysstat.pl -H  > $outfile "); }
   elsif ($selected_output =~ /^vol\.txt/) {
      system ("psf_vol.pl -H -X  > $outfile "); }
   elsif ($selected_output =~ /^vol_by_rg\.txt/) {
      system ("psf_vol.pl -H -X  > $outfile "); }
   elsif ($selected_output =~ /^vol_by_rg_tot\.txt/) {
      system ("psf_vol.pl -H -X  > $outfile "); }
   elsif ($selected_output =~ /^vol_summ\.txt/) {
      system ("psf_vol.pl -H -X  > $outfile "); }
   elsif ($selected_output =~ /^lun_info\.txt/) {
      system ("psf_lun_info.pl -H -D $f_clause $filenames[0] > $outfile"); }
   elsif ($selected_output =~ /^lun\.txt/) {
      system ("psf_lun.pl -H  > $outfile "); }
   elsif ($selected_output =~ /^lun_detail\.txt/) {
      system ("psf_lun.pl -H  > $outfile "); }
   elsif ($selected_output =~ /^lun_summ\.txt/) {
      system ("psf_lun.pl -H  > $outfile "); }
   elsif ($selected_output =~ /^lun_iops_kbs\.txt/) {
      system ("psf_lun_iops_kbs.pl -H  > $outfile "); }
   elsif ($selected_output =~ /^ldisk\.txt/) {
      system ("psf_ldisk.pl -H  > $outfile "); }
   elsif ($selected_output =~ /^cp_times\.txt/) {
      system ("psf_cp_times.pl -H  > $outfile "); }
   elsif ($selected_output =~ /^cp_kinds\.txt/) {
      system ("psf_cp_kinds.pl -H  > $outfile "); }
   elsif ($selected_output =~ /^lun_hist\.txt/) {
      system ("psf_hist.pl -H  > $outfile "); }
   elsif ($selected_output =~ /^lun_hist_stats\.txt/) {
      system ("psf_hist.pl -H -S > $outfile "); }
   elsif ($selected_output =~ /^lun_rt_ge_16\.txt/) {
      system ("psf_hist.pl -H -S > $outfile "); }
   elsif ($selected_output =~ /^lun_rt_ge_128\.txt/) {
      system ("psf_hist.pl -H -S > $outfile "); }
   elsif ($selected_output =~ /^lun_rt_ge_1024\.txt/) {
      system ("psf_hist.pl -H -S > $outfile "); }
   elsif ($selected_output =~ /^nfsv3_hist\.txt/) {
      system ("psf_hist.pl -H  > $outfile "); }
   elsif ($selected_output =~ /^nfsv3_hist_stats\.txt/) {
      system ("psf_hist.pl -H -S > $outfile "); }
   elsif ($selected_output =~ /^cifs_hist\.txt/) {
      system ("psf_hist.pl -H  > $outfile "); }
   elsif ($selected_output =~ /^cifs_hist_stats\.txt/) {
      system ("psf_hist.pl -H -S > $outfile "); }
   elsif ($selected_output =~ /^wafl_q_time_hist\.txt/) {
      system ("psf_hist.pl -H  > $outfile "); }
   elsif ($selected_output =~ /^wafl_q_time_hist_stats\.txt/) {
      system ("psf_hist.pl -H -S > $outfile "); }
   elsif ($selected_output =~ /^wafl_q_time_ge_16\.txt/) {
      system ("psf_hist.pl -H -S > $outfile "); }
   elsif ($selected_output =~ /^mem\.txt/) {
      system ("psf_mem.pl -H -S $symbol > $outfile "); }
   elsif ($selected_output =~ /^ps\.txt/) {
      system ("psf_ps.pl -H $ps_s_clause > $outfile "); }
   elsif ($selected_output =~ /^recycle_q\.txt/) {
      system ("psf_recycle_q.pl -H $ps_s_clause > $outfile "); }
   elsif ($selected_output =~ /^sol_iostat\.txt/) {
      system ("psf_aggr_solaris_iostat.pl -H > $outfile "); }

   foreach $file (@filenames) {
           chomp $file;
      if ($selected_output =~ /^wafl\.txt/) {
	   system ("psf_wafl.pl $f_clause $file >> $outfile") ; }
      elsif ($selected_output =~ /^wafl_susp\.txt/) {
	   system ("psf_wafl_susp.pl $f_clause $file >> $outfile") ; }
      elsif ($selected_output =~ /^statit\.txt/) {
	   system ("psf_statit.pl -X $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^statit_first\.txt/) {
	   system ("psf_statit.pl -X -1 $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^statit_summ\.txt/) {
	   system ("psf_statit.pl -S -X $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^sysstat\.txt/) {
	   system ("psf_sysstat.pl $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^vol\.txt/) {
	   system ("psf_vol.pl -X -t $f_clause $vinfo_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^vol_by_rg\.txt/) {
	   system ("psf_vol.pl -X -r -R $f_clause $vinfo_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^vol_by_rg_tot\.txt/) {
	   system ("psf_vol.pl -X -t -r -R $f_clause $vinfo_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^vol_summ\.txt/) {
	   system ("psf_vol.pl -S -X -t $f_clause $vinfo_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^lun\.txt/) {
	   system ("psf_lun.pl $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^lun_detail\.txt/) {
	   system ("psf_lun.pl -A -L $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^lun_summ\.txt/) {
	   system ("psf_lun.pl -S  $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^lun_iops_kbs\.txt/) {
	   system ("psf_lun_iops_kbs.pl $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^ldisk\.txt/) {
	   system ("psf_ldisk.pl -A $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^cp_times\.txt/) {
	   system ("psf_cp_times.pl $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^cp_kinds\.txt/) {
	   system ("psf_cp_kinds.pl $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^lun_hist\.txt/) {
	   system ("psf_hist.pl -N scsi $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^lun_hist_stats\.txt/) {
	   system ("psf_hist.pl -S -N scsi $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^lun_rt_ge_16\.txt/) {
	   system ("psf_hist.pl -O 16 -N scsi $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^lun_rt_ge_128\.txt/) {
	   system ("psf_hist.pl -O 128 -N scsi $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^lun_rt_ge_1024\.txt/) {
	   system ("psf_hist.pl -O 1024 -N scsi $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^nfsv3_hist\.txt/) {
	   system ("psf_hist.pl -N v3 $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^nfsv3_hist_stats\.txt/) {
	   system ("psf_hist.pl -R -S -N v3 $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^cifs_hist\.txt/) {
	   system ("psf_hist.pl -N CIFS $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^cifs_hist_stats\.txt/) {
	   system ("psf_hist.pl -S -N CIFS $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^wafl_q_time_hist\.txt/) {
	   system ("psf_hist.pl -N WAFL $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^wafl_q_time_hist_stats\.txt/) {
	   system ("psf_hist.pl -S -N WAFL $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^wafl_q_time_ge_16\.txt/) {
	   system ("psf_hist.pl -O 16 -N WAFL $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^mem\.txt/) {
	   system ("psf_mem.pl -S $symbol $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^ps\.txt/) {
	   system ("psf_ps.pl $ps_s_clause $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^recycle_q\.txt/) {
	   system ("psf_recycle_q.pl $f_clause $file >> $outfile "); }
      elsif ($selected_output =~ /^sol_iostat\.txt/) {
	   if ($symbol) { 
		$str_opt = "-S" . $symbol;
	   } else {
		$str_opt = "";
	   }
	   system ("psf_aggr_solaris_iostat.pl $str_opt $f_clause $file >> $outfile "); }
   }
}
if ($selected_output =~ /^lun_summ.txt/) {
   foreach $filer (@filers) {
        chomp $filer;
        $psf_files = sprintf ("%s%s", $filer , "_psf_files");
        $infile = catfile ($psf_files , "lun_summ.txt");
        $outfile = catfile ($psf_files , "lun_summ_sort.txt");
        system ("sort $infile > $outfile ");
        $infile = catfile ($psf_files , "lun_summ_sort.txt");
        $outfile = catfile ($psf_files , "lun_summ_riops.txt");
        system ("psf_transpose.pl -C 2 $infile > $outfile ");
        $outfile = catfile ($psf_files , "lun_summ_wiops.txt");
        system ("psf_transpose.pl -C 3 $infile > $outfile ");
        $outfile = catfile ($psf_files , "lun_summ_rkbs.txt");
        system ("psf_transpose.pl -C 5 $infile > $outfile ");
        $outfile = catfile ($psf_files , "lun_summ_wkbs.txt");
        system ("psf_transpose.pl -C 6 $infile > $outfile ");
        $outfile = catfile ($psf_files , "lun_summ_rt.txt");
        system ("psf_transpose.pl -C 7 $infile > $outfile ");
        $outfile = catfile ($psf_files , "lun_summ_qlen.txt");
        system ("psf_transpose.pl -C 8 $infile > $outfile ");
        # Compute rsize and wsize files
        $iops_file = catfile ($psf_files , "lun_summ_riops.txt");
        $kbs_file = catfile ($psf_files , "lun_summ_rkbs.txt");
        $outfile = catfile ($psf_files , "lun_summ_rsize.txt");
        system ("psf_compute_xfer_sz.pl $iops_file $kbs_file > $outfile ");
        $iops_file = catfile ($psf_files , "lun_summ_wiops.txt");
        $kbs_file = catfile ($psf_files , "lun_summ_wkbs.txt");
        $outfile = catfile ($psf_files , "lun_summ_wsize.txt");
        system ("psf_compute_xfer_sz.pl $iops_file $kbs_file > $outfile ");
   }
}

if ($selected_output =~ /^lun_detail.txt/) {
   foreach $filer (@filers) {
        chomp $filer;
        $psf_files = sprintf ("%s%s", $filer , "_psf_files");
        $infile = catfile ($psf_files , "lun_detail.txt");
        $outfile = catfile ($psf_files , "lun_detail_sort.txt");
        system ("sort $infile > $outfile ");
        $infile = catfile ($psf_files , "lun_detail_sort.txt");
        $outfile = catfile ($psf_files , "lun_riops.txt");
        system ("psf_transpose.pl -C 2 $infile > $outfile ");
        $outfile = catfile ($psf_files , "lun_wiops.txt");
        system ("psf_transpose.pl -C 3 $infile > $outfile ");
        $outfile = catfile ($psf_files , "lun_rkbs.txt");
        system ("psf_transpose.pl -C 5 $infile > $outfile ");
        $outfile = catfile ($psf_files , "lun_wkbs.txt");
        system ("psf_transpose.pl -C 6 $infile > $outfile ");
        $outfile = catfile ($psf_files , "lun_rt.txt");
        system ("psf_transpose.pl -C 7 $infile > $outfile ");
        $outfile = catfile ($psf_files , "lun_qlen.txt");
        system ("psf_transpose.pl -C 8 $infile > $outfile ");
	# Compute rsize and wsize files 
	$iops_file = catfile ($psf_files , "lun_riops.txt");
	$kbs_file = catfile ($psf_files , "lun_rkbs.txt");
	$outfile = catfile ($psf_files , "lun_rsize.txt");
	system ("psf_compute_xfer_sz.pl $iops_file $kbs_file > $outfile ");
	$iops_file = catfile ($psf_files , "lun_wiops.txt");
	$kbs_file = catfile ($psf_files , "lun_wkbs.txt");
	$outfile = catfile ($psf_files , "lun_wsize.txt");
	system ("psf_compute_xfer_sz.pl $iops_file $kbs_file > $outfile ");
   }
}
if ($selected_output =~ /^vol_summ.txt/) {
   foreach $filer (@filers) {
        chomp $filer;
        $psf_files = sprintf ("%s%s", $filer , "_psf_files");
        $infile = catfile ($psf_files , "vol_summ.txt");
        $outfile = catfile ($psf_files , "vol_summ_sort.txt");
        system ("sort $infile > $outfile ");
        $infile = catfile ($psf_files , "vol_summ_sort.txt");
        $outfile = catfile ($psf_files , "vol_summ_util.txt");
        system ("psf_transpose.pl -C 3 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_summ_iops.txt");
        system ("psf_transpose.pl -C 4 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_summ_riops.txt");
        system ("psf_transpose.pl -C 5 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_summ_rchain.txt");
        system ("psf_transpose.pl -C 6 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_summ_rusecs.txt");
        system ("psf_transpose.pl -C 7 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_summ_wiops.txt");
        system ("psf_transpose.pl -C 8 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_summ_wchain.txt");
        system ("psf_transpose.pl -C 9 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_summ_wusecs.txt");
        system ("psf_transpose.pl -C 10 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_summ_cpriops.txt");
        system ("psf_transpose.pl -C 11 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_summ_cprchain.txt");
        system ("psf_transpose.pl -C 12 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_summ_cprusecs.txt");
        system ("psf_transpose.pl -C 13 $infile > $outfile ");
        # Compute Response Time files
        $chain_file = catfile ($psf_files , "vol_summ_rchain.txt");
        $usecs_file = catfile ($psf_files , "vol_summ_rusecs.txt");
        $outfile = catfile ($psf_files , "vol_summ_rmsecs.txt");
        system ("psf_compute_vol_rt.pl $chain_file $usecs_file > $outfile ");
        $chain_file = catfile ($psf_files , "vol_summ_wchain.txt");
        $usecs_file = catfile ($psf_files , "vol_summ_wusecs.txt");
        $outfile = catfile ($psf_files , "vol_summ_wmsecs.txt");
        system ("psf_compute_vol_rt.pl $chain_file $usecs_file > $outfile ");
        # Compute KBS files
        $chain_file = catfile ($psf_files , "vol_summ_rchain.txt");
        $iops_file = catfile ($psf_files , "vol_summ_riops.txt");
        $outfile = catfile ($psf_files , "vol_summ_rkbs.txt");
        system ("psf_compute_vol_kbs.pl $chain_file $iops_file > $outfile ");
        $chain_file = catfile ($psf_files , "vol_summ_wchain.txt");
        $iops_file = catfile ($psf_files , "vol_summ_wiops.txt");
        $outfile = catfile ($psf_files , "vol_summ_wkbs.txt");
        system ("psf_compute_vol_kbs.pl $chain_file $iops_file > $outfile ");
        $chain_file = catfile ($psf_files , "vol_summ_cprchain.txt");
        $iops_file = catfile ($psf_files , "vol_summ_cpriops.txt");
        $outfile = catfile ($psf_files , "vol_summ_cprkbs.txt");
        system ("psf_compute_vol_kbs.pl $chain_file $iops_file > $outfile ");

   }
}

if ($selected_output =~ /^statit.txt/) {
   foreach $filer (@filers) {
        chomp $filer;
        $psf_files = sprintf ("%s%s", $filer , "_psf_files");
        $infile = catfile ($psf_files , "statit.txt");
        $outfile = catfile ($psf_files , "statit_sort.txt");
        system ("sort $infile > $outfile ");
	$ren_file = catfile ($psf_files , "statit_nosort.txt");
	system ("mv $infile $ren_file");
	system ("mv $outfile $infile");
   }
}

if ($selected_output =~ /^statit_summ.txt/) {
   foreach $filer (@filers) {
        chomp $filer;
        $psf_files = sprintf ("%s%s", $filer , "_psf_files");
        $infile = catfile ($psf_files , "statit_summ.txt");
        $outfile = catfile ($psf_files , "statit_summ_sort.txt");
        system ("sort $infile > $outfile ");
        $ren_file = catfile ($psf_files , "statit_summ_nosort.txt");
        system ("mv $infile $ren_file");
        system ("mv $outfile $infile");
   }
}

if ($selected_output =~ /^wafl.txt/) {
   foreach $filer (@filers) {
        chomp $filer;
        $psf_files = sprintf ("%s%s", $filer , "_psf_files");
        $infile = catfile ($psf_files , "wafl.txt");
        $outfile = catfile ($psf_files , "wafl_sort.txt");
        system ("sort $infile > $outfile ");
        $ren_file = catfile ($psf_files , "wafl_nosort.txt");
        system ("mv $infile $ren_file");
        system ("mv $outfile $infile");
   }
}

if ($selected_output =~ /^vol.txt/) {
   foreach $filer (@filers) {
        chomp $filer;
        $psf_files = sprintf ("%s%s", $filer , "_psf_files");
        $infile = catfile ($psf_files , "vol.txt");
        $outfile = catfile ($psf_files , "vol_sort.txt");
        system ("sort $infile > $outfile ");
        $infile = catfile ($psf_files , "vol_sort.txt");
        $outfile = catfile ($psf_files , "vol_util.txt");
        system ("psf_transpose.pl -C 3 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_iops.txt");
        system ("psf_transpose.pl -C 4 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_riops.txt");
        system ("psf_transpose.pl -C 5 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_rchain.txt");
        system ("psf_transpose.pl -C 6 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_rusecs.txt");
        system ("psf_transpose.pl -C 7 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_wiops.txt");
        system ("psf_transpose.pl -C 8 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_wchain.txt");
        system ("psf_transpose.pl -C 9 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_wusecs.txt");
        system ("psf_transpose.pl -C 10 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_cpriops.txt");
        system ("psf_transpose.pl -C 11 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_cprchain.txt");
        system ("psf_transpose.pl -C 12 $infile > $outfile ");
        $outfile = catfile ($psf_files , "vol_cprusecs.txt");
        system ("psf_transpose.pl -C 13 $infile > $outfile ");
        # Compute Response Time files 
        $chain_file = catfile ($psf_files , "vol_rchain.txt");
        $usecs_file = catfile ($psf_files , "vol_rusecs.txt");
        $outfile = catfile ($psf_files , "vol_rmsecs.txt");
        system ("psf_compute_vol_rt.pl $chain_file $usecs_file > $outfile ");
        $chain_file = catfile ($psf_files , "vol_wchain.txt");
        $usecs_file = catfile ($psf_files , "vol_wusecs.txt");
        $outfile = catfile ($psf_files , "vol_wmsecs.txt");
        system ("psf_compute_vol_rt.pl $chain_file $usecs_file > $outfile ");
        # Compute KBS files 
        $chain_file = catfile ($psf_files , "vol_rchain.txt");
        $iops_file = catfile ($psf_files , "vol_riops.txt");
        $outfile = catfile ($psf_files , "vol_rkbs.txt");
        system ("psf_compute_vol_kbs.pl $chain_file $iops_file > $outfile ");
        $chain_file = catfile ($psf_files , "vol_wchain.txt");
        $iops_file = catfile ($psf_files , "vol_wiops.txt");
        $outfile = catfile ($psf_files , "vol_wkbs.txt");
        system ("psf_compute_vol_kbs.pl $chain_file $iops_file > $outfile ");
        $chain_file = catfile ($psf_files , "vol_cprchain.txt");
        $iops_file = catfile ($psf_files , "vol_cpriops.txt");
        $outfile = catfile ($psf_files , "vol_cprkbs.txt");
        system ("psf_compute_vol_kbs.pl $chain_file $iops_file > $outfile ");

   }
}
if ($selected_output =~ /^sysstat.txt/) {
   foreach $filer (@filers) {
        chomp $filer;
        $psf_files = sprintf ("%s%s", $filer , "_psf_files");

	# check the size of the sysstat.txt file, splitting it
        # if it is too large for Excel.
        $outfile = catfile ($psf_files , "sysstat.txt");
        $sysstat_lines = `psf_count_lines.pl $outfile`;
        if ($sysstat_lines > 32766) {
                $outbase =  catfile ($psf_files , "sysstat");
                system ("psf_split_sysstat.pl $outfile $outbase");
                $new_sysstat_name = catfile ($psf_files , "sysstat_too_big.txt");
                rename $outfile, $new_sysstat_name;
        }
   }
}
