#!/usr/bin/env perl

# This script is provided "as is" it is
# not warranted but may be supported on a limited basis
# by Network Appliance.
#
# psf_hist.pl
#
#   PerfStatFilter for any histograms in perfstat files. 
#
#   v0.04  01-26-2005 : Added Column for -O switch which is percentage of ops over.  
#   v0.03  11-09-2004 : Added -R if you want op rates. The graphs don't 
#                       use this column and that is the only reason we need
#                       to call psf_versions.pl Performance optimization. 
#			Also added -T <timestamp> so we don't have to 
#			call psf_filer_timestamp.pl 
#   v0.02  03-23-2004 : Added -f <filer> for multi-filer perfstat files.
#                       Only tested against perfstat 5.20 Linux version.
#
#   v0.01  03-17-2004 : Initial implementation.
#

# EXPORTED CONSTANTS (Not really, you have to copy them)

use constant _NA_TIMESTAMP                 => 0;
use constant _NA_HIST_OP                   => 1;
use constant _NA_HIST_STATS_COUNT          => 2;
use constant _NA_HIST_STATS_MEAN           => 3;
use constant _NA_HIST_STATS_MIN            => 4;
use constant _NA_HIST_STATS_MAX            => 5;
use constant _NA_HIST_STATS_MODE           => 6;
use constant _NA_HIST_STATS_STD_DEV        => 7;
use constant _NA_HIST_STATS_90_PERCENTILE  => 8;
use constant _NA_HIST_STATS_95_PERCENTILE  => 9;

use constant _NA_HIST_COUNTS_1             => 2;
use constant _NA_HIST_COUNTS_2             => 3;
use constant _NA_HIST_COUNTS_3             => 4;
use constant _NA_HIST_COUNTS_4             => 5;
use constant _NA_HIST_COUNTS_5             => 6;
use constant _NA_HIST_COUNTS_6             => 7;
use constant _NA_HIST_COUNTS_7             => 8;
use constant _NA_HIST_COUNTS_8             => 9;
use constant _NA_HIST_COUNTS_16            => 10;
use constant _NA_HIST_COUNTS_24            => 11;
use constant _NA_HIST_COUNTS_32            => 12;
use constant _NA_HIST_COUNTS_40            => 13;
use constant _NA_HIST_COUNTS_48            => 14;
use constant _NA_HIST_COUNTS_56            => 15;
use constant _NA_HIST_COUNTS_64            => 16;
use constant _NA_HIST_COUNTS_128           => 17;
use constant _NA_HIST_COUNTS_192           => 18;
use constant _NA_HIST_COUNTS_256           => 19;
use constant _NA_HIST_COUNTS_320           => 20;
use constant _NA_HIST_COUNTS_384           => 21;
use constant _NA_HIST_COUNTS_448           => 22;
use constant _NA_HIST_COUNTS_512           => 23;
use constant _NA_HIST_COUNTS_1024          => 24;
use constant _NA_HIST_COUNTS_1536          => 25;
use constant _NA_HIST_COUNTS_2048          => 26;
use constant _NA_HIST_COUNTS_2560          => 27;
use constant _NA_HIST_COUNTS_3072          => 28;
use constant _NA_HIST_COUNTS_3584          => 29;
use constant _NA_HIST_COUNTS_4K            => 30;
use constant _NA_HIST_COUNTS_8K            => 31;
use constant _NA_HIST_COUNTS_12K           => 32;
use constant _NA_HIST_COUNTS_16K           => 33;
use constant _NA_HIST_COUNTS_20K           => 34;
use constant _NA_HIST_COUNTS_24K           => 35;
use constant _NA_HIST_COUNTS_28K           => 36;
use constant _NA_HIST_COUNTS_32K           => 37;
use constant _NA_HIST_COUNTS_64K           => 38;
use constant _NA_HIST_COUNTS_96K           => 39;
use constant _NA_HIST_COUNTS_128K          => 40;
use constant _NA_HIST_COUNTS_160K          => 41;
use constant _NA_HIST_COUNTS_192K          => 42;
use constant _NA_HIST_COUNTS_224K          => 43;
use constant _NA_HIST_COUNTS_256K          => 44;

# Imported
use constant    _NA_VERS_PERFSTAT_MAJOR => 0;
use constant    _NA_VERS_PERFSTAT_MINOR => 1;
use constant    _NA_VERS_ONTAP_VERSION  => 2;
use constant    _NA_VERS_ONTAP_MAJOR    => 3;
use constant    _NA_VERS_ONTAP_MINOR    => 4;
use constant    _NA_VERS_FILER_MODEL    => 5;
use constant    _NA_VERS_FILER_MAGIC1   => 6;
use constant    _NA_VERS_FILER_MAGIC2   => 7;
use constant    _NA_VERS_PERFSTAT_PLATFORM => 8;
use constant    _NA_VERS_NUM_SAMPLES    => 9;
use constant    _NA_VERS_SECS_PER_SAMPLE => 10;
use constant    _NA_VERS_SECS_PER_ITERATION => 11;

$inread = 0;
$inwrite = 0;
$readcnt = 0;
$writecnt = 0;
$index = 0;

use Getopt::Std;
getopts("N:dDf:hHnSPO:RT:");

if ($opt_h) { usage(); }

if ($opt_N) {
   $name_template = $opt_N ;
}

if ($opt_d) {
   $debug = 1;
} else {
   $debug = 0;
}

if ($opt_D) {
   $data_plus_header = 1;
} else {
   $data_plus_header = 0;
}

if ($opt_f) {
   $filer = $opt_f;
}

if ($opt_H) {
   $header = 1;
} else {
   $header = 0;
}

if ($opt_n) {
   $nice_output = 1;
} else {
   $nice_output = 0;
}

if ($opt_R) {
   $rates_wanted = 1;
} else {
   $rates_wanted = 0;
}

if ($opt_S) {
   $statistics  = 1;
} else {
   $statistics = 0;
}

if ($opt_T) {
   $timestamp  = $opt_T;
} 

if ($opt_P) {
   $percentiles = 1;
} else {
   $percentiles= 0;
}

if ($opt_O) {
   $over = $opt_O;
} else {
   $over = 0;
}

@bucket_max;
@bucket_start = (0,16,128,1024,8192,65536);
$index = 1;
for ( $j = 0 ; $j <= 5; $j++ ) {
   for ( $i = 0 ; $i <= 7; $i++ ) {
      if ( $bucket_start[$j] == 0 ) {
         $bucket_max[$index] = $i + 1;  # Add 1 because the first line 
                                         # which has labels "0 1 2 3 4 5 6 7" really
                                         # should read "<1 <2 <3 ... <8"
      }
      else {
         if ( $i == 7 ) { last };        # For these lines the last entry is unused
         $bucket_max[$index] = $bucket_start[$j] +
                                ($i * ($bucket_start[$j] / 2) );
      }
      if ($debug) {
         printf ("%d, %d %d %d %d \n",
                     $j, $i, $index, $bucket_start[$j],
                     $bucket_max[$index]);
      }
      $index++;
   }                            
}
$num_buckets = $index - 1;

# Compute bucket midpoints 
$bucket_min = 0;
for ($j = 1 ; $j < $num_buckets; $j++ ) {
   $bucket_midpoint[$j] = $bucket_min + 
                           (( $bucket_max[$j] - $bucket_min ) / 2);
   $bucket_min = $bucket_max[$j];         # For next iteration
}

# Print header
if ( $header ) {
   printf ("Timestamp          , %25s, ", "op");
   if ( $statistics ) {
      printf ("Count, Mean,  Min,  Max, Mode, StdDev, 90%tile, 95%tile \n"); 
   }
   elsif ( $percentiles ) {
      printf ("50th, 60th, 70th, 80th, 90th, 95th, 99th, 100th, \n");
   }
   elsif ( $over ) {
      printf ("Total OP Count, Over this msecs, Count, Percent Over, \n");
   }
   else {   
      for ( $j = 1; $j <= $num_buckets; $j++ ) {
         if ( $bucket_max[$j] < 4096 ) {
            printf ("%d, ", $bucket_max[$j]);
         }
         else {
            printf ("%dK, ", $bucket_max[$j] / 1024 );
         }
      }
      printf ("\n");
   }
   if (!$data_plus_header) {
     exit;
   }
}

# ops 
%op_name_index = ("wafl restart count", 0,
            "scsi_inquiry", 0,
            "scsi_read_extended", 0,
            "scsi_write_extended", 0,
            "scsi_get_vdisk_attr", 0,
            "scsi_test_unit_ready", 0,
            "scsi_read", 0,
            "scsi_write", 0,
            "scsi_read_capacity", 0,
            "scsi_report_luns", 0,
            "non-WAFL msg queue time", 0,
            "WAFL msg queue time", 0,
            "v3 getattr", 0,
            "v3 setattr", 0,
            "v3 lookup", 0,
            "v3 access", 0,
            "v3 readlink", 0,
            "v3 read", 0,
            "v3 write", 0,
            "v3 create", 0,
            "v3 mkdir", 0,
            "v3 symlink", 0,
            "v3 remove", 0,
            "v3 rmdir", 0,
            "v3 rename", 0,
            "v3 link", 0,
            "v3 readdir", 0,
            "v3 readdirplus", 0,
            "v3 fsstat", 0,
            "v3 fsinfo", 0,
            "v3 pathconf", 0,
            "CIFS request processing time", 0,
            "Time spent by oplock breaks being delayed", 0,
            "Time spent on write raw queue", 0 );


$perfstat_file = $ARGV[0];
open(INPUTA, '<' . $perfstat_file) || die 'Unable to open perfstat file (arg1) .';

if ($rates_wanted) {
	$version_line = `psf_versions.pl $perfstat_file`;
	@versions = split (/,/, $version_line);
	$seconds = $versions[_NA_VERS_SECS_PER_ITERATION] * 60; 
}

if (!$timestamp) {
   if ( $filer ) {
      $timestamp = `psf_filer_timestamp.pl -f $filer $perfstat_file`;
   }
   else {
      $timestamp = `psf_filer_timestamp.pl $perfstat_file`;
   }
} 
chop $timestamp;
   


# Get to POSTSTATs
while ($Line = <INPUTA>) {
   if ($Line =~ /POSTSTAT/ ) {  
      last ;
   }
}

$found_group = 0;
$hist_num = 0;
$done = 0;
while ($Line = <INPUTA>) {
   if ($done) {
      last;
   }
   if ( ( $Line =~ /^=-=-/ ) &&
        ( $Line =~ /POSTSTATS/ ) ) {
      @line = split (/\s+/, $Line);
      $curr_filer = $line[2];
      if ( $debug ) {
         printf ("Current statit filer : %s \n", $curr_filer);
      }
   }

   if ( !($Line =~ /millisecond units/ ) ) { # skip lines not starting with alphanumeric
      next;
   }
   if ( $debug ) {
      print $Line;
   }
   $found_op = 0;
   if ( $name_template ) {
      if ( !($Line =~ /$name_template/) ) {
         if ( $debug ) {
               print "Skipping due to name mismatch";
         }
         next; 
      }
      elsif ($found_group &&
             ($Line =~ /POSTSTAT/)) {
         $done = 1;
         last;      # This test is an optimization to avoid having to read the 
                    # entire file if we specified a name_template and we have 
                    # found it. It assumes only one sample per perfstat file. 
      }
   }
   if ( ($filer) &&
        !($filer =~ /^$curr_filer/) ) {
      if ( $debug ) {
            print "Skipping due to filer mismatch";
      }
      next;
   }

   if ( $debug ) {
      print "Looking through ops ";
   }
   $found_op = 0;
   foreach $op (keys(%op_name_index)) { 
      if ( $debug ) {
         printf ("Comparing %s with %s \n",  $Line, $op);
      }
      if ( ($Line =~ /^$op:/)  ) {
         if ( !$op_name_index {$op} ) {
            # Not a duplicate because fo FCP & iSCSI running 
            $hist_num++;
            $op_name_index {$op} = $hist_num;
            $op_name[$hist_num] = $op;
            if ( $debug ) { 
               printf ("Found hist op : %s %d \n", $op, $hist_num);
            }
            $found_op = 1;
            $found_group = 1; 
            last;
         }
      }
      next;
   }
   if ( !$found_op ) {
      next;
   }
   $index = 1;
   $Line = <INPUTA>;
   $Line = <INPUTA>;
   if ($Line =~ /(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/){
      if ( $debug ) { print $Line; }
      @line = split (/\s+/, $Line);
      for ( $i = 1 ; $i <= 8; $i++ ) {
           $cnt[$hist_num][$index] = $line[$i];
           $index++;
      }
      for ( $grp = 1; $grp <= 5; $grp++ ) { 
         $Line = <INPUTA>;       # labels
         $Line = <INPUTA>;       # data    
         if ($Line =~ /(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/){
            @line = split (/\s+/, $Line);
            for ( $i = 1 ; $i <= 7; $i++ ) {
               $cnt[$hist_num][$index] = $line[$i];
               $index++;
            }
         }
         if ( $debug ) { print $Line; }
      }
   }
}

# Compute statistics : Mean, Mode, Min, Max, StdDev, 90th%ile, 95th%ile 
for ( $i = 1; $i <= $hist_num ; $i++ ) {
   $total_count[$i] = 0;
   $total_msecs[$i] = 0;
   $mode_population[$i] = 0;
   $min_bucket[$i] = 0;
   $max_bucket[$i] = 1;
   $percentile_50[$i] = 0;
   $percentile_60[$i] = 0;
   $percentile_70[$i] = 0;
   $percentile_80[$i] = 0;
   $percentile_90[$i] = 0;
   $percentile_95[$i] = 0;
   $percentile_99[$i] = 0;
   for ( $j = 1; $j < $num_buckets; $j++ ) {
       $total_count[$i] += $cnt[$i][$j];
       $total_msecs[$i] += $cnt[$i][$j] * $bucket_midpoint[$j];
       if ( $cnt[$i][$j] > $mode_population[$i] ) {
          $mode_population[$i] = $cnt[$i][$j];
          $mode_bucket[$i] = $j;
       }
       if ( !$min_bucket[$i] && $cnt[$i][$j] ) {
          $min_bucket[$i] = $j;
       }
       if ( $cnt[$i][$j] > $cnt[$i][$max_bucket] ) { 
          $max_bucket[$i] = $j;
       }
   }
   if ( $total_count[$i] ) {
      $mean[$i] = $total_msecs[$i] / $total_count[$i];
   }

# Compute standard deviation
   $summed_squared_deviation = 0;
   for ( $j = 1; $j < $num_buckets; $j++ ) {
       $deviation = ( $bucket_midpoint[$j] - $mean[$i] ) ** 2;
       $summed_squared_deviation += $deviation * $cnt[$i][$j];
   }
   if ( $total_count[$i] ) {
      $mean_squared_deviation = $summed_squared_deviation / 
                             $total_count[$i] ;
   }
   $std_dev[$i] = sqrt ($mean_squared_deviation);

# Do the percentiles now that we have the total counts 
   if ( $total_count[$i] ) {
       $tmp_count = 0;
       for ( $j = 1; $j < $num_buckets; $j++ ) {
           $tmp_count += $cnt[$i][$j];
           if ( ($tmp_count / $total_count[$i]) > .5 ) {
               $percentile_50_bucket[$i] = $j + 1;
               last;
           }
       }

       $tmp_count = 0;
       for ( $j = 1; $j < $num_buckets; $j++ ) {
           $tmp_count += $cnt[$i][$j];
           if ( ($tmp_count / $total_count[$i]) > .6 ) {
               $percentile_60_bucket[$i] = $j + 1;
               last;
           }
       }

       $tmp_count = 0;
       for ( $j = 1; $j < $num_buckets; $j++ ) {
           $tmp_count += $cnt[$i][$j];
           if ( ($tmp_count / $total_count[$i]) > .7 ) {
               $percentile_70_bucket[$i] = $j + 1;
               last;
           }
       }

       $tmp_count = 0;
       for ( $j = 1; $j < $num_buckets; $j++ ) {
           $tmp_count += $cnt[$i][$j];
           if ( ($tmp_count / $total_count[$i]) > .8 ) {
               $percentile_80_bucket[$i] = $j + 1;
               last;
           }
       }

       $tmp_count = 0;
       for ( $j = 1; $j < $num_buckets; $j++ ) {
           $tmp_count += $cnt[$i][$j]; 
           if ( ($tmp_count / $total_count[$i]) > .9 ) { 
               $percentile_90_bucket[$i] = $j + 1; 
               last;
           }
       }
       $tmp_count = 0;
       for ( $j = 1; $j < $num_buckets; $j++ ) {
           $tmp_count += $cnt[$i][$j]; 
           if ( ($tmp_count / $total_count[$i]) > .95 ) { 
               $percentile_95_bucket[$i] = $j + 1; 
               last;
           }
       }
       $tmp_count = 0;
       for ( $j = 1; $j < $num_buckets; $j++ ) {
           $tmp_count += $cnt[$i][$j];
           if ( ($tmp_count / $total_count[$i]) > .99 ) {
               $percentile_99_bucket[$i] = $j + 1;
               last;
           }
       }
       if ( $over ) { 
           # Find the bucket to start counting
          $tmp_count = 0;
          for ( $j = 1; $j < $num_buckets; $j++ ) {
              if ( $bucket_max[$j] >= $over ) {
                 $tmp_count += $cnt[$i][$j];
              }
          }
          $over_count = $tmp_count;
       }


   }
   else {
       $percentile_90_bucket[$i] = 0;
       $percentile_95_bucket[$i] = 0;
   }

# Print  
   if ( $nice_output ) {
       printf ("%s \n", $op_name[$i] ); 
       if ( $statistics ) {
          printf ("   Count           : %d \n", $total_count[$i]);
          printf ("   Mean            : %.1f \n", $mean[$i]);
          printf ("   Min             : %.1f \n", 
                     $bucket_max[$min_bucket[$i]-1] );
          printf ("   Max             : %.1f \n", 
                     $bucket_max[$max_bucket[$i]] );
          printf ("   Mode            : %.1f \n", 
                     $bucket_midpoint[$mode_bucket[$i]] );
          printf ("   Std Deviation   : %.1f \n", 
                     $std_dev[$i] );
          printf ("   50th percentile : %.1f \n", 
                     $bucket_max[$percentile_50_bucket[$i]] );
          printf ("   60th percentile : %.1f \n", 
                     $bucket_max[$percentile_60_bucket[$i]] );
          printf ("   70th percentile : %.1f \n", 
                     $bucket_max[$percentile_70_bucket[$i]] );
          printf ("   80th percentile : %.1f \n", 
                     $bucket_max[$percentile_80_bucket[$i]] );
          printf ("   90th percentile : %.1f \n", 
                     $bucket_max[$percentile_90_bucket[$i]] );
          printf ("   95th percentile : %.1f \n", 
                     $bucket_max[$percentile_95_bucket[$i]] );
          printf ("   99th percentile : %.1f \n", 
                     $bucket_max[$percentile_99_bucket[$i]] );
       }
       else {
          for ( $j = 1; $j <= $num_buckets; $j++ ) {
             if ( $bucket_max[$j] < 4096 ) {
                printf ("%4d  : \t %d \n", 
                      $bucket_max[$j], 
                      $cnt[$i][$j] ); 
             }
             else {
                printf ("%4dK : \t %d \n",
                      $bucket_max[$j] / 1024, 
                      $cnt[$i][$j] ); 
             }
          }
      }
   }
   else {
       printf ("%s, %s, ", $timestamp, $op_name[$i]);
       if ( $statistics ) {
          if ($seconds > 0 ) {
             $op_rate = $total_count[$i] / $seconds ;
          } else {
             $op_rate = 0;
          }
          printf ("%5d, %4.1f, %4.1f, %4.1f, %4.1f, %6.1f, %4.1f   , %4.1f, %.1f, ",
             $total_count[$i],
             $mean[$i], 
             $bucket_max[$min_bucket[$i]-1],
             $bucket_max[$max_bucket[$i]],
             $bucket_midpoint[$mode_bucket[$i]],
             $std_dev[$i],
             $bucket_max[$percentile_90_bucket[$i]],
             $bucket_max[$percentile_95_bucket[$i]],
             $op_rate );
       }
       elsif ($percentiles) {
          printf ("%.1f, %.1f, %.1f, %.1f, %.1f, %.1f, %.1f, %.1f,  \n",
                   $bucket_max[$percentile_50_bucket[$i]], 
                   $bucket_max[$percentile_60_bucket[$i]], 
                   $bucket_max[$percentile_70_bucket[$i]], 
                   $bucket_max[$percentile_80_bucket[$i]], 
                   $bucket_max[$percentile_90_bucket[$i]],  
                   $bucket_max[$percentile_95_bucket[$i]], 
                   $bucket_max[$percentile_99_bucket[$i]],
                   $bucket_max[$max_bucket[$i]] );
       }
       elsif ($over) {
          printf ("%d, %d, %d, %.2f, \n",
                   $total_count[$i], $over, $over_count, $over_count/$total_count[$i]);
       }
       else {
          for ( $j = 1; $j <= $num_buckets; $j++ ) {
              printf ("%d, ", $cnt[$i][$j]);
          }
       }
       printf ("\n");
   }
}

sub usage {
   print <<EOM;
psf_hist.pl [-dDHhnS] [-f <filer>] [-N <name-template>] <perfstat filename>

Extracts and optionally summarizes response time histogram data.   

    Arguments and switches -
       -d          Print debug messages.
       -D          Print data as well as a header. Only relevent if -H specified.
       -f <filer>  For multi-filer perfstats choose this one.
       -H          Print a header line only unless -D specified.
       -h          This message.
       -n          Print Nice output, a value per line labeled with what it is.
       -N <name-template> Match name template, e.g. scsi or v3
       -S          Print statistics rather than counts. Statistics include : 
                     Count, Mean, Min, Max, StdDev, 90th percentile, 95th percentile
EOM
exit;

}

