#!/usr/bin/env perl

# This script is provided "as is" it is
# not warranted but may be supported on a limited basis
# by Network Appliance.
#
# psf_ldisk.pl 
# 
#   PerfStatFilter for Windows 200x NTFS Logical Disk stats.
#
#   v0.03  12-09-2003 : Changed from specificying #!/usr/local/bin to       
#                        #!/usr/bin/env perl . This even seemed to work on 
#                        my Win2K system. 
#   v0.02  11-21-2003 : Modified to work with output of perfstat version 2.1 
#                       and above only. Previous versions of perfstat usually 
#                       had Logicial DIsk data in the wrong columns. 
#   v0.01  11-04-2003 : Initial implementation. 
#

# EXPORTED CONSTANTS (Not really, you have to copy them)

use constant    _NA_TIMESTAMP           => 0;    # Timestamp 
use constant    _NA_LDISK_NAME          => 1;    # Disk Name (i.e. Drive Letter) 
use constant    _NA_LDISK_RIOPS         => 2;    # Avg Read IOs Per Second
use constant    _NA_LDISK_WIOPS         => 3;    # Avg Write IOs Per Second
use constant    _NA_LDISK_RSZ           => 4;    # Avg Read Size (KB)
use constant    _NA_LDISK_WSZ           => 5;    # Avg Write Size (KB)
use constant    _NA_LDISK_QLEN          => 6;    # Avg Queue Length
use constant    _NA_LDISK_RQLEN         => 7;    # Avg Read Queue Length 
use constant    _NA_LDISK_WQLEN         => 8;    # Avg Write Queue Length 
use constant    _NA_LDISK_RRT           => 9;    # Avg Read Response Time (msecs) 
use constant    _NA_LDISK_WRT           => 10;   # Avg Write Response Time (msecs)
use constant    _NA_LDISK_STRT_OFFSET   => 11;   # Logical Disk Starting Offset 
use constant    _NA_LUNID               => 12;   # Logical DIsk LUN ID 

use constant    _NA_VERS_PERFSTAT_MAJOR => 0;
use constant    _NA_VERS_PERFSTAT_MINOR => 1;
use constant    _NA_VERS_ONTAP_VERSION  => 2;
use constant    _NA_VERS_ONTAP_MAJOR    => 3;
use constant    _NA_VERS_ONTAP_MINOR    => 4;
use constant    _NA_VERS_FILER_MODEL    => 5;
use constant    _NA_VERS_FILER_MAGIC1   => 6;
use constant    _NA_VERS_FILER_MAGIC2   => 7;
use constant    _NA_VERS_PERFSTAT_PLATFORM => 8;

use Getopt::Std;
getopts("Af:L:ShHntdD");

if ($opt_h) { usage(); }

if ($opt_A) {
   $AllDrives = 1;
} else {
   $AllDrives = 0;
}

if ($opt_S) {
   $avg_all_samples = 1;
} else {
   $avg_all_samples = 0;
}

if ($opt_d) {
   $debug = 1;  
} else {
   $debug = 0;
}

if ($opt_D) {
   $data_plus_header = 1;  
} else {
   $data_plus_header = 0;  
}

if ($opt_H) {
   $header = 1;  
} else {
   $header = 0;
}

if ($opt_L) {
   $drive_letter= $opt_L;  
}

if ($opt_n) {
   $nice_output = 1;  
} else {
   $nice_output = 0;
}

if ($opt_t) {
   $show_totals = 1;  
} else {
   $show_totals = 0;
}

if ( !$nice_output && $header ) {
   $header_line =  "Timestamp, Drive, Rd/s, Wr/s, RdSzKB, WrSzKB, Qlen, Rqlen, Wqlen, ";
   printf ("%s", $header_line);
   $header_line =  "Rdmsecs, Wrmsecs, StrtOff, Lun, \n";
   printf ("%s" , $header_line);
   if (!$data_plus_header) {
     exit;
   }
}

$drive_id = 0;
$next_drive_id = 0;
$samples=0;

%drive_name_index;

$perfstat_file = $ARGV[0];
$version_line = `psf_versions.pl $perfstat_file`;
@versions = split (/,/, $version_line);
$perfstat_major = $versions[_NA_VERS_PERFSTAT_MAJOR];
$perfstat_minor = $versions[_NA_VERS_PERFSTAT_MINOR];
$ontap_vers = $versions[_NA_VERS_ONTAP_VERSION];
$model = $versions[_NA_VERS_FILER_MODEL];
if ( $debug ) { print @versions; }

open(INPUTA, '<' . $perfstat_file) || die 'Unable to open perfstat file (arg1) .';
     
while ($Line = <INPUTA>) {
   if ($Line =~ /ComponentsStorage/ ) {
      while ($Line = <INPUTA>) {
        if ($Line =~ /HOST CONFIG/ ) {
          last;
        }
        else {
          next;
        }
      }
      last;
   }
   elsif ($Line =~ /POSTSTATS/ ) {
      last;
   }
   else {
      next;
   }
}

# Get Windows disk information
if ($drive_letter) {
           $next_drive_id++;
           $drive_id = $next_drive_id;
           $drive_name_index {$drive_letter} = $drive_id;
           $drive_starting_offset[$drive_id] = $tmp_starting_offset;
           $drive_name[$drive_id] = $drive_letter;
           # Initialize counts
           $avg_qlen[$drive_id]   = 0;
           $avg_Rqlen[$drive_id]  = 0;
           $avg_Wqlen[$drive_id]  = 0;
           $avg_Rmsecs[$drive_id] = 0;
           $avg_Wmsecs[$drive_id] = 0;
           $avg_Riops[$drive_id]  = 0;
           $avg_Wiops[$drive_id]  = 0;
           $avg_Rsize[$drive_id]  = 0;
           $avg_Wsize[$drive_id]  = 0;
}
elsif (!AllDrives) {
  while ($Line = <INPUTA>) {
   if ($Line =~ /^Drive\s+\w+:/ ) {
     @line = split (/\s+/, $Line);
     $drive_letter = $line[1];
     if ( $debug ) { printf ("found drive %s \n", $drive_letter); }
     $Line = <INPUTA>;
     if ($Line =~ /Local Fixed Disk/ ) { # Look for Starting Offset
        $skip_drive = 0;
        while ($Line = <INPUTA>) {
           if ($Line =~ /Starting Offset/ ) { 
               @line = split (/\s+/, $Line);
               $tmp_starting_offset = $line[2];
               if ( $debug ) { printf ("Starting Offset  %d \n", $tmp_starting_offset); }
               last;
           }
           elsif ($Line =~ /^\s+/)  {
               $skip_drive = 1;
               last;
           }
           else {
               next;
           }
        }
        if ($skip_drive) {
           $skip_drive = 0;
           next;
        }
        $Line = <INPUTA>;                 # This line is Drive Description
        $Line = <INPUTA>;                 # This line is Drive Manufacturer 
        $Line = <INPUTA>;                 # This line is Drive Model       
        if ( $Line =~ /NETAPP\s+LUN/ ) {    # One of ours
           if ( $debug ) { printf ("found netapp lun %s \n", $drive_letter); }
           $next_drive_id++;
           $drive_id = $next_drive_id;
           $drive_name_index {$drive_letter} = $drive_id;
           $drive_starting_offset[$drive_id] = $tmp_starting_offset;
           $drive_name[$drive_id] = $drive_letter;
           # Initialize counts
           $avg_qlen[$drive_id]   = 0;
           $avg_Rqlen[$drive_id]  = 0;
           $avg_Wqlen[$drive_id]  = 0;
           $avg_Rmsecs[$drive_id] = 0;
           $avg_Wmsecs[$drive_id] = 0;
           $avg_Riops[$drive_id]  = 0;
           $avg_Wiops[$drive_id]  = 0;
           $avg_Rsize[$drive_id]  = 0;
           $avg_Wsize[$drive_id]  = 0;
           $Line = <INPUTA>;                 # This line is Bytes per Sector
           $Line = <INPUTA>;                 # This line is Media Loaded    
           $Line = <INPUTA>;                 # This line is Media type
           $Line = <INPUTA>;                 # This line is Partitions 
           $Line = <INPUTA>;                 # This line is SCSIBus    
           $Line = <INPUTA>;                 # This line is SCSILogicalUnit 
           @line = split (/\s+/, $Line);
           $lun[$drive_id] = $line[2];
        }
        elsif (length ($Line) < 2 )  {
               last;
        }
        elsif ($Line =~ /^\s+/)  {
               last;
        }
     }
     elsif ($Line =~ /HOST CONFIG/ ) {
        last;
     }
     else {
        if ( $debug ) { printf ("Not a Fixed Disk %s \n",  $Line); }
        next;
     }
   }
   elsif ($Line =~ /HOST CONFIG/ ) {
     last;
   }
   elsif ($Line =~ /\[SCSI\]/ ) {
     last;
   }
   else {
     next;
   }
   # next;
  }
}
        
while ($Line = <INPUTA>) {
   if  ($Line =~ /ldisk_perfmon/ ) {
           $Line = <INPUTA>;                    # This line has the date
           @line = split (/ +/, $Line);
           $date = $line[1];
           if ($perfstat_major >= 5) {
              $time = $line[2];
              $ampm = "NONE";
           }
           else {
              $Line = <INPUTA>;                    # This line has the time
              @line = split (/ +/, $Line);
              if ( length($line[0]) ) {
                 $time = $line[0];
                 $ampm = $line[1];
              }
              else {
                 $time = $line[1];
                 $ampm = $line[2];
              }
              if ( length($ampm) == 0 ) {
                 chop $time;
                 chop $time;
                 $ampm = chop $time;
                 if ( $ampm =~ /p/ ) {
                    $ampm = "PM";
                 }
                 else {
                    $ampm = "AM";
                 }
              }
              if ($debug) { printf ("AM/PM     : %s \n", $ampm); }
           }
           @timeparts = split (/:/, $time);
           $hour = @timeparts[0];
           $min  = @timeparts[1];
           if ( ( $ampm =~ /PM/ ) &&
                ( $hour < 12 ) )  {
              # Add 12 hours to time
              $hour += 12;
           }
           elsif ( ( $ampm =~ /AM/ ) &&
                   ( $hour == 12 ) ) {
              $hour = 0;
           }
           $time = sprintf ("%02d:%02d:%02d", $hour, $min, 0);

           if ($debug) { printf ("Base Time : %s \n", $time); }
           $base_timestamp = sprintf ("%s %s", $date, $time);
           if ($debug) { printf ("Base timestamp : %s \n", $base_timestamp ); }
           $timestamp = $base_timestamp;
           $interval = 10;
        # Now get to the stats 
        while ($Line = <INPUTA>) { 
           if ($Line =~ /: / ) {
                    chop ($Line);
                    @line = split (/ +/, $Line);
                    $drive_letter = $line[1];
                    $drive_id =  $drive_name_index {$drive_letter} ;
                    if ($debug) { printf ("Found sample for drive %s ID %d \n ", $drive_letter, $drive_id); }

                    if ((!$drive_id) && 
                        ($AllDrives)) {
                       $next_drive_id++;
                       $drive_id = $next_drive_id;
                       $drive_name_index {$drive_letter} = $drive_id;
                       $drive_starting_offset[$drive_id] = $tmp_starting_offset;
                       $drive_name[$drive_id] = $drive_letter;
                    }

                    if ( $drive_id ) { 
                       $qlen   = $line[6];
                       $Rqlen  = $line[7];
                       $Wqlen  = $line[8];
                       $Rmsecs = $line[10] * 1000;
                       $Wmsecs = $line[11] * 1000;
                       $Riops  = $line[13];
                       $Wiops  = $line[14];
                       $Rsize  = $line[19];
                       $Wsize  = $line[20];
                       if ( $avg_all_samples ) { 
                          $avg_qlen[$drive_id]   += $qlen;
                          $avg_Rqlen[$drive_id]  += $Rqlen;
                          $avg_Wqlen[$drive_id]  += $Wqlen;
                          $avg_Rmsecs[$drive_id] += $Rmsecs; 
                          $avg_Wmsecs[$drive_id] += $Wmsecs;
                          $avg_Riops[$drive_id]  += $Riops;
                          $avg_Wiops[$drive_id]  += $Wiops;
                          $avg_Rsize[$drive_id]  += $Rsize;
                          $avg_Wsize[$drive_id]  += $Wsize;
                       }
                       else {    # print the line 
                          $avg_qlen[$drive_id]   = $qlen;
                          $avg_Rqlen[$drive_id]  = $Rqlen;
                          $avg_Wqlen[$drive_id]  = $Wqlen;
                          $avg_Rmsecs[$drive_id] = $Rmsecs;
                          $avg_Wmsecs[$drive_id] = $Wmsecs;
                          $avg_Riops[$drive_id]  = $Riops;
                          $avg_Wiops[$drive_id]  = $Wiops;
                          $avg_Rsize[$drive_id]  = $Rsize;
                          $avg_Wsize[$drive_id]  = $Wsize;
                          print_drive_stats ($drive_id);
                       }
                    }
                    else {
                       next;
                    }
           }
           elsif ($Line =~ /Total/ ) { #End of sample  
                 $timestamp = bump_timestamp ($timestamp, $interval);
                 $samples++; 
           }
           elsif ($Line =~ /^=-=-/ ) {
              last;
           }
           else {
              next;
           }
        }
   }
   else {
        next;
   }
   if ($Line =~ /^=-=-/ ) {
      last;
   }
}

if ( $avg_all_samples ) { 
   foreach $drive (keys %drive_name_index) {
      $drive_id = $drive_name_index{$drive};
      if ( $debug ) {
          printf ("drive name : %s  drive_id : %d \n", $drive, $drive_id );
      }
      average_drive_stats ( $drive_id );
      if ( !$show_totals ) {
         print_drive_stats ($drive_id);
      }
   }
   if ( $show_totals ) {
      aggregate_all_drives ();
      $drive_name[0] = "All_NETAPP_Drives";
      print_drive_stats (0);
   }
}

# END OF MAIN 

sub print_drive_stats {
   local ($drive_id) = @_ ;

   if ( $nice_output ) {
             printf ("Summary drive statistics for drive %s \n", $drive_name[$drive_id]);
             printf ("   Avg Read  IOPS         : \t %.0f \n", $avg_Riops[$drive_id]);
             printf ("   Avg Write IOPS         : \t %.0f \n", $avg_Wiops[$drive_id]);
             printf ("   Avg Read  size (KB)    : \t %.0f \n", $avg_Rsize[$drive_id]);
             printf ("   Avg Write size (KB)    : \t %.0f \n", $avg_Wsize[$drive_id]);
             printf ("   Avg Queue Length       : \t %.0f \n", $avg_qlen[$drive_id]);
             printf ("   Avg Read Queue Length  : \t %.0f \n", $avg_Rqlen[$drive_id]);
             printf ("   Avg Write Queue Length : \t %.0f \n", $avg_Wqlen[$drive_id]);
             printf ("   Avg Read RT (ms)       : \t %.1f \n", $avg_Rmsecs[$drive_id]);
             printf ("   Avg Write RT (ms)      : \t %.1f \n", $avg_Wmsecs[$drive_id]);
             printf ("   Starting Offset        : \t %d \n",   $drive_starting_offset[$drive_id]);
             printf ("   LUN ID                 : \t %d \n",   $lun[$drive_id]);
   }
   else {
             printf ("%s, %s, %.0f, %.0f, ",
                    $timestamp,
                    $drive_name[$drive_id], 
                    $avg_Riops[$drive_id], $avg_Wiops[$drive_id] ); 
             printf ("%.0f, %.0f, ",
                    $avg_Rsize[$drive_id], $avg_Wsize[$drive_id] );
             printf ("%.0f, ", $avg_qlen[$drive_id] );
             printf ("%.0f, %.0f, ",
                    $avg_Rqlen[$drive_id], $avg_Wqlen[$drive_id] );
             printf ("%.1f, %.1f, ",
                    $avg_Rmsecs[$drive_id],
                    $avg_Wmsecs[$drive_id] );
             printf ("%d, %d, ",
                    $drive_starting_offset[$drive_id],
                    $lun[$drive_id] );
             printf ("\n");
   }
}


sub average_drive_stats {
   local ($drive_id) = @_ ;

 if ($samples) { 
   $avg_qlen[$drive_id]   /= $samples; 
   $avg_Rqlen[$drive_id]  /= $samples;
   $avg_Wqlen[$drive_id]  /= $samples;
   $avg_Rmsecs[$drive_id]  /= $samples;
   $avg_Wmsecs[$drive_id]  /= $samples;
   $avg_Riops[$drive_id]  /= $samples;
   $avg_Wiops[$drive_id]  /= $samples;
   $avg_Rsize[$drive_id]  /= $samples;
   $avg_Wsize[$drive_id]  /= $samples;
   # Use Little's Law over the averages to smooth out
   # variations including response times = 0. 
#   if ($avg_Riops[$drive_id]) {
#      $avg_Rmsecs[$drive_id] = ($avg_Rqlen[$drive_id] / $avg_Riops[$drive_id]) * 1000 ;
#   }
#   if ($avg_Wiops[$drive_id]) {
#      $avg_Wmsecs[$drive_id] = ($avg_Wqlen[$drive_id] / $avg_Wiops[$drive_id]) * 1000 ;
#   }
 }
}

sub aggregate_all_drives {

   $avg_qlen[0]   = 0;
   $avg_Rqlen[0]  = 0;
   $avg_Wqlen[0]  = 0;
   $avg_Rmsecs[0] = 0;
   $avg_Wmsecs[0] = 0;
   $avg_Riops[0]  = 0;
   $avg_Wiops[0]  = 0;
   $avg_Rsize[0]  = 0;
   $avg_Wsize[0]  = 0;

   foreach $drive (keys %drive_name_index) {
        $drive_id = $drive_name_index{$drive};

      $avg_qlen[0] += $avg_qlen[$drive_id];
      $avg_Rqlen[0] += $avg_Rqlen[$drive_id];
      $avg_Wqlen[0] += $avg_Wqlen[$drive_id];
      $avg_Rmsecs[0] += $avg_Rmsecs[$drive_id] * $avg_Riops[$drive_id]; 
      $avg_Wmsecs[0] += $avg_Wmsecs[$drive_id] * $avg_Wiops[$drive_id]; 
      $avg_Riops[0] += $avg_Riops[$drive_id];
      $avg_Wiops[0] += $avg_Wiops[$drive_id];
      $avg_Rsize[0] += $avg_Rsize[$drive_id] * $avg_Riops[$drive_id];
      $avg_Wsize[0] += $avg_Wsize[$drive_id] * $avg_Wiops[$drive_id];
   }

   if ( $avg_Riops[0] ) {
      $avg_Rmsecs[0] /= $avg_Riops[0];
      $avg_Rsize[0] /= $avg_Riops[0];
   }
   if ( $avg_Wiops[0] ) {
      $avg_Wmsecs[0] /= $avg_Wiops[0];
      $avg_Wsize[0] /= $avg_Wiops[0];
   }

   # Qlengths are totals, IOPS are totals ,
   # msecs and sizes are weighted averages 
}

sub bump_timestamp {
   local ($timestamp, $interval) = @_;

   local $sec, $min, $hour;

   @timestamp_parts = split (/ /, $timestamp);
   $date = $timestamp_parts[0];
   @dateparts = split (/\//, $date);
   $mo = $dateparts[0]; 
   $day = $dateparts[1];
   $year = $dateparts[2];
   $time = $timestamp_parts[1];
   if ($debug) { printf ("Time coming into bump_timestamp: %s \n", $time); }
   @timeparts = split (/:/, $time);
   $hour = $timeparts[0];
   $min  = $timeparts[1];
   $sec  = $timeparts[2];
   if ($debug) { printf ("Date: %s %d %d %d Time : %d %d %d \n",
                           $date, $mo, $day, $year, $hour, $min, $sec ); }
   if ( $sec < (60 - $interval) ) {
      $sec += $interval;
   }
   else {
      $sec = ($sec + $interval) - 60;
      if ( $min < 59 ) {
         $min++; 
      }
      else { 
         $min = 0;
         if ( $hour < 23 ) {
            $hour++;
         }
         else {
            $hour = 0;
            $day++;      # I am not going to figure out
                         # days in month and leap years.
         }
      }
   }
   $newdate = sprintf ("%02d\/%02d\/%d", $mo, $day, $year);
   $newtime = sprintf ("%02d:%02d:%02d", $hour, $min, $sec);
   if ($debug) { printf ("New Time : %s \n", $newtime); }
   $newtimestamp = sprintf ("%s %s", $newdate, $newtime);
   return $newtimestamp;
}
  
sub usage {
   print <<EOM;
psf_ldisk.pl [-DhHnSt]  <perfstat filename>

Extracts key WIN2K logical disk stats from perfstat file. 

    Arguments and switches - 
       -D          Print data as well as header. Relevent only if -H specified. 
       -h          This message.
       -H          Print header line only unless -D specified.
       -n          Nice output instead of comma separated values.
       -S          Average all samples for one ldisk. 
       -t          Total all logical disk stats together. 
                   Only NETAPP LUNs are used and Response times and 
                   Sizes use weighted averages. 
EOM
exit;

}
