#!/usr/bin/env perl 

# This script is provided "as is" it is
# not warranted but may be supported on a limited basis
# by Network Appliance.
#
# psf_lun.pl
#
#   PerfStatFilter for Filer LUN stats .
#   v.09   01-25-2005 : Fixed bug where timestamps did not have leading zeros on
#                       month and day. This would cause sorting to produce the 
#                       wrong order. 
#   v.08   10-22-2004 : Added -A option to show all luns in each sample. 
#   v.07   09-02-2004 : Finally realized my problem with different files needing 
#                       different chping / chomping was due to DOS carraige returns.
#                       fixed this.
#   v0.06  06-24-2004 : Changes to support perfstat 5.30 and above. 
#   v0.05  03-23-2004 : Added -f <filer> for multi-filer perfstat files.
#                       Only tested against perfstat 5.20 Linux version.
#   v0.04  01-19-2004 : fixed bug where sample counts (and resulting time average 
#                       were incorrect on UNIX versions past 5.05. Also bug with
#                       time display on same versions. Added option to 
#                       read files having only lun stats in them, i.e. not 
#                       from perfstat. For people who don't use perfstat. 
#   v0.03  12-09-2003 : Changed from specificying #!/usr/local/bin to       
#                        #!/usr/bin/env perl . This even seemed to work on 
#                        my Win2K system. 
#                       Fixed lots of bugs.
#
#   v0.01  11-04-2003 : Initial implementation.
#

# EXPORTED CONSTANTS (Not really, you have to copy them)

use constant    _NA_LUN_TIMESTAMP       => 0;
use constant    _NA_LUN_NAME            => 1;
use constant    _NA_LUN_RIOPS           => 2;
use constant    _NA_LUN_WIOPS           => 3;
use constant    _NA_LUN_OIOPS           => 4;
use constant    _NA_LUN_RKBS            => 5;
use constant    _NA_LUN_WKBS            => 6;
use constant    _NA_LUN_RT              => 7;
use constant    _NA_LUN_QLEN            => 8;
use constant    _NA_LUN_QFULL           => 9;
use constant    _NA_LUN_PRTNR_IOPS      => 10;
use constant    _NA_LUN_PRTNR_KBS       => 11;

use Getopt::Std;
getopts("AdDf:hHi:LnO:Sz");

if ($opt_h) { usage(); }

if ($opt_d) {
   $debug = 1;  
} else {
   $debug = 0;
}

if ($opt_D) {
   $data_plus_header = 1;  
} else {
   $data_plus_header = 0;  
}

if ($opt_f) {
   $filer = $opt_f;
}

if ($opt_H) {
   $header = 1;  
} else {
   $header = 0;
}

if ($opt_i) {
   $interval = $opt_i;
}

if ($opt_L) {
   $show_luns = 1;  
} else {
   $show_luns = 0;
}

if ($opt_n) {
   $nice_output = 1;  
} else {
   $nice_output = 0;
}

if ($opt_O) {
   $ontap_rev   = $opt_O;  
} 

if ($opt_A) {
   $show_all_luns_per_sample = 1;
} else {
   $show_all_luns_per_sample = 0;
}
 
if ($opt_S) {
   $avg_all_samples = 1;  
} else {
   $avg_all_samples = 0;  
}

if ($opt_z) {
   $non_perfstat_file = 1;  
} else {
   $non_perfstat_file = 0;  
}

# from psf_versions.pl 

use constant    _NA_VERS_PERFSTAT_MAJOR => 0;
use constant    _NA_VERS_PERFSTAT_MINOR => 1;
use constant    _NA_VERS_ONTAP_VERSION  => 2;
use constant    _NA_VERS_ONTAP_MAJOR    => 3;
use constant    _NA_VERS_ONTAP_MINOR    => 4;
use constant    _NA_VERS_FILER_MODEL    => 5;
use constant    _NA_VERS_FILER_MAGIC1   => 6;
use constant    _NA_VERS_FILER_MAGIC2   => 7;
use constant    _NA_VERS_PERFSTAT_PLATFORM => 8;
use constant    _NA_VERS_NUM_SAMPLES    => 9;
use constant    _NA_VERS_SECS_PER_SAMPLE => 10;


$in_lun_stats = 0;
$lunid = 0;
$nextlunid = 0;
$old_vol = "xyzzy";
$samples=1;
$lun = "xyzzy";

@line;
@lun_name;
@lun_riops, @lun_rKBs, @lun_svct;
@lun_oiops, @lun_qfull, @lun_partner_iops, @lun_partner_KBs;
@lun_wiops, @lun_wKBs, @lun_qlen;
%lun_name_index;

$header_line = "Timestamp, LUN, Riops, Wiops, Oiops, RKBs, WKBs, RT, Qlen, QFull, Prtnriops, PrtnrKBs, \n";

if ( !$nice_output && $header ) {
   print $header_line;
   if (!$data_plus_header) {
     exit;
   }
}

$perfstat_file = $ARGV[0];
$version_line = `psf_versions.pl $perfstat_file`;
@versions = split (/,/, $version_line);
$perfstat_major = $versions[_NA_VERS_PERFSTAT_MAJOR];
$perfstat_minor = $versions[_NA_VERS_PERFSTAT_MINOR];
$perfstat_version = ($perfstat_major * 100) + $perfstat_minor;
$perfstat_platform = $versions[_NA_VERS_PERFSTAT_PLATFORM];
$ontap_major = $versions[_NA_VERS_ONTAP_MAJOR];
$ontap_minor = $versions[_NA_VERS_ONTAP_MINOR];
$model = $versions[_NA_VERS_FILER_MODEL];
$perfstat_samples = $versions[_NA_VERS_NUM_SAMPLES];
$perfstat_interval = $versions[_NA_VERS_SECS_PER_SAMPLE];
$timestamp = `psf_filer_timestamp.pl $perfstat_file`;
chomp $timestamp; 
$timestamp_list[$samples] = $timestamp;

if ( $debug ) { print @versions; }
     
open(INPUTA, '<' . $perfstat_file) || die 'Unable to open perfstat file (arg1) .';

     if ( $non_perfstat_file ) {
        $in_lun_stats = 1;
        $perfstat_major = 5; # to turn off the Windows stuff
        if ($ontap_rev) { #Passed in
           $ontap_major = $ontap_rev;
           if ($ontap_major = 4) {
              while ($Line = <INPUTA>) {
                 if ($Line =~ /asvc_t/) {
                    last;
                 }
              }
           }
        } else {
           $ontap_major = 5;
        }
        if ($debug) { printf ("Using ONTAP Major = %d \n", $ontap_major); }
     }

     # The following code is because ONTAP 6.4 delimits samples of the 
     # lun stats with a header prior to the data, whereas 
     # ONTAP 6.5 delimits samples with "---" following the sample. 
     # This is a cludge, but it works.
     if ( $ontap_major == 4 ) {
       $samples = 0; 
     }
     if ( $non_perfstat_file ) {
        $in_lun_stats = 1;
        $perfstat_major = 5; # to turn off the Windows stuff 
        if ($ontap_rev) { #Passed in 
           $ontap_major = $ontap_rev;
           if ($ontap_major = 4) {
              while ($Line = <INPUTA>) { 
                 if ($Line =~ /asvc_t/) {
                    last;
                 }
              }
           }
        } else {
           $ontap_major = 5; 
        }
        if ($debug) { printf ("Using ONTAP Major = %d \n", $ontap_major); }
     }
 
     while ($Line = <INPUTA>) { 

        if ( $in_lun_stats ) { 
                 $Line =~ s/\r//g;     # Remove DOS carraige returns
                 chomp $Line;
                 @line = split (/ +/, $Line);
                 if (($line[1] =~ /Read/ ) &&
                     ($line[2] =~ /Write/ )) {
                     if ($line[3] =~ /Other/ ) {
                        $extended_lun_stats = 1;
                     }
                     else {
                        $extended_lun_stats = 0;
                     }
                     next;
                 }
                 if (($line[1] =~ /Ops/ ) &&
                     ($line[2] =~  /Ops/ ))  {
                    next;
                 }
                 if ($Line =~ /=-=-=- rsh/ ) {
                    next;
                 }
                 #  if ($ontap_major >= 5) {
                 if ($extended_lun_stats) {
                     if ( $line[0] ) {
                        $lun = $line[10];
                     }
                     else {
                        $lun = $line[11];
                     }
                     if ( $debug ) { print @line; 
                                     printf ("\n");
                                     printf ("%s \n", $lun);}
                 }
                 else {
                     if ($ontap_major == 4)  {
                       #  ($ontap_minor == 1)) {  # Completely experimental should be gone
                        if ( $line[0] ) {
                           $lun = $line[7];
                        }
                        else {
                           $lun = $line[8];
                        }
                        if ( $debug ) { 
                            for ($k=0; $k<12; $k++) {
                                 printf ("%s ", $line[$k]);
                            }
                            printf ("\n");
                            printf ("%s \n", $lun);
                        }
                     }
                     else {
                        if ( $line[0] ) {
                           $lun = $line[6];
                        }
                        else {
                           $lun = $line[7];
                        }
                        if ( $debug ) { 
                            for ($k=0; $k<12; $k++) {
                                 printf ("%s ", $line[$k]);
                            }
                            printf ("\n");
                            printf ("%s \n", $lun);
                        }
                     }
                 }
                 if ( $lun =~ /vol/ ) {  
                       $in_end_of_sample = 0;
                       $found = 0;
                       while (($key,$value) = each %lun_name_index) {
                          if ($lun eq $key ) {
                             $lunid = $value;
                             $found = 1;
                          }
                       } 
                       if (!$found) {
                             $next_lunid++;
                             $lunid = $next_lunid;
                             $lun_name_index {$lun} = $lunid;
                             $lun_name[$lunid]          = $lun;
                             @lun_name_parts = split (/\//, $lun);
                             $vol = $lun_name_parts[2];
                             $found_vol = 0;
                             while (($key,$value) = each %vol_name_index) {
                                if ($vol eq $key ) {
                                   $volid = $value;
                                   $found_vol = 1;
                                }
                             }
                             if (!$found_vol) {
                                $next_volid++;
                                $volid = $next_volid;
                                $vol_name_index {$vol} = $volid;
                             }
                             $lun_vol[$lunid] = $volid;
                             if ($show_all_luns_per_sample) {
                                # print null samples for previous samples if any 
                                $curr_samples = $samples;
                                $curr_timestamp = $timestamp;
                                for ($samples=1;$samples < $curr_samples;$samples++) {
                                   $timestamp = $timestamp_list[$samples];
                                   print_lun_sample ($lunid);
                                }
                                $samples = $curr_samples; 
                                $timestamp = $curr_timestamp;
                             }
                             if ( $debug ) { printf ("New lun %s %d \n", $lun, $lunid); }
                       }
                       if ($line[0]) {
                          $index = 0;
                       }
                       else {
                          $index = 1;
                       }
                       $lun_riops[$lunid][$samples]        = $line[$index]; $index++;
                       $lun_wiops[$lunid][$samples]        = $line[$index]; $index++;
                       if (($extended_lun_stats) || 
                          ($ontap_major ==4 )) {
                          $lun_oiops[$lunid][$samples]        = $line[$index]; $index++;
                       }
                       if ($extended_lun_stats &&  ($ontap_major >= 5)) {
                          $lun_qfull[$lunid][$samples]        = $line[$index]; $index++;
                       }
                       $lun_rKBs[$lunid][$samples]         = $line[$index]; $index++;
                       $lun_wKBs[$lunid][$samples]         = $line[$index]; $index++;
                       $lun_svct[$lunid][$samples]         = $line[$index]; $index++;
                       $lun_qlen[$lunid][$samples]         = $line[$index]; $index++;
                       if ($extended_lun_stats &&  ($ontap_major >= 5)) {
                          $lun_partner_iops[$lunid][$samples] = $line[$index]; $index++;
                          $lun_partner_KBs[$lunid][$samples]  = $line[$index]; $index++;
                       }
                       if ( $debug ) { printf ("Found lun %s riops %d wiops %d qfull %d \n", 
                                                     $lun, $lun_riops[$lunid][$samples], 
                                                     $lun_wiops[$lunid][$samples],
                                                     $lun_qfull[$lunid][$samples]); }
                       if ( ( $show_luns ) && 
                            ( !$avg_all_samples ) && 
                            ( !$show_all_luns_per_sample )) {
                               print_lun_sample ($lunid);
                       }
                  }
                  elsif ( (( $ontap_major == 4 )  && 
                           ( $Line =~ /asvc_t/ )) || 
                          (( $ontap_major >= 5)  &&
                           (( $Line =~ /--/    ) ||
                            ( !$Line )             ) ))   { #end of sample
                       if (!$in_end_of_sample) { # avoid multiple consecutive
                                                 # end of sample delimiters
                          agg_sample_by_vol ();
                          if ( ( !$show_luns ) && 
                               ( !$avg_all_samples ) ) {
                             print_sample_by_vol ();
                          }
                          if ($show_all_luns_per_sample) {
                             print_sample_by_lun ();
                          } 
                          $samples++;
                          if ($debug) { printf ("Found end of sample \n"); }
                          $timestamp = bump_timestamp ($timestamp);
                          $timestamp_list[$samples] = $timestamp;
                       }
                       $in_end_of_sample = 1;
                       next; 
                  }
                  elsif ($Line =~ / POSTSTAT/ ) {
                       $in_lun_stats = 0;
                       last;
                  }
        } # if $in_lun_stats
        # elsif    ( (((($perfstat_major == 2) && 
        if    ( (((($perfstat_major == 2) && 
                   ($perfstat_minor > 0)) || 
                  (($perfstat_major == 5) &&
                   ($perfstat_platform =~ /Windows/))) &&
                   ($Line =~ /=-= lun_stats.out/)) ||
                  (((($perfstat_major == 5) &&
                    ($perfstat_minor >= 2)) ||
                    ($perfstat_major > 5)) &&
               #    ($perfstat_platform =~ /Unix/) &&
                   ($Line =~ /lun_stats.out/ )) ||
                  (($perfstat_platform =~ /Unix/ ) &&
                   ($Line =~ /lun_stats/ ))) {
           @line = split (/\s+/, $Line);
           $curr_filer = $line[2];
           if ( $debug ) {
              printf ("Current statit filer : %s \n", $curr_filer);
           }
           if ( (!$filer) ||
                ($filer =~ /^$curr_filer/) ) {
              $in_lun_stats = 1;
           }
           else {
              next;
           }
           if ($debug) { printf ("Found lun stats \n"); }
           # For Windows versions (and maybe very old UNIX versions) the
           # next two lines have time and date and the following one has 
           # interval. For new UNIX versions this is not provided. 
           if (($perfstat_major == 2 ) ||
               (($perfstat_major == 5) &&
                ($perfstat_minor <= 24) &&
                ($perfstat_platform =~ /Windows/ ))) { 
              $Line = <INPUTA>;
              @line = split (/ +/, $Line);
              $date = $line[1];
              if ($debug) { printf ("Base Date : %s \n", $date); }
              if (($perfstat_major == 5 ) && 
                  ($perfstat_minor >= 30 ) ) {
                 $time = $line[2];
                 $ampm = "";
              }
              else {
                 $Line = <INPUTA>;
                 @line = split (/ +/, $Line);
                 if ( length($line[0]) ) {
                    $time = $line[0];
                    $ampm = $line[1];
                 }
                 else {
                    $time = $line[1];
                    $ampm = $line[2];
                 }
                 if ( length($ampm) == 0 ) {
                    chop $time; 
                    chop $time; 
                    $ampm = chop $time;
                    if ( $ampm =~ /p/ ) {
                       $ampm = "PM"; 
                    }
                    else {
                       $ampm = "AM";
                    }
                 }
                 if ($debug) { printf ("AM/PM     : %s \n", $ampm); }
              }
              @timeparts = split (/:/, $time);
              $hour = @timeparts[0];
              $min  = @timeparts[1];
              if ( ( $ampm =~ /PM/ ) && 
                   ( $hour < 12 ) )  {
                 # Add 12 hours to time
                 $hour += 12;
              }
              elsif ( ( $ampm =~ /AM/ ) &&
                      ( $hour == 12 ) ) {
                 $hour = 0;
              }
              $time = sprintf ("%02d:%02d:%02d", $hour, $min, 0);

              if ($debug) { printf ("Base Time : %s \n", $time); }
              $timestamp = sprintf ("%s %s", $date, $time);
              if ($debug) { printf ("Base timestamp : %s \n", $timestamp ); }
              if ($perfstat_interval > 0) {
                 $interval = $perfstat_interval;
                 if ($debug) { printf ("Interval       : %s \n", $interval );}
              }
              else {
                 $Line = <INPUTA>;
                 @line = split (/ +/, $Line);
                 $interval = $line[15];
                 if ($debug) { printf ("Interval       : %s \n", $interval ); }
              }
           }
           else {
              if ($perfstat_interval > 0) {
                 $interval = $perfstat_interval;
                 if ($debug) { printf ("Interval       : %s \n", $interval ); }
              }
              else {
                 $interval=10;
                 if ($debug) { printf ("Interval       : %s \n", $interval ); }
              }
           }
           if (!$interval) {
                $interval = 10;
                if ($debug) { printf ("Interval       : %s \n", $interval ); }
           }
        }
      } #while

     $samples--;
     if ($debug) { 
        printf ("Num Samples : %d \n", $samples);
     }

     if ( (!$show_luns) &&
          ($avg_all_samples) ) {
        agg_vol_samples();
        print_vol_avgs ();
     }
     
     if ( ($show_luns) &&
          ($avg_all_samples) ) {
        foreach $lun (keys %lun_name_index) {
           $lunid = $lun_name_index{$lun};
           if ( $debug ) {
               printf ("lun name : %s  lunid : %d \n", $lun, $lunid );
           }
           agg_lun_samples ( $lunid );
           print_lun_avgs ($lunid);  
        }
    }

# END OF MAIN 


sub print_lun_avgs {
   local ($lunid) = @_ ;

   if ( $nice_output ) {
             printf ("Summary lun statistics for lun %s \n", $lun_name[$lunid]);
             printf ("   Avg Read  IOPS         : \t %.0f \n", $lun_riops[$lunid][0]);
             printf ("   Avg Write IOPS         : \t %.0f \n", $lun_wiops[$lunid][0]);
             printf ("   Avg Other IOPS         : \t %.0f \n", $lun_oiops[$lunid][0]);
             printf ("   Avg Read  KBs          : \t %.0f \n", $lun_rKBs[$lunid][0]);
             printf ("   Avg Write KBs          : \t %.0f \n", $lun_wKBs[$lunid][0]);
             printf ("   Avg Residence Time (ms): \t %.1f \n", $lun_svct[$lunid][0]);
             printf ("   Avg Queue Length       : \t %.1f \n", $lun_qlen[$lunid][0]);
             if ($extended_lun_stats && ($ontap_major >= 5)) {
                printf ("   Avg QFull              : \t %.0f \n", $lun_qfull[$lunid][0]);
                printf ("   Avg Partner IOPS       : \t %.0f \n", $lun_partner_iops[$lunid][0]);
                printf ("   Avg Partner KBs        : \t %.0f \n", $lun_partner_KBs[$lunid][0]);
             }
   }
   else {
             printf ("%s, %s, ", $timestamp, $lun_name[$lunid]); 
             printf ("%.0f, %.0f, ", $lun_riops[$lunid][0], $lun_wiops[$lunid][0]);
             printf ("%.0f, ",
                    $lun_oiops[$lunid][0]);
             printf ("%.0f, %.0f, ",
                    $lun_rKBs[$lunid][0], $lun_wKBs[$lunid][0]);
             printf ("%.1f, %.1f, ",
                    $lun_svct[$lunid][0], $lun_qlen[$lunid][0]);
             if ($extended_lun_stats && ($ontap_major >= 5)) {
                printf ("%.0f, %.0f, %.0f,  ",
                    $lun_qfull[$lunid][0],
                    $lun_partner_iops[$lunid][0], $lun_partner_KBs[$lunid][0]);
             }
             printf ("\n");
   }
}

sub print_lun_sample {
    local ($lunid) = @_ ;

             printf ("%s, %s, ", $timestamp, $lun_name[$lunid]);
             printf ("%.0f, %.0f, ", $lun_riops[$lunid][$samples], $lun_wiops[$lunid][$samples]);
             printf ("%.0f, ",
                    $lun_oiops[$lunid][$samples]);
             printf ("%.0f, %.0f, ",
                    $lun_rKBs[$lunid][$samples], $lun_wKBs[$lunid][$samples]);
             printf ("%.1f, %.1f, ",
                    $lun_svct[$lunid][$samples], $lun_qlen[$lunid][$samples]);
             if ($extended_lun_stats && ($ontap_major >= 5)) {
                printf ("%.0f, %.0f, %.0f,  ",
                    $lun_qfull[$lunid][$samples],
                    $lun_partner_iops[$lunid][$samples], $lun_partner_KBs[$lunid][$samples]);
             }
             printf ("\n");
}

sub print_sample_by_lun {
    my ($lun, $lunid) ;

    foreach $lun (keys %lun_name_index) {
	$lunid = $lun_name_index{$lun};
	print_lun_sample ($lunid);
    }
}

sub print_sample_by_vol {
    my ($vol, $volid, $lun, $lunid) ;

   foreach $vol (keys %vol_name_index) {
             $volid = $vol_name_index{$vol};
             printf ("%s, %s, ", $timestamp, $vol);
             printf ("%.0f, %.0f, ", $vol_riops[$volid][$samples], $vol_wiops[$volid][$samples]);
             printf ("%.0f, ",
                    $vol_oiops[$volid][$samples]);
             printf ("%.0f, %.0f, ",
                    $vol_rKBs[$volid][$samples], $vol_wKBs[$volid][$samples]);
             printf ("%.1f, %.1f, ",
                    $vol_svct[$volid][$samples], $vol_qlen[$volid][$samples]);
             if ($extended_lun_stats && ($ontap_major >= 5)) {
                printf ("%.0f, %.0f, %.0f,  ",
                    $vol_qfull[$volid][$samples],
                    $vol_partner_iops[$volid][$samples], $vol_partner_KBs[$volid][$samples]);
             }
             printf ("\n");
  } 
}

sub print_vol_avgs {
    my ($vol, $volid, $lun, $lunid) ;

   foreach $vol (keys %vol_name_index) {
             $volid = $vol_name_index{$vol};
             printf ("%s, %s, ", $timestamp, $vol);
             printf ("%.0f, %.0f, ", $vol_riops[$volid][0], $vol_wiops[$volid][0]);
             printf ("%.0f, ",
                    $vol_oiops[$volid][0]);
             printf ("%.0f, %.0f, ",
                    $vol_rKBs[$volid][0], $vol_wKBs[$volid][0]);
             printf ("%.1f, %.1f, ",
                    $vol_svct[$volid][0], $vol_qlen[$volid][0]);
             if ($extended_lun_stats && ($ontap_major >= 5)) {
                printf ("%.0f, %.0f, %.0f,  ",
                    $vol_qfull[$volid][0],
                    $vol_partner_iops[$volid][0], $vol_partner_KBs[$volid][0]);
             }
             printf ("\n");
  }
}

sub agg_sample_by_vol {
   my ($vol, $volid);

   foreach $vol (keys %vol_name_index) {
      $volid = $vol_name_index{$vol};
      $vol_riops[$volid][$samples] = 0;
      $vol_wiops[$volid][$samples] = 0;
      $vol_oiops[$volid][$samples] = 0;
      $vol_rKBs[$volid][$samples] = 0;
      $vol_wKBs[$volid][$samples] = 0;
      $vol_svct[$volid][$samples] = 0;
      $vol_qlen[$volid][$samples] = 0;
      $vol_qfull[$volid][$samples] = 0;
      $vol_partner_iops[$volid][$samples] = 0;
      $vol_partner_KBs[$volid][$samples] = 0;
      foreach $lun (keys %lun_name_index) {
         $lunid = $lun_name_index{$lun};
         if ($volid == $lun_vol[$lunid]) {
            $vol_riops[$volid][$samples] += $lun_riops[$lunid][$samples];
            $vol_wiops[$volid][$samples] += $lun_wiops[$lunid][$samples];
            $vol_oiops[$volid][$samples] += $lun_oiops[$lunid][$samples];
            $vol_rKBs[$volid][$samples] += $lun_rKBs[$lunid][$samples];
            $vol_wKBs[$volid][$samples] += $lun_wKBs[$lunid][$samples];
            $vol_svct[$volid][$samples] += $lun_svct[$lunid][$samples] * 
                                            ( $lun_riops[$lunid][$samples] +
                                              $lun_wiops[$lunid][$samples] );
            $vol_qlen[$volid][$samples] += $lun_qlen[$lunid][$samples]; 
            $vol_qfull[$volid][$samples] += $lun_qfull[$lunid][$samples]; 
            $vol_partner_iops[$volid][$samples] += $lun_partner_iops[$lunid][$samples]; 
            $vol_partner_KBs[$volid][$samples] += $lun_partner_KBs[$lunid][$samples]; 
         }
      }
      if ( $vol_riops[$volid][$samples] +
           $vol_wiops[$volid][$samples] ) {
         $vol_svct[$volid][$samples] /= ( $vol_riops[$volid][$samples] +
                                       $vol_wiops[$volid][$samples] );
      }
   }
}

sub agg_vol_samples {
   my ($vol, $volid);
   
   if ($debug) { printf ("In agg_vol_samples : \n"); }
   foreach $vol (keys %vol_name_index) {
      $volid = $vol_name_index{$vol};
      $vol_riops[$volid][0] = 0;
      $vol_wiops[$volid][0] = 0;
      $vol_oiops[$volid][0] = 0;
      $vol_rKBs[$volid][0] = 0;
      $vol_wKBs[$volid][0] = 0;
      $vol_svct[$volid][0] = 0;
      $vol_qlen[$volid][0] = 0;
      $vol_qfull[$volid][0] = 0;
      $vol_partner_iops[$volid][0] = 0;
      $vol_partner_KBs[$volid][0] = 0;

      for ($sample = 1; $sample <= $samples; $sample++) {
         $vol_riops[$volid][0] += $vol_riops[$volid][$sample];
         $vol_wiops[$volid][0] += $vol_wiops[$volid][$sample];
         $vol_oiops[$volid][0] += $vol_oiops[$volid][$sample];
         $vol_rKBs[$volid][0] += $vol_rKBs[$volid][$sample];
         $vol_wKBs[$volid][0] += $vol_wKBs[$volid][$sample];
         $vol_svct[$volid][0] += $vol_svct[$volid][$sample] * 
                                  ( $vol_riops[$volid][$sample] + 
                                    $vol_wiops[$volid][$sample] );
         $vol_qlen[$volid][0] += $vol_qlen[$volid][$sample];
         $vol_qfull[$volid][0] += $vol_qfull[$volid][$sample];
         $vol_partner_iops[$volid][0] += $vol_partner_iops[$volid][$sample];
         $vol_partner_KBs[$volid][0] += $vol_partner_KBs[$volid][$sample];
         if ($debug) {
            printf ("sample %d %s %d %d %d \n", $sample, $vol, 
                          $vol_riops[$volid][$sample],
                          $vol_wiops[$volid][$sample],
                          $vol_oiops[$volid][$sample]);
         }
      }
      if ( $samples ) {
         $vol_riops[$volid][0] /= $samples;
         $vol_wiops[$volid][0] /= $samples;
         $vol_oiops[$volid][0] /= $samples;
         $vol_rKBs[$volid][0] /= $samples;
         $vol_wKBs[$volid][0] /= $samples;
         if ( $vol_riops[$volid][0] + $vol_wiops[$volid][0] ) {
            $vol_svct[$volid][0] /= $samples * 
                                 ( $vol_riops[$volid][0] + 
                                   $vol_wiops[$volid][0] );
         }
         $vol_qlen[$volid][0] /= $samples;
         $vol_qfull[$volid][0] /= $samples;
         $vol_partner_iops[$volid][0] /= $samples;
         $vol_partner_KBs[$volid][0] /= $samples;
      }
   }
}

sub agg_lun_samples {
   local ($lunid) = @_ ;

   if ($debug) { printf ("In average_lun_stats. samples = %d \n", $samples) };
   $lun_riops[$lunid][0]         = 0;
   $lun_wiops[$lunid][0]         = 0;
   $lun_oiops[$lunid][0]         = 0;
   $lun_rKBs[$lunid][0]          = 0;
   $lun_wKBs[$lunid][0]          = 0;
   $lun_svct[$lunid][0]          = 0;
   $lun_qlen[$lunid][0]          = 0;
   $lun_qfull[$lunid][0]         = 0;
   $lun_partner_iops[$lunid][0]  = 0;
   $lun_partner_KBs[$lunid][0]   = 0;

   for ($sample = 1; $sample <= $samples; $sample++) {
         $lun_riops[$lunid][0] += $lun_riops[$lunid][$sample];
         $lun_wiops[$lunid][0] += $lun_wiops[$lunid][$sample];
         $lun_oiops[$lunid][0] += $lun_oiops[$lunid][$sample];
         $lun_rKBs[$lunid][0]  += $lun_rKBs[$lunid][$sample];
         $lun_wKBs[$lunid][0]  += $lun_wKBs[$lunid][$sample];
         $lun_svct[$lunid][0]  += $lun_svct[$lunid][$sample] * 
                                  ( $lun_riops[$lunid][$sample] + 
                                    $lun_wiops[$lunid][$sample] );
         $lun_qlen[$lunid][0]  += $lun_qlen[$lunid][$sample];
         $lun_qfull[$lunid][0]  += $lun_qfull[$lunid][$sample];
         $lun_partner_iops[$lunid][0]  += $lun_partner_iops[$lunid][$sample];
         $lun_partner_KBs[$lunid][0]  += $lun_partner_KBs[$lunid][$sample];
   }

   $lun_riops[$lunid][0]         /= $samples;
   $lun_wiops[$lunid][0]         /= $samples;
   $lun_oiops[$lunid][0]         /= $samples;
   $lun_rKBs[$lunid][0]          /= $samples;
   $lun_wKBs[$lunid][0]          /= $samples; 
   if ( ($$lun_riops[$lunid][0] + $lun_wiops[$lunid][0]) ) {
      $lun_svct[$lunid][0]          /= $samples *
                                    ( $lun_riops[$lunid][0] +
                                      $lun_wiops[$lunid][0] );
   }
   $lun_qlen[$lunid][0]          /= $samples;
   $lun_qfull[$lunid][0]         /= $samples;
   $lun_partner_iops[$lunid][0]  /= $samples;
   $lun_partner_KBs[$lunid][0]   /= $samples;
}

sub month_to_numeric {
   local ($month) = @_ ;
   my $result;

   if ($month =~ /Jan/ ) {
        $result = 1;
   }
   elsif ($month =~ /Feb/ ) {
        $result = 2;
   }
   elsif ($month =~ /Mar/ ) {
        $result = 3;
   }
   elsif ($month =~ /Apr/ ) {
        $result = 4;
   }
   elsif ($month =~ /May/ ) {
        $result = 5;
   }
   elsif ($month =~ /Jun/ ) {
        $result = 6;
   }
   elsif ($month =~ /Jul/ ) {
        $result = 7;
   }
   elsif ($month =~ /Aug/ ) {
        $result = 8;
   }
   elsif ($month =~ /Sep/ ) {
        $result = 9;
   }
   elsif ($month =~ /Oct/ ) {
        $result = 10;
   }
   elsif ($month =~ /Nov/ ) {
        $result = 11;
   }
   elsif ($month =~ /Dec/ ) {
        $result = 12;
   }
   else {
        $result = -1;
   }
   return $result;
}

sub bump_timestamp {
   local ($timestamp) = @_;

   @timestamp_parts = split (/ /, $timestamp);
   $date = $timestamp_parts[0];
   @dateparts = split (/[\/,-]/, $date);
   $mo = $dateparts[0]; 
   $day = $dateparts[1];
   $year = $dateparts[2];
   $time = $timestamp_parts[1];
   @timeparts = split (/:/, $time);
   $hour = $timeparts[0];
   $min  = $timeparts[1];
   $sec  = $timeparts[2];
   if ($debug) { printf ("Date: %s %d %d %d Time : %d %d %d \n",
                           $date, $mo, $day, $year, $hour, $min, $sec ); }
   if ( $sec < (60 - $interval) ) {
      $sec += $interval;
   }
   else {
      $sec = ($sec + $interval) - 60;
      if ( $min < 59 ) {
         $min++; 
      }
      else { 
         $min = 0;
         if ( $hour < 23 ) {
            $hour++;
         }
         else {
            $hour = 0; 
            $day++;      # I am not going to figure out 
                         # days in month and leap years. 
         }
      }
   }
   $newdate = sprintf ("%02d\/%02d\/%d", $mo, $day, $year);
   $newtime = sprintf ("%02d:%02d:%02d", $hour, $min, $sec);
   if ($debug) { printf ("New Time : %s \n", $newtime); }
   $newtimestamp = sprintf ("%s %s", $newdate, $newtime);
   return $newtimestamp;
}            

sub usage {
   print <<EOM;
psf_lun.pl [-ADhHLnS] [-f <filer>] <perfstat filename>

Summarizes lun statistics.

    Arguments and switches - 
       -A          Show each lun in each sample, even if no activity. 
       -D          Display Data as well as headers
       -f <filer>  For multi-protocol filers choose this one. 
       -h          This message.
       -H          Display header.
       -L          Show individual LUN stats. Default is to 
                   aggregate by volume.
       -n          Display in nice output. 
       -S          Average all samples together. 
EOM
exit;

}
