#!/usr/bin/env perl 

# This script is provided "as is" it is
# not warranted but may be supported on a limited basis
# by Network Appliance.
#
# psf_lun_info.pl
#
#   PerfStatFilter for Filer LUN information .
#
#   v0.02  03-23-2004 : Added -f <filer> for multi-filer perfstat files.
#                       Only tested against perfstat 5.20 Linux version.
#
#   v0.01  12-17-2003 : Initial implementation.
#

# EXPORTED CONSTANTS (Not really, you have to copy them)

use constant    _NA_LUN_NAME            => 0;
use constant    _NA_LUN_VOL_NAME        => 1;
use constant    _NA_LUN_TYPE            => 2;
use constant    _NA_LUN_SIZE            => 3;

use Getopt::Std;
getopts("dDf:hH");

if ($opt_h) { usage(); }

if ($opt_d) {
   $debug = 1;  
} else {
   $debug = 0;
}

if ($opt_D) {
   $data_plus_header = 1;  
} else {
   $data_plus_header = 0;  
}

if ($opt_f) {
   $filer = $opt_f;
}

if ($opt_H) {
   $header = 1;  
} else {
   $header = 0;
}

# from psf_versions.pl 

use constant    _NA_VERS_PERFSTAT_MAJOR => 0;
use constant    _NA_VERS_PERFSTAT_MINOR => 1;
use constant    _NA_VERS_ONTAP_VERSION  => 2;
use constant    _NA_VERS_ONTAP_MAJOR    => 3;
use constant    _NA_VERS_ONTAP_MINOR    => 4;
use constant    _NA_VERS_FILER_MODEL    => 5;
use constant    _NA_VERS_FILER_MAGIC1   => 6;
use constant    _NA_VERS_FILER_MAGIC2   => 7;

 
$perfstat_file = $ARGV[0];
open(INPUTA, '<' . $perfstat_file) || die 'Unable to open perfstat file (arg1) .';

@line;

$header_line = "LUN Name, Vol Name, LUN Type, LUN Size, \n";
$version_line = `psf_versions.pl $perfstat_file`;
@versions = split (/,/, $version_line);
$perfstat_major = $versions[_NA_VERS_PERFSTAT_MAJOR];
$perfstat_minor = $versions[_NA_VERS_PERFSTAT_MINOR];
$ontap_major = $versions[_NA_VERS_ONTAP_MAJOR];
$ontap_minor = $versions[_NA_VERS_ONTAP_MINOR];
$model = $versions[_NA_VERS_FILER_MODEL];

if ( $debug ) { print @versions; }

if ( !$nice_output && $header ) {
   print $header_line;
   if (!$data_plus_header) {
     exit;
   }
}

while ($Line = <INPUTA>) { 
   if ( ( $Line =~ /^=-=-/ ) &&
           ( $Line =~ /PRESTATS/ ) &&
           ( $Line =~ /lun show -v all/ ) ) {
      if ($Line =~ /sanlun/ ) {   # Skip this one and get the filer lun show 
         next;
      }
      @line = split (/\s+/, $Line);
      $curr_filer = $line[2];
      if ( $debug ) {
         printf ("Current statit filer : %s \n", $curr_filer);
      }
      if ( (!$filer) ||
            ($filer =~ /^$curr_filer/) )  {
         last;
      }
   }
   next;
}

while ($Line = <INPUTA>) { 
   if ( $Line =~ /=-=-=-/ ) {
      last;
   }
   elsif ( $Line =~ /\/vol\// ) {
      @line = split (/\s+/, $Line);
      $lun = $line[1];
      @lun_name_parts = split (/\//, $lun);
      $vol = sprintf ("%s", $lun_name_parts[2]);
      $lun_size = $line[2];
      $Line = <INPUTA>;        # Serial#
      $Line = <INPUTA>;        # Share  
      $Line = <INPUTA>;        # Space Reservations
      $Line = <INPUTA>;        # Multiprotocol type
      @line = split (/\s+/, $Line);
      $lun_type = $line[3];
      printf ("%s, %s, %s, %s, \n",
               $lun,
               $vol,
               $lun_type,
               $lun_size);
   }
}

sub usage {
   print <<EOM;
psf_lun_info.pl [-DhH] [-f <filer>] <perfstat filename>

Summarizes lun statistics.

    Arguments and switches - 
       -D          Display Data as well as headers
       -f <filer>  For multi-filer perfstat choose this filer.
       -h          This message.
       -H          Display header.
EOM
exit;

}
