#!/usr/bin/env perl 

# from psf_versions.pl
use constant    _NA_VERS_PERFSTAT_MAJOR => 0;
use constant    _NA_VERS_PERFSTAT_MINOR => 1;
use constant    _NA_VERS_ONTAP_VERSION  => 2;
use constant    _NA_VERS_ONTAP_MAJOR    => 3;
use constant    _NA_VERS_ONTAP_MINOR    => 4;
use constant    _NA_VERS_FILER_MODEL    => 5;
use constant    _NA_VERS_FILER_MAGIC1   => 6;
use constant    _NA_VERS_FILER_MAGIC2   => 7;


use Getopt::Std;
getopts("dDf:hnH");

if ($opt_h) { usage(); }

if ($opt_d) {
   $debug = 1;
} else {
   $debug = 0;
}

if ($opt_D) {
   $data_plus_header = 1;
   $nice_output = 0;
} else {
   $data_plus_header = 0;
}

if ($opt_f) {
   $filer = $opt_f;
}

if ($opt_H) {
   $header = 1;
   $nice_output = 0;
} else {
   $header = 0;
}

if ($opt_n) {   
    $nice_output = 1;
} else {
    $nice_output = 0;
}

$header_line1 = "Timestamp, LUN_riops, LUN_wiops, LUN_rKBs, LUN_wKBs, ";

if (!$nice_output && $header) {
   print $header_line1;
   printf ("\n");
   if (!$data_plus_header) {
     exit;
   }
}

$perfstat_file = $ARGV[0];
open(INPUTA, '<' . $perfstat_file) || die 'Unable to open perfstat file.';

$found_lun_stats = 0;

$version_info = `psf_versions.pl $perfstat_file`;
@line = split (/,/, $version_info);
$ontap_version = @line[_NA_VERS_ONTAP_VERSION];
$ontap_major =  @line[_NA_VERS_ONTAP_MAJOR];
$ontap_minor =  @line[_NA_VERS_ONTAP_MINOR];
$model = @line[_NA_VERS_FILER_MODEL];
$perfstat_major =  @line[_NA_VERS_PERFSTAT_MAJOR];
$perfstat_minor =  @line[_NA_VERS_PERFSTAT_MINOR];

$timestamp = `psf_filer_timestamp.pl $filer_str $perfstat_file`;
chomp $timestamp;

# Get to POSTSTATs 
while ($Line = <INPUTA>) {
   if ($Line =~ /POSTSTAT/ ) {  
      last ;
   }
}

$got_secs = 0;
while ($Line = <INPUTA>) {
  if ($Line =~ / POSTSTAT.*lun stats/ ) {
      @line = split (/\s+/, $Line);
      $curr_filer = $line[2];
      if ( (!$filer) ||
           ($filer =~ /^$curr_filer/) )  {
         if ( $debug ) { print "Found lun stats \n"; }
         $found_lun_stats = 1;
         $Line = <INPUTA>;
         @line = split (/\s+/, $Line);
         $fcp_read_KBs = 0;
         $fcp_write_KBs = 0;
         $fcp_read_iops = 0;
         $fcp_write_iops = 0;
         while ($line[1] =~ /vol/ ) {
            if (!$got_secs) {
               # Must consider lun pathnames that have spaces in them
               # Such as :
#  /vol/SQLVol2/SQL DATABASE 01_02/sql2 database 01_02.lun  (0 hours, 0 minutes, 5 seconds)
               # First split on commas and then on white space. 
               @line = split (/,/, $Line);
               $secs_str = $line[2];
               $minutes_str = $line[1];
               @secs_parts = split (/ +/, $secs_str);
               $secs = $secs_parts[1];
               @minutes_parts = split (/ +/, $minutes_str);
               $minutes = $minutes_parts[1];
               $secs = (60 * $minutes) + $secs;
               $got_secs = 1;
            }
            $Line = <INPUTA>;
            $Line = <INPUTA>;
            @line = split (/\s+/, $Line);
            $fcp_read_KBs += $line[1] / $secs;
            $fcp_write_KBs += $line[2] / $secs;
            $fcp_read_iops += $line[3] / $secs;
            $fcp_write_iops += $line[4] / $secs;
            $Line = <INPUTA>;
            @line = split (/ +/, $Line);
         }
      }
      else {
         next;
      }
   }
   elsif (($found_lun_stats) && 
          ($Line =~ /=-=-=/) ) {
      last;
   }
}

printf ("%s, %d, %d, %d, %d, \n",
         $timestamp, $fcp_read_iops, $fcp_write_iops,  $fcp_read_KBs, $fcp_write_KBs);

sub usage {
   print <<EOM;
lun_iops_kbs.pl [DHh] [-f <filer>]  <perfstat filename>

    Arguments and switches - 
       -D          Print data as well as header. Only 
                   relevent if -H  and -c specified.
       -H          Print a header line only unless -D specified. Only
                   relevant if -c specified.
       -h          This message.
EOM
exit;

}

