#!/usr/bin/env perl 

# This script is provided "as is" it is
# not warranted but may be supported on a limited basis
# by Network Appliance.
#
# psf_wafl_cp_kinds.pl
#
#   PerfStatFilter for certain WAFL CP related stats .
#
#   v0.03  11-09-2004 : Removed call to psf_versions.pl since none of values are used.
#   v0.02  03-23-2004 : Added -f <filer> for multi-filer perfstat files.
#                       Only tested against perfstat 5.20 Linux version.#
#   v0.01  02-25-2004 : Initial implementation.
#

# EXPORTED CONSTANTS (Not really, you have to copy them)

use constant _NA_TIMESTAMP                  => 0;
use constant _NA_SYMBOL                     => 1;
use constant _NA_ALLOCS                     => 2;
use constant _NA_BYTES                      => 3;

# Imported
use constant    _NA_VERS_PERFSTAT_MAJOR => 0;
use constant    _NA_VERS_PERFSTAT_MINOR => 1;
use constant    _NA_VERS_ONTAP_VERSION  => 2;
use constant    _NA_VERS_ONTAP_MAJOR    => 3;
use constant    _NA_VERS_ONTAP_MINOR    => 4;
use constant    _NA_VERS_FILER_MODEL    => 5;
use constant    _NA_VERS_FILER_MAGIC1   => 6;
use constant    _NA_VERS_FILER_MAGIC2   => 7;
use constant    _NA_VERS_PERFSTAT_PLATFORM => 8;


use Getopt::Std;
getopts("dDf:hHnT:S:");

if ($opt_h) { usage(); }

if ($opt_d) {
   $debug = 1;
} else {
   $debug = 0;
}

if ($opt_D) {
   $data_plus_header = 1;
} else {
   $data_plus_header = 0;
}

if ($opt_f) {
   $filer = $opt_f;
}

if ($opt_H) {
   $header = 1;
} else {
   $header = 0;
}

if ($opt_S) {
   $symbol = $opt_S;
} else {
   usage();
}

if ($opt_T) {
   $timestamp = $opt_T;
}

$header_line = "Timestamp,      Symbol, Cum Allocs, Cum Bytes, Allocs, Bytes, ";

if (  !$nice_output && $header ) {
   printf ("%s \n", $header_line);
   if ( !$data_plus_header ) {
      exit;
   }
}

$perfstat_file = $ARGV[0];
open(INPUTA, '<' . $perfstat_file) || die 'Unable to open perfstat file.';

$seconds = 0;

if (!$timestamp) {
   if ( $filer ) {
      $timestamp = `psf_filer_timestamp.pl -f $filer $perfstat_file`;
   }
   else {
      $timestamp = `psf_filer_timestamp.pl $perfstat_file`;
   }
}
chop $timestamp;

# Get to POSTSTATs 
while ($Line = <INPUTA>) {
   if ($Line =~ /POSTSTAT/ ) {  
      last ;
   }
}

$found_mem_stats = 0;
$found_mem_stats_e = 0;
while ($Line = <INPUTA>) {
   if ( $found_mem_stats && $found_mem_stats_e ) {
      last;
   }
   if ( ( $Line =~ /^=-=-/ ) &&
           ( $Line =~ /POSTSTATS/ ) &&
           ( $Line =~ /mem_stats$/ ) ) {
      @line = split (/\s+/, $Line);
      $curr_filer = $line[2];
      if ( $debug ) {
         printf ("Current statit filer : %s \n", $curr_filer);
      }
      if ( (!$filer) ||
             ($filer =~ /^$curr_filer/) )  {
		$found_mem_stats = 1;
		if ( $debug ) { print "Found mem_stats\n"; }
		while ($Line = <INPUTA>) { 
			if ($Line =~ /POSTSTAT/ ) {
				$done = 1;
				last;
			}
			if ($Line =~ /$symbol/ ){
				$Line =~ s/\s+//;
				@line = split (/\s+/, $Line);
				$cum_allocs = $line[0];          
				$cum_bytes = $line[1];
				if ($debug) {
					printf ("Symbol : %s Allocs: %d Bytes :%d \n", 
						$symbol, $cum_allocs, $cum_bytes);
				}
				last; 
			}
		}
      }
   }
   elsif ( ( $Line =~ /^=-=-/ ) &&
           ( $Line =~ /POSTSTATS/ ) &&
           ( $Line =~ /mem_stats -e/ ) ) {
      @line = split (/\s+/, $Line);
      $curr_filer = $line[2];
      if ( $debug ) {
         printf ("Current statit filer : %s \n", $curr_filer);
      }
      if ( (!$filer) ||
             ($filer =~ /^$curr_filer/) )  {
		$found_mem_stats_e = 1;
		if ( $debug ) { print "Found mem_stats_e\n"; }
		while ($Line = <INPUTA>) { 
			if ($Line =~ /POSTSTAT/ ) {
				$done = 1;
				last;
			}
			if ($Line =~ /$symbol/ ){
				$Line =~ s/\s+//;
				@line = split (/\s+/, $Line);
				$allocs = $line[0];          
				$bytes = $line[1];
				if ($debug) {
					printf ("Symbol : %s Allocs: %d Bytes :%d \n", 
						$symbol, $allocs, $bytes);
				}
				last; 
			}
		}
      }
   }
}
printf ("%s, %s, %d, %d, %d, %d, \n", 
	$timestamp, $symbol, $cum_allocs, $cum_bytes, $allocs, $bytes);

sub usage {
   print <<EOM;
psf_mem.pl [-dDHhn]  [-f <filer>] -S <symbol>  <perfstat filename>

Gives mem_stats for the given symbol. 

    Arguments and switches -
       -d          Print debug messages.
       -D          Print data as well as a header. Only relevent if -H specified. 
       -f <filer>  For multi-filer perfstats choose this one.
       -H          Print a header line only unless -D specified. 
       -h          This message.
       -S 	   Symbol - Required. 
EOM
exit;

}
