#!/usr/bin/env perl

# This script is provided "as is" it is
# not warranted but may be supported on a limited basis
# by Network Appliance.
#
# psf_ps.pl 
#
#	 v0.01  09-21-2005 : Initial implementation.
#

use constant _NA_TIMESTAMP             => 0;
use constant _NA_THD_NAME              => 1;
use constant _NA_THD_DOMAIN            => 2;
use constant _NA_THD_MAX_MSECS_CPU     => 3;
use constant _NA_THD_CPU_MSECS_PER_SEC => 4;


use Getopt::Std;
getopts("dDf:hHT:s");

if ($opt_h) { usage(); }

if ($opt_d) {
   $debug = 1;
} else {
   $debug = 0;
}

if ($opt_D) {
   $data_plus_header = 1;
} else {
   $data_plus_header = 0;
}

if ($opt_f) {
   $filer = $opt_f;
}

if ($opt_H) {
   $header = 1;
} else {
   $header = 0;
}

if ($opt_T) {  
   $timestamp  = $opt_T;
}

if ($opt_s) {
   $only_want_ps_runtimes = 1;
}

$header_line1 = "Timestamp, Thread, Domain, MaxCPUTenure_ms, CPU_msecs_per_sec, PercentCPU, ";  
$header_line2 = "Timestamp, ps secs, ps -s secs, ";

if ( $header ) {
   if ($only_want_ps_runtimes) {
	printf ("%s \n", $header_line2);
   } else {
	printf ("%s \n", $header_line1);
   }
   if ( !$data_plus_header ) {
      exit;
   }
}

$perfstat_file = $ARGV[0];
open(INPUTA, '<' . $perfstat_file) || die 'Unable to open perfstat file.';

$seconds = 0;

# Get to POSTSTATs
while ($Line = <INPUTA>) {
   if ($Line =~ /POSTSTAT/ ) {
      last ;
   }
}

if (!$timestamp) {
   if ( $filer ) {
      $timestamp = `psf_filer_timestamp.pl -f $filer $perfstat_file`;
   }
   else {
      $timestamp = `psf_filer_timestamp.pl $perfstat_file`;
   }
}
chop $timestamp;

$done = 0;
$Line = <INPUTA>;
while (!eof(INPUTA)) {
	if ( $done_ps_s && $done_ps  ) {
		last;
	}
	if ( ( $Line =~ /^=-=-/ ) &&
           ( $Line =~ /POSTSTATS/ ) &&
           ( $Line =~ /ps \-s/ ) ) {
                @line = split (/\s+/, $Line);
                $curr_filer = $line[2];
                if ( (!$filer) ||
                        ($filer =~ /^$curr_filer/) )  {

                        if ( $debug ) {
                                printf ("Found ps -s : %s \n", $curr_filer);
                        }
                        # Found ps -s. Get the seconds on the next line.
                        $found_ps_s_stats = 1;
                        $Line = <INPUTA>;
                        @line = split (/ +/, $Line);
                        $ps_s_seconds = $line[3];
			if ($only_want_ps_runtimes) {
				next;
			}
                        while ($Line = <INPUTA>) {
				if ($debug) {
					printf ("%s", $Line);
				}
                                if ($Line =~ /POSTSTAT/ ) {
                                        $done_ps_s = 1;
                                        last;
                                }
                                elsif ($Line =~ /RunCount/) {
                                        next;
                                }
                                else {
                                        @line = split (/\s+/, $Line);
                                        $RunCount = $line[2];
                                        $RunMax   = $line[3];
                                        $RunAve   = $line[4];
                                        $OnQMax   = $line[5]; 
                                        $OnQAve   = $line[6];
                                        $ThdName  = $line[9];
                                        $Runtime  = $RunCount * $RunAve;  # microseconds
                                        $usecs_per_sec = $Runtime / $ps_s_seconds;
                        		if ( $debug ) {
                                		printf ("Thd : %s  usecs/Sec: %d MaxCPUusecs: %d \n",
							$ThdName, $usecs_per_sec,
							$RunMax );
                        		}
                                        # Skip threads with less than 0.1% or 1000 usecs/sec
					# or less than 1 msec max cpu tenure. 
                                        if (($usecs_per_sec < 1000) &&
					    ($RunMax < 1000))  {
                                                next; 
                                        }
					if ($ThdName =~ /idle/) {
						next;
					}
                                        $ThreadMsecsPerSec{$ThdName} = $usecs_per_sec / 1000 ;
                                        $ThreadMaxtime{$ThdName} = $RunMax / 1000;
                        		if ( $debug ) {
                                		printf ("Thd : %s  Msecs/Sec: %d MaxCPUMsecs: %d \n",
							$ThdName, $ThreadMsecsPerSec{$ThdName} , 
							$ThreadMaxtime{$ThdName} );
                        		}
                                }
                        }
		}
		else {
			$Line = <INPUTA>;
		}
	}
	elsif ( ( $Line =~ /^=-=-/ ) &&
           ( $Line =~ /POSTSTATS/ ) &&
           ( $Line =~ /ps$/ ) ) {
                @line = split (/\s+/, $Line);
                $curr_filer = $line[2];
                if ( (!$filer) ||
                        ($filer =~ /^$curr_filer/) )  {

                        if ( $debug ) {
                                printf ("Found ps s : %s \n", $curr_filer);
                        }
                        # Found ps . Get the seconds on the next line.
                        $found_ps_stats = 1;
                        $Line = <INPUTA>;
                        @line = split (/ +/, $Line);
                        $ps_seconds = $line[3];
			if ($only_want_ps_runtimes) {
				next;
			}
                        while ($Line = <INPUTA>) {
				if ($debug) {
					printf ("%s", $Line);
				}
                                if ($Line =~ /POSTSTAT/ ) {
                                        $done_ps = 1;
                                        last;
                                }
                                elsif ($Line =~ /Domain/) {
                                        next;
                                }
                                else {
										# Remove leading spaces 
										$Line =~ s/^\s+//;
                                        @line = split (/\s+/, $Line);
                                        $Domain   = $line[2];
                                        $PercCPU  = $line[3];
                                        $ThdName  = $line[6];
                                        if ($ThdName =~ /idle/) {
                                                next;
                                        }
                                        $ThreadDomain{$ThdName} = $Domain;
                                        $ThreadPercentCPU{$ThdName} = $PercCPU;
                                }
                        }
                }
		else {
			$Line = <INPUTA>;
		}
	}
	else {
		 $Line = <INPUTA>;
	}
}

if ($only_want_ps_runtimes) {
	printf ("%s, %d, %d, \n", $timestamp, $ps_seconds, $ps_s_seconds);
} else {
	foreach $ThdName (keys %ThreadMsecsPerSec) {
		$Domain = $ThreadDomain{$ThdName};
		$PercentCPU = $ThreadPercentCPU{$ThdName};
		$MaxTenure = $ThreadMaxtime{$ThdName};
		$MsecsPerSec = $ThreadMsecsPerSec{$ThdName};
		printf ("%s, %s, %s, %d, %d, %d, \n", 
			$timestamp, $ThdName, $Domain, $MsecsPerSec, $MaxTenure, $PercentCPU );
	}
}

