#!/usr/bin/env perl 

# This script is provided "as is" it is
# not warranted but may be supported on a limited basis
# by Network Appliance.
#
# psf_qnm_fes.pl 
# 
#   Utility for PerfStatFiler scripts providing a 
# load dependent Queuing Network Model of disk array
#
#   v0.03  02-10-2004 : Updated calculation of seek time to use
#                       a seek profile table as produced by 
#                       DIXtrac.
#   v0.02  01-21-2004 : Updated calculation of seek time to use
#                       Lee's Equation. See DiskSim Reference Manual
#                       at CMU for details. 
#   v0.01  11-04-2003 : Initial implementation. 
#

# EXPORTED CONSTANTS (Not really, you have to copy them)

use constant    _NA_QNM_FESC_N          => 0;    # Populations i.e. num in system. 
use constant    _NA_QNM_FESC_X          => 1;    # Throughput (IOPS) 
use constant    _NA_QNM_FESC_R          => 2;    # Avg Residence time
use constant    _NA_QNM_FESC_S          => 3;    # Avg Residence time
use constant    _NA_QNM_FESC_U          => 4;    # Avg Per Disk Utilization
use constant    _NA_QNM_FESC_Q          => 5;    # Avg Qlength per disk

# For ST336704FC 
use constant    _NA_DISK_SEEK_TIME_RD          => .0052;
use constant    _NA_DISK_MAX_SEEK_TIME_RD      => .0110;
use constant    _NA_DISK_LAT                   => .003;
use constant    _NA_DISK_SEEK_1                => .0006;
use constant    _NA_DISK_NUM_CYLS              => 14100;

# For ST336605FC 
use constant    _NA_X5_DISK_SEEK_TIME_RD       => .0047;
use constant    _NA_X5_DISK_MAX_SEEK_TIME_RD   => .0086;
use constant    _NA_X5_DISK_LAT                => .003;
use constant    _NA_X5_DISK_SEEK_1             => .0004;
use constant    _NA_X5_DISK_NUM_CYLS           => 29549;

use constant    _NA_15K_DISK_SEEK_TIME_RD      => .0036;
use constant    _NA_15K_DISK_MAX_SEEK_TIME_RD  => .0065;
use constant    _NA_15K_DISK_LAT               => .002;
use constant    _NA_15K_DISK_SEEK_1            => .0002;
use constant    _NA_15K_DISK_NUM_CYLS          => 49854;

use constant    _NA_DISK_MAX_Q_DEPTH           => 3;
use constant    _NA_DISK_MAX_MAX_Q_DEPTH       => 16;

$max_qdepth = _NA_DISK_MAX_Q_DEPTH ;

use Getopt::Std;
getopts("1:2:3:4:5:6:7:8:E:FN:M:D:hdnHQ:S:V:s:");


if ($opt_1) {
   $svct[1] = $opt_1;  
} 
if ($opt_2) {
   $svct[2] = $opt_2;  
} 
if ($opt_3) {
   $svct[3] = $opt_3;  
} 
if ($opt_4) {
   $svct[4] = $opt_4;  
} 
if ($opt_5) {
   $svct[5] = $opt_5;  
} 
if ($opt_6) {
   $svct[6] = $opt_6;  
} 
if ($opt_7) {
   $svct[7] = $opt_7;  
} 
if ($opt_8) {
   $svct[8] = $opt_8;  
} 

if ($opt_h) { usage(); }

if ($opt_d) {
   $debug = 1;  
} else {
   $debug = 0;
}

if ($opt_D) {
   $Ndisks = $opt_D;  
} else {
   $num_disks = 7;           
}

if ($opt_E) {
   $every  = $opt_E;  
} else {
   $every  = 0;           
}

if ($opt_F) {
   $fast_drives = 1;
} else {
   $fast_drives = 0;           
}

if ($opt_H) {
   $header = 1;  
} else {
   $header = 0;
}

if ($opt_n) {
   $nice_output = 1;  
} else {
   $nice_output = 0;
}

if ($opt_N) {
   $Nusers = $opt_N;  
} else {
   print (" -N required - Number of customers \n");
   exit;
}

if ($opt_M) {
   $max_qdepth = $opt_M;  
}

if ($opt_V) {
   $Nvisits = $opt_V;  
} else {
   $Nvisits = 1;           
}

if ($opt_Q) {
   $other_workloads_disk_q = $opt_Q ;  
} else {
   $other_workloads_disk_q = 0;           
}

if ($opt_S) {
   $short_stroke_percent = $opt_S / 100;  
} else {
   $short_stroke_percent = 1;           
}

if ($opt_s) {
   $seek_profile_file  = $opt_s;  
}

if ((!$nice_output) && ($header) ) { 
      printf ("N,  X,  R, S, U, Q  \n");
}

if ($fast_drives) { 
   $avg_seek   = _NA_15K_DISK_SEEK_TIME_RD;
   $min_seek   = _NA_15K_DISK_SEEK_1;
   $max_seek   = _NA_15K_DISK_MAX_SEEK_TIME_RD;
   $avg_lat    = _NA_15K_DISK_LAT;
   $num_cyls   = _NA_15K_DISK_NUM_CYLS;
}
else {
   $avg_seek   = _NA_X5_DISK_SEEK_TIME_RD;
   $min_seek   = _NA_X5_DISK_SEEK_1;
   $max_seek   = _NA_X5_DISK_MAX_SEEK_TIME_RD;
   $avg_lat    = _NA_X5_DISK_LAT;
   $num_cyls   = _NA_X5_DISK_NUM_CYLS;
}

if ( $seek_profile_file ) {
   open(SEEKF, '<' . $seek_profile_file) || die 'Can not open seek time file. ';
   $num_cyls = 0;
   while ($Line = <SEEKF>) {
      $num_cyls++;
      @line = split (/\s+/, $Line);
      $seek_time_array[$num_cyls] = $line[1];
      if ($debug) { printf ("Seek time for %d cylinders is %.6f \n", 
                             $num_cyls, $seek_time_array[$num_cyls] ); }
   }
}
$used_cyls = $short_stroke_percent * $num_cyls;

for ( $i = 1; $i <= _NA_DISK_MAX_MAX_Q_DEPTH; $i++) {
    if ( ! $svct[$i] ) {       # If not specified. 
       $max_cyls_per_seek = $used_cyls/$i;
       $avg_cyls_per_seek = $max_cyls_per_seek / 3;
       if ( $seek_profile_file ) {
          $seek_time =  $seek_time_array[$avg_cyls_per_seek]/1000;
       }
       else {
          # Compute Lee's Seek Time Equation 
          $a = ((-10*$min_seek) + (15*$avg_seek) - (5*$max_seek)) / 
                             (3*sqrt($num_cyls));
          $b = ((7*$min_seek) - (15*$avg_seek) + (8*$max_seek)) / 
                             (3*$num_cyls);
          $c = $min_seek;
          $x = $avg_cyls_per_seek;
          $seek_time = ($a*sqrt($x-1)) + $b*($x-1) + $c;
       }

       $svct[$i] = $avg_lat + $seek_time;
    }
}

if ($debug) {
    printf ("Num cylinders : %d \n", $used_cyls);
    for ( $i = 1; $i <= $max_qdepth ; $i++) {
       printf ("Disk Service time for qlen = %d is %.4f \n",
                       $i, $svct[$i]);
    }
}
 
for ( $i = $max_qdepth+1; $i <= $Nusers; $i++) {
    $svct[$i] = $svct[$max_qdepth];
} 
$Q_dist[0][0] = 1.0;
$N = $Nusers;
$K = $Ndisks;
$V = $Nvisits / $K;
for ( $n=1; $n <= $N; $n++ ) {
    # Making some optimizations since all service centers are the same
#    for ( $k=1; $k <= $K; $k++ ) { 
#       $R[$k] = 0;
        $Rdisk = 0;
        $S = 0;
        $S0 = 0;     # Svct without queueing 
        for ( $j=1; $j <= $n; $j++ ) {
                $S += ($j + $other_workloads_disk_q) * $svct[$j] * $Q_dist[$j-1][$n-1];
#                $S += $j * $svct[$j] * $Q_dist[$j-1][$n-1];
                $S0 += $svct[$j] * $Q_dist[$j-1][$n-1];
        }
        $Rdisk = $S * $V;
#       $R[$k] = $S * $V;
#    }
#    $RT = 0;
#    for ( $k=1; $k <= $K; $k++ ) {    # Compute R, system residence time
#       $RT += $R[$k];
#    }
     $RT = $Rdisk * $K;

     $X = $n / $RT ;                   # Little's Law, equation #1 

#    for ( $k=1; $k <= $K; $k++ ) {    # Compute new qlen dist 
       for ( $j=1; $j <= $n; $j++ ) {
          $svc_rate = 1 / $svct[$j];
#         $Q_dist[$j][$n] =  ( $X / $svc_rate ) * 
          $Q_dist[$j][$n] =  ( ($X/$K) / $svc_rate ) * 
                               $Q_dist[$j-1][$n-1];
       }
       $Q_dist[0][$n] = 1.0 ;
       for ( $i=1; $i <= $n; $i++ ) {
          $Q_dist[0][$n] -= $Q_dist[$i][$n];
       }
   # }  All service centers are the same. 
    # Get Utilization and Queue depth of disks. 
    $U = $S0 * ( $X / $K);
    $Q = $RT * ($Nvisits * $X / $K);
    if (( $every ) && !( $n % $every )) {
       printf ("%d, %d, %.1f, %.1f, %.1f, %.1f, \n",
            $n, $X, $RT * 1000, $S0 * 1000, $U * 100, $Q );
    }
}
if ($nice_output) {
   printf ("N = \t%d\n", $Nusers); 
   printf ("X = \t%d\n", $X); 
   printf ("R = \t%.1f\n", $RT * 1000); 
   printf ("S = \t%.1f\n", $S0 * 1000); 
   printf ("U = \t%.1f\n", $U * 100); 
   printf ("Q = \t%.1f\n", $Q); 
}
else {
   if ( (($every) && ( $Nusers % $every )) || !$every ){ 
      printf ("%d, %d, %.1f, %.1f, %.1f, %.1f, \n", 
            $Nusers, $X, $RT * 1000, $S0 * 1000, $U * 100, $Q );
   }
}

sub usage {
   print <<EOM;
psf_qnm_fesc.pl [-FhHn] 
                [-[1..8] <svct at this qdepth>]
                [-D <#data disks>]
                [-E <N>]
                [-N <#customers or system qdepth>]
                [-M <max per disk qdepth>]
                [-Q <qdepth of from other workloads>]
                [-S <percent of the disk to use for short-stroking>]
                [-V <visits per customer>]

A Load Dependent QNM for disk arrays. Models 10K RPM disks of a certain maker 
with average seek times of 4.9 msecs for reads. Models load dependent service
times as a function of queue depth. We assume that tagged queued requests are
always uniformly distributed over the disk and the disk always chooses the 
closest one to go to. Also can model "short-stroking" with the -S option. This 
effectively reduces the number of cylinders used. And can model 15K RPM disks. 

    Arguments and switches - 
       -1 <msecs   Disk service time when qlen = 1.
       -2 <msecs   Disk service time when qlen = 2.
        ...
       -8 <msecs   Disk service time when qlen = 8.
       -D <#disks> Num Disks. 
       -E <N>      Report results every N users. 
       -F          Use 15K RPM disks. Default is 10K. 
       -h          This message.
       -H          Print header line. 
       -n          Nice output instead of comma separated values.
       -N <#cust>  Num customers. 
       -M <qdepth> Max per disk queue depth.
       -Q <qdepth> Avg qdepth per disk put there by other workloads. 
                   This is a cheap way of modeling multi-class workloads
                   Note that the Utilization & Qdepth reported do not
                   include these other guys. 
       -S <percent>Percent of the disks to use if "short-stroking". 
       -V <visits> Avg visits per customer.
EOM
exit;

}
