#!/usr/bin/env perl 

# This script is provided "as is" it is
# not warranted but may be supported on a limited basis
# by Network Appliance.
#
# psf_recycle_q.pl
#
#   PerfStatFilter for waf_susp recycle queue stats.
#
#   v0.01  11-30-2005 : Initial implementation.
#

# EXPORTED CONSTANTS (Not really, you have to copy them)

use constant _NA_TIMESTAMP             => 0;
use constant _NA_NORMALQ_SCAV          => 1;
use constant _NA_NORMALQ_VSCAV         => 2;
use constant _NA_FLUSHQ_SCAV           => 3;
use constant _NA_FLUSHQ_VSCAV          => 4;
use constant _NA_NORMALQ_NOMAP_SCAV    => 5;
use constant _NA_FLUSHQ_NOMAP_SCAV     => 6;
use constant _NA_IBUFQ_SCAV            => 7;
use constant _NA_IBUFQ_VSCAV           => 8;
use constant _NA_NORMALQ_NUM_BUFS      => 9;
use constant _NA_FLUSHQ_NUM_BUFS       => 10;
use constant _NA_NORMALQ_NOMAP_NUM_BUFS =>  11;
use constant _NA_FLUSHQ_NOMAP_NUM_BUFS =>  12;
use constant _NA_IBUFQ_NUM_BUFS        => 13;
use constant _NA_EMPTY_LIST_NUM_BUFS   => 14;
use constant _NA_FREE_BDATA_NUM_BUFS   => 15;
use constant _NA_NORMALQ_MSECS         => 16; 
use constant _NA_FLUSHQ_MSECS          => 17;
use constant _NA_NORMALQ_NOMAP_MSECS   => 18;
use constant _NA_FLUSHQ_NOMAP_MSECS    => 19;
use constant _NA_IBUFQ_MSECS           => 20;
use constant _NA_INTERVAL_SECS         => 21;


# Imported
use constant    _NA_VERS_PERFSTAT_MAJOR => 0;
use constant    _NA_VERS_PERFSTAT_MINOR => 1;
use constant    _NA_VERS_ONTAP_VERSION  => 2;
use constant    _NA_VERS_ONTAP_MAJOR    => 3;
use constant    _NA_VERS_ONTAP_MINOR    => 4;
use constant    _NA_VERS_FILER_MODEL    => 5;
use constant    _NA_VERS_FILER_MAGIC1   => 6;
use constant    _NA_VERS_FILER_MAGIC2   => 7;
use constant    _NA_VERS_PERFSTAT_PLATFORM => 8;

# Internal 

use constant FIRST_DATA                => 1;
use constant LAST_DATA                 => 21;


use Getopt::Std;
getopts("dDf:hHnT:");

if ($opt_h) { usage(); }

if ($opt_d) {
   $debug = 1;
} else {
   $debug = 0;
}

if ($opt_D) {
   $data_plus_header = 1;
} else {
   $data_plus_header = 0;
}

if ($opt_f) {
   $filer = $opt_f;
}

if ($opt_H) {
   $header = 1;
} else {
   $header = 0;
}

if ($opt_T) {
   $timestamp  = $opt_T;
} 

if ($opt_n) {
   $nice_output = 1;
} else {
   $nice_output = 0;
}


$data[_NA_NORMALQ_SCAV]          = \$normalq_scav;
$data[_NA_NORMALQ_VSCAV]         = \$normalq_vscav;
$data[_NA_FLUSHQ_SCAV]           = \$flushq_scav;
$data[_NA_FLUSHQ_VSCAV]          = \$flushq_vscav;
$data[_NA_NORMALQ_NOMAP_SCAV]    = \$normalq_nomap_scav;
$data[_NA_FLUSHQ_NOMAP_SCAV]     = \$flushq_nomap_scav;
$data[_NA_IBUFQ_SCAV]            = \$ibufq_scav;
$data[_NA_IBUFQ_VSCAV]           = \$ibufq_vscav;
$data[_NA_NORMALQ_NUM_BUFS]      = \$normalq_num_bufs; 
$data[_NA_FLUSHQ_NUM_BUFS]       = \$flushq_num_bufs;
$data[_NA_NORMALQ_NOMAP_NUM_BUFS] = \$normalq_nomap_num_bufs;
$data[_NA_FLUSHQ_NOMAP_NUM_BUFS] = \$flushq_nomap_num_bufs;
$data[_NA_IBUFQ_NUM_BUFS]        = \$ibufq_num_bufs;
$data[_NA_EMPTY_LIST_NUM_BUFS]   = \$empty_list_num_bufs;
$data[_NA_FREE_BDATA_NUM_BUFS]   = \$free_bdata_num_bufs;
$data[_NA_NORMALQ_MSECS]         = \$normalq_msecs;
$data[_NA_FLUSHQ_MSECS]          = \$flushq_msecs;
$data[_NA_NORMALQ_NOMAP_MSECS]   = \$normalq_nomap_msecs;
$data[_NA_FLUSHQ_NOMAP_MSECS]    = \$flushq_nomap_msecs;
$data[_NA_IBUFQ_MSECS]           = \$ibufq_msecs;
$data[_NA_INTERVAL_SECS]         = \$seconds;

$header_line = "Timestamp,      normalq_scav, normalq_vscav, flushq_scav, flushq_vscav, normalq_nomap_scav, flushq_nomap_scav, ibufq_scav, ibufq_vscav, normalq_num_bufs, flushq_num_bufs, normalq_nomap_num_bufs, flushq_nomap_num_bufs, ibufq_num_bufs, empty_list_num_bufs, free_bdata_num_bufs, normalq_msecs, flushq_msecs, normalq_nomap_msecs, flushq_nomap_msecs, ibufq_msecs, secs,";

if ( !$nice_output && $header ) {
   printf ("%s \n", $header_line);
   if ( !$data_plus_header ) {
      exit;
   }
}

$perfstat_file = $ARGV[0];
open(INPUTA, '<' . $perfstat_file) || die 'Unable to open perfstat file.';

$seconds = 0;
$found_wafl_stats = 0;

# Get to POSTSTATs 
while ($Line = <INPUTA>) {
   if ($Line =~ /POSTSTAT/ ) {  
      last ;
   }
}

if (!$timestamp) {
   if ( $filer ) {
      $timestamp = `psf_filer_timestamp.pl -f $filer $perfstat_file`;
   }
   else {
      $timestamp = `psf_filer_timestamp.pl $perfstat_file`;
   }
}
chop $timestamp;

$done = 0;
while ($Line = <INPUTA>) {
   if ( $done_wafl_susp ) {
      last;
   }
   if ( ( $Line =~ /^=-=-/ ) &&
           ( $Line =~ /POSTSTATS/ ) &&
           ( $Line =~ /wafl_susp -w/ ) ) {
      @line = split (/\s+/, $Line);
      $curr_filer = $line[2];
      if ( $debug ) {
         printf ("Current statit filer : %s \n", $curr_filer);
      }
   }
   elsif ( ($Line =~ /WAFL statistics over / )  &&
           ( (!$filer) ||
             ($filer =~ /^$curr_filer/) ) ) {
      if ( $debug ) { print "Found wafl_susp \n"; }
      $found_wafl_stats = 1;
      @line = split (/ +/, $Line);
      $seconds = $line[3];
      while ($Line = <INPUTA>) { 
               if ($Line =~ /POSTSTAT/ ) {
                  $done_wafl_susp = 1;
                  last;
               }
               if ($Line =~ /^normalq\s+/ ){
                  @line = split (/\s+/, $Line);
                  $normalq_num_bufs = $line[1];
                  $normalq_scav     = $line[2];
                  $normalq_vscav    = $line[3];
#                  $normalq_msecs    = $line[4] / ($normalq_scav + $normalq_vscav) ;
		  $normalq_scav     /= $seconds;
		  $normalq_vscav    /= $seconds;
                  if ($debug) {
			printf ("normalq scav per sec  : %d \n", $normalq_scav);
			printf ("normalq vscav per sec : %d \n", $normalq_vscav);
			printf ("normalq num_bufs      : %d \n", $normalq_num_bufs);
			printf ("normalq msecs         : %d \n", $normalq_msecs);
		  }
                  next;
               }
               elsif ($Line =~ /^flushq\s+/ ){
                  @line = split (/ +/, $Line);
                  $flushq_num_bufs = $line[1];
                  $flushq_scav     = $line[2];
                  $flushq_vscav    = $line[3];
#                  $flushq_msecs    = $line[4] / ($flushq_scav + $flushq_vscav) ;
		  $flushq_scav     /= $seconds;
		  $flushq_vscav    /= $seconds;
                  if ($debug) {
			printf ("flushq scav per sec  : %d \n", $flushq_scav);
			printf ("flushq vscav per sec : %d \n", $flushq_vscav);
			printf ("flushq num_bufs      : %d \n", $flushq_num_bufs);
			printf ("flushq msecs         : %d \n", $flushq_msecs);
		  }
                  next;
               }
               elsif ($Line =~ /^normalq_nomap/ ){
                  @line = split (/ +/, $Line);
                  $normalq_nomap_num_bufs = $line[1];
                  $normalq_nomap_scav     = $line[2];
#                  $normalq_nomap_msecs    = $line[4] / ($normalq_scav) ;
		  $normalq_nomap_scav     /= $seconds;
                  if ($debug) {
			printf ("normalq_nomap scav per sec  : %d \n", $normalq_nomap_scav);
			printf ("normalq_nomap num_bufs      : %d \n", $normalq_nomap_num_bufs);
			printf ("normalq_nomap msecs         : %d \n", $normalq_nomap_msecs);
		  }
                  next;
               }
               elsif ($Line =~ /^flushq_nomap/ ){
                  @line = split (/ +/, $Line);
                  $flushq_nomap_num_bufs = $line[1];
                  $flushq_nomap_scav     = $line[2];
                  $flushq_nomap_vscav    = $line[3];
#                  $flushq_nomap_msecs    = $line[4] / ($flushq_scav) ;
		  $flushq_nomap_scav     /= $seconds;
                  if ($debug) {
			printf ("flushq_nomap scav per sec  : %d \n", $flushq_nomap_scav);
			printf ("flushq_nomap num_bufs      : %d \n", $flushq_nomap_num_bufs);
			printf ("flushq_nomap msecs         : %d \n", $flushq_nomap_msecs);
		  }
                  next;
               }
               elsif ($Line =~ /^ibufq/ ){
                  @line = split (/ +/, $Line);
                  $ibufq_num_bufs = $line[1];
                  $ibufq_scav     = $line[2];
                  $ibufq_vscav    = $line[3];
#                  $ibufq_msecs    = $line[4] / ($ibufq_scav + $ibufq_vscav) ;
		  $ibufq_scav     /= $seconds;
		  $ibufq_vscav    /= $seconds;
                  if ($debug) {
			printf ("ibufq scav per sec  : %d \n", $ibufq_scav);
			printf ("ibufq vscav per sec : %d \n", $ibufq_vscav);
			printf ("ibufq num_bufs      : %d \n", $ibufq_num_bufs);
			printf ("ibufq msecs         : %d \n", $ibufq_msecs);
		  }
                  next;
               }
               elsif ($Line =~ /^empty list/ ){
                  @line = split (/ +/, $Line);
                  $empty_listnum_bufs = $line[1];
                  next;
               }
               elsif ($Line =~ /^free bdata/ ){
                  @line = split (/ +/, $Line);
                  $freee_bdata_num_bufs = $line[1];
                  next;
               }
	}
   }
}
if ( $found_wafl_stats ) {

      printf ("%s, ", $timestamp);
      for ( $i=FIRST_DATA; $i <= LAST_DATA; $i++ ) {
         printf ("%.1f,    ", ${$data[$i]} );
      }
      printf ("\n");
}


sub usage {
   print <<EOM;
psf_recycle_q.pl [-dDHh] [-f <filer>] <perfstat filename>

Summarizes the wafl recycle queue stats from perfstat outpt files.

    Arguments and switches -
       -d          Print debug messages.
       -D          Print data as well as a header. Only relevent if -H specified. 
       -f <filer>  For multi-filer perfstats choose this one.  
       -H          Print a header line only unless -D specified. 
       -h          This message.
EOM
exit;

}

