#!/usr/bin/env perl 

# Due to the huge infrastructure around including/exporting Perl constants a,
# I am defining these constsnt for the field definitions returned by sub perl scripts 
# explicitly in each source file that needs them, 

use constant   _NA_TIMESTAMP          => 0;
use constant   _NA_SAN_VOL_NAME       => 1;
use constant   _NA_SAN_LUN_RIOPS      => 2;
use constant   _NA_SAN_LUN_WIOPS      => 3;
use constant   _NA_SAN_LUN_RKBS       => 4; 
use constant   _NA_SAN_LUN_WKBS       => 5;
use constant   _NA_SAN_DISK_RIOPS     => 6;
use constant   _NA_SAN_DISK_WIOPS     => 7;
use constant   _NA_SAN_DISK_RKBS      => 8;
use constant   _NA_SAN_DISK_WKBS      => 9;
use constant   _NA_SAN_DISK_OTHER_RIOPS => 10;
use constant   _NA_SAN_DISK_GWIOPS    => 11;
use constant   _NA_SAN_LUN_RESP_TIME  => 12;
use constant   _NA_SAN_DISK_RESP_TIME => 13;
use constant   _NA_SAN_DISK_SVC_TIME  => 14;
use constant   _NA_SAN_LUN_Q_LEN      => 15;
use constant   _NA_SAN_VOL_NDISKS     => 16;
use constant   _NA_SAN_DISK_Q_LEN     => 17;
use constant   _NA_SAN_DISK_Q_BUSY    => 18;
use constant   _NA_SAN_DISK_UTIL      => 19;

# from psf_vol.pl
use constant   _NA_TIMESTAMP          => 0;
use constant   _NA_VOL_NAME           => 1;
use constant   _NA_VOL_RG             => 2;
use constant   _NA_VOL_UT             => 3;
use constant   _NA_VOL_XFERS          => 4; # Note this is avg per disk by default
use constant   _NA_VOL_UREADS         => 5;
use constant   _NA_VOL_UREAD_CHAIN    => 6;
use constant   _NA_VOL_UREAD_USECS    => 7;
use constant   _NA_VOL_UWRITES        => 8;
use constant   _NA_VOL_UWRITE_CHAIN   => 9;
use constant   _NA_VOL_UWRITE_USECS   => 10;
use constant   _NA_VOL_CPREADS        => 11;
use constant   _NA_VOL_CPREAD_CHAIN   => 12;
use constant   _NA_VOL_CPREAD_USECS   => 13;
use constant   _NA_VOL_GREADS         => 14;
use constant   _NA_VOL_GREAD_CHAIN    => 15;
use constant   _NA_VOL_GREAD_USECS    => 16;
use constant   _NA_VOL_GWRITES        => 17;
use constant   _NA_VOL_GWRITE_CHAIN   => 18;
use constant   _NA_VOL_GWRITE_USECS   => 19;
use constant   _NA_VOL_SVCTIME        => 20;
use constant   _NA_VOL_RESPONSE_TIME  => 21;
use constant   _NA_VOL_Q_LEN          => 22;
use constant   _NA_VOL_Q_BUSY         => 23;
use constant   _NA_VOL_NUM_DISKS      => 24; # Num data disks

# from psf_lun.pl 
use constant    _NA_LUN_TIMESTAMP       => 0;
use constant    _NA_LUN_NAME            => 1;
use constant    _NA_LUN_RIOPS           => 2;
use constant    _NA_LUN_WIOPS           => 3;
use constant    _NA_LUN_OIOPS           => 4;
use constant    _NA_LUN_RKBS            => 5;
use constant    _NA_LUN_WKBS            => 6;
use constant    _NA_LUN_RT              => 7;
use constant    _NA_LUN_QLEN            => 8;
use constant    _NA_LUN_QFULL           => 9;
use constant    _NA_LUN_PRTNR_IOPS      => 10;
use constant    _NA_LUN_PRTNR_KBS       => 11;

# from psf_versions.pl
use constant    _NA_VERS_PERFSTAT_MAJOR => 0;
use constant    _NA_VERS_PERFSTAT_MINOR => 1;
use constant    _NA_VERS_ONTAP_VERSION  => 2;
use constant    _NA_VERS_ONTAP_MAJOR    => 3;
use constant    _NA_VERS_ONTAP_MINOR    => 4;
use constant    _NA_VERS_FILER_MODEL    => 5;
use constant    _NA_VERS_FILER_MAGIC1   => 6;
use constant    _NA_VERS_FILER_MAGIC2   => 7;

$nice_output = 0;

use Getopt::Std;
getopts("dDhH");

if ($opt_h) { usage(); }

if ($opt_d) {
   $debug = 1;
} else {
   $debug = 0;
}

if ($opt_D) {
   $data_plus_header = 1;
   $nice_output = 0;
} else {
   $data_plus_header = 0;
}

if ($opt_H) {
   $header = 1;
   $nice_output = 0;
} else {
   $header = 0;
}

$header_line1 = "Timestamp, Volume, lun_riops, lun_wiops, lun_rKBs, lun_wKBs, ";
$header_line2 = "Dsk_riops, Dsk_wiops, Dsk_rKBs, Dsk_wKbs, Dsk_Oriops, Dsk_Gwiops, ";
$header_line3 = "Rlun, Rdsk, Sdsk, Qlun, Ndsks, Qdsk, Qbusydsk, Udsk, ";

if (!$nice_output && $header) {
   print $header_line1;
   print $header_line2;
   print $header_line3;
   printf ("\n");
   if (!$data_plus_header) {
     exit;
   }
}

# Open files
$perfstat_file = $ARGV[0];
open(INPUTA, '<' . $perfstat_file) || die 'Unable to open perfstat file.';

%lun_vol;
%non_lun_vol;
@vol_stats = `psf_vol.pl -t -S $perfstat_file`;  # This will give transfers per volume
@disk_stats = `psf_vol.pl -S -X $perfstat_file`; # This will give transfers per disk
@lun_stats = `psf_lun.pl -S $perfstat_file`;
$version_info = `psf_versions.pl $perfstat_file`;
@line = split (/,/, $version_info);
$ontap_version = @line[_NA_VERS_ONTAP_VERSION];
$ontap_major =  @line[_NA_VERS_ONTAP_MAJOR];
$ontap_minor =  @line[_NA_VERS_ONTAP_MINOR];
$model = @line[_NA_VERS_FILER_MODEL];
if ($nice_output) {
   printf ("Filer Model : %s   Data ONTAP Version : %s \n",
         $model, $ontap_version );
}
$perfstat_major =  @line[_NA_VERS_PERFSTAT_MAJOR];
$perfstat_minor =  @line[_NA_VERS_PERFSTAT_MINOR];

 # First do lun stats to find volumes with luns
 # Previously I used psf_lun_info.pl, but that config stuff is not kept for iscsi yet. 
 foreach $lun_line (@lun_stats) {
    @lun_info = split(/,/, $lun_line);
    $temp_vol = $lun_info[_NA_LUN_NAME];   # actually a volume name in this case
    if ( !$lun_vol{$temp_vol} ) {
       if ($debug) { printf ("New LUN vol : %s ", $temp_vol); }
       $volid++;
       $lun_vol{$temp_vol} = $volid;
       if ($debug) { printf (" volid : %d \n", $lun_vol{$temp_vol}); }
    }
    $volid = $lun_vol{$temp_vol};
    if ( $debug ) { printf ("vol_name : %s \n", $temp_vol); }
    if ( $debug ) { printf ("volid : %d \n", $lun_vol{$temp_vol}); }
    if ( $debug ) { print @lun_info ; }

   # $timestamp[$volid] = $lun_info[_NA_LUN_TIMESTAMP];
    $lun_riops[$volid] = $lun_info[_NA_LUN_RIOPS];
    $lun_wiops[$volid] = $lun_info[_NA_LUN_WIOPS];
    $lun_oiops[$volid] = $lun_info[_NA_LUN_OIOPS];
    $lun_rkbs[$volid]  = $lun_info[_NA_LUN_RKBS];
    $lun_wkbs[$volid]  = $lun_info[_NA_LUN_WKBS];
    $lun_rt[$volid]    = $lun_info[_NA_LUN_RT];
    $lun_qlen[$volid]  = $lun_info[_NA_LUN_QLEN];
    $lun_qfull[$volid]  = $lun_info[_NA_LUN_QFULL];
    $lun_prtnr_iops[$volid]  = $lun_info[_NA_LUN_PRTNR_IOPS];
    $lun_prtnr_kbs[$volid]  = $lun_info[_NA_LUN_PRTNR_KBS];
 }
 
 # Get average disk stats across volumes 
 foreach $disk_line (@disk_stats) {
    @disk_info = split (/,/, $disk_line);
    $temp_vol = $disk_info[_NA_VOL_NAME];
    $volid = $lun_vol{$temp_vol};
    if ( !$volid ) {
       next;
    }
    
    $vol[$volid] = $disk_info[_NA_VOL_NAME];
    $ut[$volid]  = $disk_info[_NA_VOL_UT]; 
    $xfers[$volid] = $disk_info[_NA_VOL_XFERS];
    $reads[$volid] = $disk_info[_NA_VOL_UREADS];
    $chain[$volid] = $disk_info[_NA_VOL_UREAD_CHAIN];
    $usecs[$volid] = $disk_info[_NA_VOL_UREAD_USECS];
    $writes[$volid] = $disk_info[_NA_VOL_UWRITES];
    $write_chain[$volid] = $disk_info[_NA_VOL_UWRITE_CHAIN];
    $write_usecs[$volid] = $disk_info[_NA_VOL_UWRITE_USECS];
    $cpreads[$volid] = $disk_info[_NA_VOL_CPREADS];
    $greads[$volid] = $disk_info[_NA_VOL_GREADS];
    $gwrites[$volid] = $disk_info[_NA_VOL_GWRITES];
    $read_rt[$volid]  = ($chain[$volid] * $usecs[$volid])/1000;
    $write_rt[$volid] = ($write_chain[$volid] * $write_usecs[$volid]) / 1000;
    $svct[$volid] = $disk_info[_NA_VOL_SVCTIME]; 
    $rt[$volid] = $disk_info[_NA_VOL_RESPONSE_TIME];
    $qlen[$volid] = $disk_info[_NA_VOL_Q_LEN];
    $qlen_busy[$volid] = $disk_info[_NA_VOL_Q_BUSY];
    $ndisks[$volid] = $disk_info[_NA_VOL_NUM_DISKS];
 } 

 # Now get per volume totals
 foreach $disk_line (@vol_stats) {
    @disk_info = split (/,/, $disk_line);
    $temp_vol = $disk_info[_NA_VOL_NAME];
    $volid = $lun_vol{$temp_vol};
    if ( !$volid ) {
       next
    }

    $timestamp[$volid] = $disk_info[_NA_TIMESTAMP];
    $vol_xfers[$volid] = $disk_info[_NA_VOL_XFERS];
    $vol_ureads[$volid] = $disk_info[_NA_VOL_UREADS];
    $vol_writes[$volid] = $disk_info[_NA_VOL_UWRITES];
    $vol_cpreads[$volid] = $disk_info[_NA_VOL_CPREADS];
    $vol_greads[$volid] = $disk_info[_NA_VOL_GREADS];
    $vol_gwrites[$volid] = $disk_info[_NA_VOL_GWRITES];
    $other_reads[$volid] =  $vol_cpreads[$volid] + $vol_greads[$volid];
    $other_writes[$volid] = $vol_gwrites[$volid];
 }
 
 # Now finally consolidate vol, disk, and lun data
 foreach $vol_name (keys %lun_vol) {
    $volid = $lun_vol{$vol_name};
    printf ("%s, %s, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %.1f, %.1f, %.1f, %.1f, %d, %.1f, %.1f, %.1f, \n",
           $timestamp[$volid], $vol_name,
           $lun_riops[$volid], $lun_wiops[$volid], $lun_rkbs[$volid], $lun_wkbs[$volid],
           $vol_ureads[$volid], $vol_writes[$volid], 
           ($chain[$volid]*$vol_ureads[$volid]*4),
           ($write_chain[$volid]*$vol_writes[$volid]*4),
           $other_reads[$volid], $other_writes[$volid],
           $lun_rt[$volid], $rt[$volid], $svct[$volid],
           $lun_qlen[$volid], $ndisks[$volid], $qlen[$volid], $qlen_busy[$volid],
           $ut[$volid]  );
 }

# ===================
sub usage {
   print <<EOM;
psf_san_vol.pl [-DHhX:]  <perfstat filename>

Attempt to determine impact of BURT 103555 on performance of a system based on a 
perfstat file. 

    Arguments and switches - 
       -D          Print data as well as header. Only 
                   relevent if -H  and -c specified.
       -H          Print a header line only unless -D specified. Only
                   relevant if -c specified.
       -h          This message.
       -X <1..3>   Print more details. 
EOM
exit;

}

