#!/usr/bin/env perl 

# This script is provided "as is" it is
# not warranted but may be supported on a limited basis
# by Network Appliance.
#
# psf_split_iterations.pl
#
#   PerfStatFilter for splitting iterations of perfstat 5.32+ files 
#
#   v0.03  12-20-2004 : Fixed bug where perfstat files without config info 
#                       would result in one output file regardless of how many
#                       iterations. 
#   v0.02  11-09-2004 : Changed format of perfstat.n.txt files to always 
#                       have 4 digits of numbers. This is so sort will return them
#                       in the correct time-sequenced order, hopefully 
#			reducing costs of SORTs in Excel and possibly eliminating
#			some of them. 
#   v0.01  06-30-2004 : Initial implementation.
#


use Getopt::Std;
getopts("b:df:");

if ($opt_b) {
   $base = $opt_b;
} else {
   $base = 0;
}

if ($opt_h) { usage(); }

if ($opt_f) {
   $file_root = $opt_f;; 
} 

if ($opt_d) {
   $debug = 1; 
} else {
   $debug = 0;
}

$perfstat_file = $ARGV[0];
$version_line = `psf_versions.pl $perfstat_file`;
@versions = split (/,/, $version_line);
$perfstat_major = $versions[_NA_VERS_PERFSTAT_MAJOR];
$perfstat_minor = $versions[_NA_VERS_PERFSTAT_MINOR];
$perfstat_platform = $versions[_NA_VERS_PERFSTAT_PLATFORM];
$ontap_major = $versions[_NA_VERS_ONTAP_MAJOR];
$ontap_minor = $versions[_NA_VERS_ONTAP_MINOR];
$model = $versions[_NA_VERS_FILER_MODEL];
$perfstat_samples = $versions[_NA_VERS_NUM_SAMPLES];
$perfstat_interval = $versions[_NA_VERS_SECS_PER_SAMPLE];

open(INPUTA, '<' . $perfstat_file) || die 'Unable to open perfstat file (arg1) .';

$samples=0;
while ($Line = <INPUTA>) { 
   if ($Line =~ / Perfstat/ ) {    # save header stuff  
      push (@header, $Line);
      while ($Line = <INPUTA>) {
         if ($Line =~ / BEGIN Iteration/) {
	    if ($debug) {
		printf ("BEGIN Iteration found\n");
	    }
            last;
         } 
	 elsif ($Line =~ /^=-=-=-=/ ) {
            last;
         }
         else {
            push (@header, $Line);
        }
      }
    }
    if ($Line =~ / BEGIN Iteration/) {
      if ($Line =~ / echo/) {
         next;
      }
      close (OUTPUTA);
      @line = split (/\s+/, $Line);
      $iteration = $line[3];
      $outfile = sprintf ("perfstat.%04d.txt",$iteration + $base);
      open(OUTPUTA, '>'. $outfile) || die 'Cant open output ';
      foreach $hdr_line (@header) {
         printf (OUTPUTA "%s", $hdr_line);
      }
      if ($iteration == 1) {
         # Get sysconfig info 
         $done_sysconfig = 0;
         while ($Line = <INPUTA>) {
           if (($Line =~ /=-= sysconfig -av/ ) ||
               ($Line =~ /=-= vol status -v/ )) {
		    if ($debug) {
			printf ("Begin sysconfig found\n");
		    }
              push (@sysconfig, $Line);
              while ($Line = <INPUTA>) {
		 chomp $Line; 
                 push (@sysconfig, $Line);
                 if ($Line =~ /^=-=-=-=/) {
                    $done_sysconfig++ ;
		    if ($debug) {
			printf ("End sysconfig found\n");
		    }
                    last;
                 }
              }
              if ($done_sysconfig > 1) { last;}
           }
	   elsif ($Line =~ /END Iteration/) {
	      # This means we did not have any configuration info.
		goto NextIteration;
	   }
           else {
              printf (OUTPUTA "%s",$Line);
           }
         }
      }
      # print sysconfig info
      foreach $sysconf_line (@sysconfig) {
         printf (OUTPUTA "%s\n", $sysconf_line);
      }

      while ($Line = <INPUTA>) {
         if ($Line =~ /END Iteration/) {
	    if ($debug) {
		printf ("END Iteration found\n");
	    }
            last;
         }
         elsif (($Line =~ /PRESTATS =-=-/ ) &&
                ($Line =~ /config show/))  {
	    if ($debug) {
		printf ("PRESTATS found \n");
	    }
            # Skip all this stuff until you find another section marker
            while ($Line = <INPUTA>) {
                if ($Line =~ /^=-=-=-=/ ) {
	    		if ($debug) {
				printf ("End PRESTATS found \n");
	   		 }
                   	last;
                }
            }
         }
        printf (OUTPUTA "%s",$Line);
      }
NextIteration:
    }
}   
