#!/usr/bin/env perl 

# This script is provided "as is" it is
# not warranted but may be supported on a limited basis
# by Network Appliance.
#
# psf_statit.pl
#
#   PerfStatFilter for certain stats from statit, not 
#        including disk statistics which are provided 
#        through psf_vol.pl  
#	
#	10-16-2006 - Added column for cifs domain.
#	   03-16-2006 : Fixed bug where timestamps did not have leading zeros in
#		        month and day. Caused problems in sorting. 
#	   12-22-2004 : Added a -1 option to only take the first sample.
#			This is for multi-iteraiton perfstat files with 
#			a ITER_INTERVAL where most of the time is not
#			being measured, and what is measured is confused
#			because of the ways stutter statit works. 
#   v0.03  03-19-2004 : Added -f <filer> for multi-filer perfstat files.
#                       Only tested against perfstat 5.20 Linux version.
#   v0.02  01-20-2004 : Fixed bug where domains utils were always 0 if
#                       not using -S option.
#   v0.01  11-04-2003 : Initial implementation.
#

# EXPORTED CONSTANTS (Not really, you have to copy them)

use constant _NA_TIMESTAMP              => 0;
use constant _NA_CPU_UTIL               => 1;
use constant _NA_CP_TIME                => 2;
use constant _NA_NFS_IOPS               => 3;
use constant _NA_CIFS_IOPS              => 4;
use constant _NA_FCP_IOPS               => 5;
use constant _NA_ISCSI_IOPS             => 6;
use constant _NA_DAFS_IOPS              => 7;
use constant _NA_NET_KB_RECV            => 8;
use constant _NA_DISK_KB_WRITTEN        => 9;
use constant _NA_FCP_KB_RECV            => 10;
use constant _NA_ISCSI_KB_RECV          => 11;
use constant _NA_NET_KB_SENT            => 12;
use constant _NA_DISK_KB_READ           => 13;
use constant _NA_FCP_KB_SENT            => 14;
use constant _NA_ISCSI_KB_SENT          => 15;
use constant _NA_KAHUNA_UTIL            => 16;
use constant _NA_NETWORK_UTIL           => 17;
use constant _NA_STORAGE_UTIL           => 18;
use constant _NA_EXEMPT_UTIL            => 19;
use constant _NA_RAID_UTIL              => 20;
use constant _NA_TARGET_UTIL            => 21;
use constant _NA_HARD_SW                => 22;
use constant _NA_WAFL_BUFS              => 23;
use constant _NA_AVG_STRIPE_SIZE        => 24;
use constant _NA_CIFS_UTIL              => 25;
use constant _NA_WAFL_EXEMPT_UTIL       => 26;
use constant _NA_WAFL_XCLEANER_UTIL     => 27;

use constant FIRST_DATA                 => _NA_CPU_UTIL;
use constant LAST_STD_DATA              => _NA_ISCSI_KB_SENT ;
use constant LAST_DATA                  => _NA_WAFL_XCLEANER_UTIL ;
use constant FIRST_OPS                  => _NA_NFS_IOPS;
use constant LAST_OPS                   => _NA_DAFS_IOPS ;
use constant FIRST_DOMAIN               => _NA_KAHUNA_UTIL  ;
use constant LAST_DOMAIN                => _NA_TARGET_UTIL  ;

use Getopt::Std;
getopts("1SdDf:hHnX");

if ($opt_h) { usage(); }

if ($opt_1) {
   $first_sample_only = 1;
} else {
   $first_sample_only = 0;
}

if ($opt_S) {
   $avg_all_samples = 1;
} else {
   $avg_all_samples = 0;
}

if ($opt_d) {
   $debug = 1;        
} else {
   $debug = 0;
}

if ($opt_D) {
   $data_plus_header = 1;        
} else {
   $data_plus_header = 0;
}

if ($opt_f) {
   $filer = $opt_f;        
} 

if ($opt_H) {
   $header = 1; 
} else {
   $header = 0;
}

if ($opt_X) {
   $extended_stats = 1; 
} else {
   $extended_stats = 0;
}

if ($opt_n) {
   $nice_output = 1;  
   $avg_all_samples = 1; 
} else {
   $nice_output = 0;
}

$sample=0;

@cpu_util, @cp_time, @kahuna, @network, @storage, @exempt, @raid, @target;
@cifs, @wafl_exempt, @wafl_xcleaner;
@nfs_iops, @cifs_iops, @fcp_iops, @iscsi_iops, @dafs_iops;
@net_KB_recv, @net_KB_sent;
@disk_KB_read, @disk_KB_written;
@fcp_KB_recv, @fcp_KB_sent,
@iscsi_KB_recv, @iscsi_KB_sent;
@hard_sw, @wafl_bufs, @avg_stripe_size;

$header_line = "Timestamp, CPU, CP, NfsOPS, CifsOPS, FcpOPS, IscsiOPS, dafs, NetKBsIn, DiskKBsWr, FcpKBsIn, IscsiKBsIn, NetKBsOut, DiskKBsRd, FcpKbsOut, IscsiKBsOut, Kahuna, Network, Storage, Exempt, Raid, Target, HardSw, Bufs, AvgStripeSz, Cifs, WaflExempt, WaflXCleaner,  \n";
 
$data[_NA_CPU_UTIL] 		= \@cpu_util;
$data[_NA_CP_TIME] 		= \@cp_time;
$data[_NA_NFS_IOPS]		= \@nfs_iops;
$data[_NA_CIFS_IOPS]		= \@cifs_iops;
$data[_NA_FCP_IOPS]		= \@fcp_iops;
$data[_NA_ISCSI_IOPS]		= \@iscsi_iops;
$data[_NA_DAFS_IOPS]		= \@dafs_iops;
$data[_NA_NET_KB_RECV]		= \@net_KB_recv;
$data[_NA_NET_KB_SENT]		= \@net_KB_sent;
$data[_NA_DISK_KB_READ]		= \@disk_KB_read;
$data[_NA_DISK_KB_WRITTEN]	= \@disk_KB_written;
$data[_NA_FCP_KB_RECV]		= \@fcp_KB_recv;
$data[_NA_FCP_KB_SENT]		= \@fcp_KB_sent;
$data[_NA_ISCSI_KB_RECV]	= \@iscsi_KB_recv;
$data[_NA_ISCSI_KB_SENT]	= \@iscsi_KB_sent;
$data[_NA_KAHUNA_UTIL]  	= \@kahuna;
$data[_NA_NETWORK_UTIL]		= \@network;
$data[_NA_STORAGE_UTIL]		= \@storage;
$date[_NA_EXEMPT_UTIL]		= \@exempt;
$data[_NA_RAID_UTIL]		= \@raid;
$data[_NA_TARGET_UTIL]		= \@target;
$data[_NA_HARD_SW]		= \@hard_sw;
$data[_NA_WAFL_BUFS]            = \@wafl_bufs; 
$data[_NA_AVG_STRIPE_SIZE]      = \@avg_stripe_size; 
$data[_NA_CIFS_UTIL]			= \@cifs;
$data[_NA_WAFL_EXEMPT_UTIL]		= \@wafl_exempt;
$data[_NA_WAFL_XCLEANER_UTIL]	= \@wafl_xcleaner;

$mp_filer = 1;   # Until we get the filer model and really see if it is.

if ( !$nice_output && $header ) {
   print $header_line;
   if (!$data_plus_header) {
     exit;
   }
}

$perfstat_file = $ARGV[0];
open(INPUTA, '<' . $perfstat_file) || die 'Unable to open perfstat file (arg1) .';

while ($Line = <INPUTA>) { 
   if ($Line =~ / Start time:/ ) {
      if (($first_sample_only) && 
          ($sample > 0)) {
          last;
      }
      @line = split (/ +/, $Line);
      $sample++;
      $potential_new_sample = 1;
      $real_new_sample = 0;
      $mo[$sample] = $line[4];
      $day[$sample] = $line[5];
      $time[$sample] = $line[6];
      $yr[$sample] = $line[8];
      next;
   }
   elsif ( ( $Line =~ /^=-=-/ ) &&
           ( $Line =~ /POSTSTATS/ ) &&
           ( $Line =~ /statit/ ) ) {
      @line = split (/\s+/, $Line);
      $curr_filer = $line[2];
      if ( $debug ) {
         printf ("Current statit filer : %s \n", $curr_filer);
      }
   }
   elsif ( ($Line =~ /CPU Statistics/ ) && 
           ( (!$filer) ||
             ($filer =~ /^$curr_filer/) ) ) {
      if ( $debug ) { print "Found statit \n"; }
      if ( $debug ) { print "Found CPU Statistics \n"; }
      $found_statit = 1;
      $Line =  <INPUTA>;
      $Line =  <INPUTA>;     # This line contains system time 
      @line = split (/ +/, $Line);
      $cpu_util[$sample] = @line[4];
      if ( $debug ) { printf ("cpu_util %.1f \n", $cpu_util[$sample]); }
      $Line =  <INPUTA>;     # This line contains rupt time 
      $Line =  <INPUTA>;     # This line contains non-rupt system time 
      $Line =  <INPUTA>;     # This line contains idle time 
      $Line =  <INPUTA>;     # This line is blank 
      $Line =  <INPUTA>;     # This line contains time in CP
      @line = split (/ +/, $Line);
      $cp_time[$sample] = @line[5];
      if ( $debug ) { printf ("cp_time %.1f \n", $cp_time[$sample]); }
      next;
   }
   elsif ( ($Line =~ / Multiprocessor Statistics \(per second/ ) &&
           ( (!$filer) ||
             ($filer =~ /^$curr_filer/) ) ) {
      $Line =  <INPUTA>;        # cpu0     cpu1      total
      if ($Line =~ /cpu3/) {
         $domain_col = 5;
      }
      else {
         $domain_col = 3;
      }
      $Line =  <INPUTA>;        # sk switches
      $Line =  <INPUTA>;        # hard switches
      chop $Line;
      @line = split (/ +/, $Line);
      $hard_sw[$sample] = $line[4];
      while ($Line = <INPUTA>) { 
	if ($Line =~ /^idle/) {
		last;
	}
      }
      $Line =  <INPUTA>;        		# kahuna
      chop $Line;
      @line = split (/ +/, $Line);
      $kahuna[$sample] = $line[$domain_col]/10000;
      $Line =  <INPUTA>;        		# network
      chop $Line;
      @line = split (/ +/, $Line);
      $network[$sample] = $line[$domain_col]/10000;
      $Line =  <INPUTA>;        		# storage
      chop $Line;
      @line = split (/ +/, $Line);
      $storage[$sample] = $line[$domain_col]/10000;
      $Line =  <INPUTA>;        		# exempt
      chop $Line;
      @line = split (/ +/, $Line);
      $exempt[$sample] = $line[$domain_col]/10000;
      $Line =  <INPUTA>;        		# raid 
      chop $Line;
      @line = split (/ +/, $Line);
      $raid[$sample] = $line[$domain_col]/10000;
      $Line =  <INPUTA>;        		# target 
      chop $Line;
      @line = split (/ +/, $Line);
      $target[$sample] = $line[$domain_col]/10000;
      $Line =  <INPUTA>;        		# netcache
	  if (!($Line =~ /netcache/) ) { next; } 
      $Line =  <INPUTA>;        		# netcache2
	  if (!($Line =~ /netcache2/) ) { next; } 
      $Line =  <INPUTA>;        		# cifs
	  if (!($Line =~ /cifs/) ) { next; } 
      chop $Line;
      @line = split (/ +/, $Line);
      $cifs[$sample] = $line[$domain_col]/10000;
      $Line =  <INPUTA>;        		# wafl_exempt
	if (!($Line =~ /wafl_exempt/) ) { next; } 
      chop $Line;
      @line = split (/ +/, $Line);
      $wafl_exempt[$sample] = $line[$domain_col]/10000;
      $Line =  <INPUTA>;        		# wafl_xcleaner
	if (!($Line =~ /wafl_xcleaner/) ) { next; } 
      chop $Line;
      @line = split (/ +/, $Line);
      $wafl_xcleaner[$sample] = $line[$domain_col]/10000;
      next;
   }
   elsif ( ($Line =~ / Miscellaneous Statistics \(per second/ )  &&
           ( (!$filer) ||
             ($filer =~ /^$curr_filer/) ) ) {
      $real_new_sample = 1;
      $potential_new_sample = 0;
      if ( $debug ) { print "Found Misc Stats \n"; }
      $Line =  <INPUTA>;     # This line contains NFS ops   
      @line = split (/ +/, $Line);
      $nfs_iops[$sample] = @line[5];
      $Line =  <INPUTA>;     # This line contains CIFS ops   
      @line = split (/ +/, $Line);
      $cifs_iops[$sample] = @line[1];
      $Line =  <INPUTA>;
      $Line =  <INPUTA>;     # This line contains network KB recv/sent   
      @line = split (/ +/, $Line);
      $net_KB_recv[$sample] = @line[1];
      $net_KB_sent[$sample] = @line[5];
      $Line =  <INPUTA>;     # This line contains disk KB read/ written  
      @line = split (/ +/, $Line);
      $disk_KB_read[$sample] = @line[1];
      $disk_KB_written[$sample] = @line[5];
      $Line =  <INPUTA>;
      $Line =  <INPUTA>;
      $Line =  <INPUTA>;     # This line contains DAFS ops  
      @line = split (/ +/, $Line);
      $dafs_iops[$sample] = @line[7];
      next;
   }
   elsif  ($Line =~ / Miscellaneous Statistics/) {
    # Was not a real sample . We saw a listing of statit data with absolute counts
    # instead of rates. Forget this as a sample.
    $sample--;
    if ($debug ) {
       printf ("Skipping sample. Resstting sample to %d\n", $sample);
    }
    next;
   }
   elsif ( ($Line =~ / WAFL Statistics \(per second/ )  &&
           ( (!$filer) ||
             ($filer =~ /^$curr_filer/) ) ) {
      while ($Line = <INPUTA>) {
         if ( $Line =~ /per second/ ) {
            last;
         }
         elsif ($Line =~ / buffers/ ) {
            @line = split (/\s+/, $Line);
            if ($Line =~ / IOWAIT suspends/) {
               $wafl_bufs[$sample] = @line[4];
            }
            else {
               $wafl_bufs[$sample] = @line[1];
            }
            last;
         }
      } 
   }
   elsif ( ($Line =~ / RAID Statistics \(per second/ )  &&
           ( (!$filer) ||
             ($filer =~ /^$curr_filer/) ) ) {
	while ($Line = <INPUTA>) {
		if ( $Line eq "" ) {
			last;
		}
         	elsif ($Line =~  /\s*(\d+\.?\d*) blocks written\s+ .* blocks read/){
			$blocks_written_per_sec = $1;
			$got_blocks_written = 1;
		}
		elsif ($Line =~ /slave tetrises\s+(\d+\.?\d*) stripes written/){
			$stripes_written_per_sec = $1;
			$got_stripes_written = 1;
		}
		if ($got_blocks_written && $got_stripes_written) {
			$avg_stripe_size[$sample] = $blocks_written_per_sec / $stripes_written_per_sec ;
			last;
		} 
	}
   }	
   elsif ( ($Line =~ /FCP Statistics \(per second/ )  &&
           ( (!$filer) ||
             ($filer =~ /^$curr_filer/) ) ) {
      $Line =  <INPUTA>;
      @line = split (/ +/, $Line);
      if ( $line[0] > 0 ) {
         $fcp_KB_recv[$sample] = $line[0]/1000;
         $fcp_KB_sent[$sample] = $line[4]/1000;
      }
      else {
         $fcp_KB_recv[$sample] = $line[1]/1000;
         $fcp_KB_sent[$sample] = $line[5]/1000;
      }
      $Line =  <INPUTA>;
      @line = split (/ +/, $Line);
      $fcp_iops[$sample] = @line[1];
      if ( $debug ) { printf ("Found FCP Stats KB sent %d \n", $fcp_KB_sent[sample]); }
      next;
   }
   elsif ( ($Line =~ /iSCSI Statistics \(per second/ ) &&
           ( (!$filer) ||
             ($filer =~ /^$curr_filer/) ) ) {
      $Line =  <INPUTA>;
      @line = split (/ +/, $Line);
      if ( $line[0] > 0 ) {
         $iscsi_KB_recv[$sample] = $line[0]/1000;
         $iscsi_KB_sent[$sample] = $line[4]/1000;
      }
      else {
         $iscsi_KB_recv[$sample] = $line[1]/1000;
         $iscsi_KB_sent[$sample] = $line[5]/1000;
      }
      $Line =  <INPUTA>;
      @line = split (/ +/, $Line);
      $iscsi_iops[$sample] = @line[1];
      if ( $debug ) { printf ("Found iSCSI Stats. iSCSI KB sent %d \n", $iscsi_KB_sent[sample]); }
      next;

   }
   elsif ($Line =~ / statit ---/ ){
      next; 
   }

}
if ( $debug ) { printf ("Num samples found : %d \n", $sample); }
# if ( $potential_new_sample ) { 
    # Was not a real sample . We saw a listing of statit data with absolute counts 
    # instead of rates. Forget this as a sample. 
#     $sample--;
# }
if ( $sample > 0 ) { 
   if  (!$avg_all_samples) {
       for ( $j=1; $j <= $sample; $j++ ) {
         $mo_num = month_to_numeric ($mo[$j]);
         $timestamp = sprintf ("%02d\/%02d\/%d %s", $mo_num, $day[$j], $yr[$j], $time[$j]);
         printf ("%s, ", $timestamp);
         for ( $i=FIRST_DATA; $i <= LAST_STD_DATA; $i++ ) {
            if ( $i <= _NA_CP_TIME) { 
               printf ("%3d,", $data[$i]->[$j]);
            }
            elsif ( ( $i >= FIRST_OPS) && ( $i <= LAST_OPS ) ) {
               printf ("%6d,", $data[$i]->[$j]);
            }
            else {
                printf ("%7d,", $data[$i]->[$j]);
            }
         }
         if ( $extended_stats && $mp_filer ) { 
            for ( $i=FIRST_DOMAIN; $i <= LAST_DOMAIN; $i++ ) {
               printf (" %4.1f,", $data[$i]->[$j]);
            }
            printf (" %4.1f,", $data[_NA_HARD_SW]->[$j]);
         }
         printf (" %d,", $data[_NA_WAFL_BUFS]->[$j]);
         printf (" %3.1f,", $data[_NA_AVG_STRIPE_SIZE]->[$j]);
         printf (" %4.1f,", $data[_NA_CIFS_UTIL]->[$j]);
         printf (" %4.1f,", $data[_NA_WAFL_EXEMPT_UTIL]->[$j]);
         printf (" %4.1f,", $data[_NA_WAFL_XCLEANER_UTIL]->[$j]);
         printf ("\n");
      }
   }
   else {
      average_statit_stats();
      print_statit_stats();
   }
}
else {
  print "No Samples found \n";
} 

# END OF MAIN 


sub print_statit_stats {

   if ( $nice_output ) {
             printf ("system time       : \t %.1f \n", $cpu_util[0]);
             printf ("CP time           : \t %.1f \n", $cp_time[0]);
             printf ("NFS  IOPS         : \t %.1f \n", $nfs_iops[0]);
             printf ("CIFS IOPS         : \t %.1f \n", $cifs_iops[0]);
             printf ("FCP  IOPS         : \t %.1f \n", $fcp_iops[0]);
             printf ("iSCSI IOPS        : \t %.1f \n", $iscsi_iops[0]);
             printf ("DAFS IOPS         : \t %.1f \n", $dafs_iops[0]);
             printf ("NET_KB_RECV       : \t %.1f \n", $net_KB_recv[0]);
             printf ("NET_KB_SENT       : \t %.1f \n", $net_KB_sent[0]);
             printf ("DISK_KB_READ      : \t %.1f \n", $disk_KB_read[0]);
             printf ("DISK_KB_WRITTEN   : \t %.1f \n", $disk_KB_written[0]);
             printf ("FCP_KB_RECV       : \t %.1f \n", $fcp_KB_recv[0]);
             printf ("FCP_KB_SENT       : \t %.1f \n", $fcp_KB_sent[0]);
             printf ("iSCSI_KB_RECV     : \t %.1f \n", $iscsi_KB_recv[0]);
             printf ("iSCSI_KB_SENT     : \t %.1f \n", $iscsi_KB_sent[0]);
             if ( $extended_stats && $mp_filer ) { 
                printf ("  kahuna          : \t %.1f \n", $kahuna[0]);
                printf ("  network         : \t %.1f \n", $network[0]);
                printf ("  storage         : \t %.1f \n", $storage[0]);
                printf ("  exempt          : \t %.1f \n", $exempt[0]);
                printf ("  raid            : \t %.1f \n", $raid[0]);
                printf ("  target          : \t %.1f \n", $target[0]);
             }
   }
   else {
             $mo_num = month_to_numeric ($mo[$sample]);
             $timestamp = sprintf ("%02d\/%02d\/%d %s", 
                           $mo_num, $day[$sample], 
                           $yr[$sample], $time[$sample]);
             printf ("%s, ", $timestamp);
             for ( $i=FIRST_DATA; $i <= LAST_STD_DATA; $i++ ) {
                if ( $i <= _NA_CP_TIME ) {
                   printf ("%3d,", $data[$i]->[0]);
                }
                elsif ( ( $i >= FIRST_OPS) && ( $i <= LAST_OPS ) ) {
                   printf ("%6d,", $data[$i]->[0]);
                }
                else {
                   printf ("%7d,", $data[$i]->[0]);
                }
             }
             if ( $extended_stats && $mp_filer ) { 
                for ( $i=FIRST_DOMAIN; $i <= LAST_DOMAIN; $i++ ) {
                   printf (" %4.1f,", $data[$i]->[0]);
                }
                printf (" %4.1f,", $data[_NA_HARD_SW]->[0]);
             }
			printf (" %d,", $data[_NA_WAFL_BUFS]->[0]);
         	printf (" %3.1f,", $data[_NA_AVG_STRIPE_SIZE]->[0]);
			printf (" %4.1f,", $data[_NA_CIFS_UTIL]->[0]);
			printf (" %4.1f,", $data[_NA_WAFL_EXEMPT_UTIL]->[0]);
			printf (" %4.1f,", $data[_NA_WAFL_XCLEANER_UTIL]->[0]);
             printf ("\n");
   }
}


sub average_statit_stats {
   my $i, $j;

   if ($debug) { print "In average_statit_stats \n"; }
   for ( $i=FIRST_DATA; $i <= LAST_DATA; $i++ ) {
      $data[$i]->[0] = 0;    # oth element used for averages
      for ( $j=1; $j <= $sample; $j++ ) {
         $data[$i]->[0] += $data[$i]->[$j];
      }   
      $data[$i]->[0] /= $sample; 
   }
}

sub month_to_numeric {
   local ($month) = @_ ;
   my $result;

   if ($month =~ /Jan/ ) {
        $result = 1;
   }
   elsif ($month =~ /Feb/ ) {
        $result = 2;
   }
   elsif ($month =~ /Mar/ ) {
        $result = 3;
   }
   elsif ($month =~ /Apr/ ) {
        $result = 4;
   }
   elsif ($month =~ /May/ ) {
        $result = 5;
   }
   elsif ($month =~ /Jun/ ) {
        $result = 6;
   }
   elsif ($month =~ /Jul/ ) {
        $result = 7;
   }
   elsif ($month =~ /Aug/ ) {
        $result = 8;
   }
   elsif ($month =~ /Sep/ ) {
        $result = 9;
   }
   elsif ($month =~ /Oct/ ) {
        $result = 10;
   }
   elsif ($month =~ /Nov/ ) {
        $result = 11;
   }
   elsif ($month =~ /Dec/ ) {
        $result = 12;
   }
   else {
        $result = -1;
   }
   return $result;
}

sub usage {
   print <<EOM;
psf_statit.pl [-DHhnSX] [-f <filer>]  <perfstat filename>

Summarizes the statit data from perfstat outpt files. 

    Arguments and switches - 
       -D          Print data as well as header. Only 
                   relevent if -H specified.
       -H          Print a header line only unless -D specified. 
       -h          This message.
       -f <filer>  For multi-filer perfstat files, indicates which filer.
       -n          Print Nice output, a value per line labeled with what it is.
       -S          Average (i.e. Summarize) all samples together.
       -X          Print extended stats. Currently this means domain
                   utilizations (iff the filer is an MP filer.)
EOM
exit;

}
