#!/usr/bin/env perl 

# This script is provided "as is" it is
# not warranted but may be supported on a limited basis
# by Network Appliance.
#
# psf_sysstat.pl
#
#   PerfStatFilter for making sysstat data nice for Excel 
#
#   v.08   01-16-2006 : Adjusted output for ONTAP versions of 7.1 or greater that do not have DAFS.
#			Output is the same. 
#   v0.07  02-02-2005 : Added a line of all zeroes to the end of the output with
#                       CP_type set to "U" for Undefined. To fix problem
#                       in BadCPs charts where multiple iterations show 
#                       incorrect state. 
#   v0.06  01-31-2005 : Fixed bug where dates of form yyyy-mm-dd did not work. 
#   v0.05  01-19-2005 : Fixed bug where dates of form dd.mm.yyyy did not work. 
#   v0.04  12-15-2004 : Fixed bug where dates of form mm-dd-yyyy did not work. 
#   v0.03  06-24-2004 : Changes to support perfstat 5.30 and above.     
#   v0.02  03-19-2004 : Added -f <filer> for multi-filer perfstat files.
#                       Only tested against perfstat 5.20 Linux version.
#   v0.01  11-04-2003 : Initial implementation.
#

# EXPORTED CONSTANTS (Not really, you have to copy them)

use constant _NA_TIMESTAMP              => 0;
use constant _NA_SYSSTAT_CPU_UTIL               => 1;
use constant _NA_SYSSTAT_NFS_IOPS               => 2;
use constant _NA_SYSSTAT_CIFS_IOPS              => 3;
use constant _NA_SYSSTAT_HTTP_IOPS              => 4;
use constant _NA_SYSSTAT_TOTAL_IOPS             => 5;
use constant _NA_SYSSTAT_NET_KB_RECV            => 6;
use constant _NA_SYSSTAT_NET_KB_SENT            => 7;
use constant _NA_SYSSTAT_DISK_KB_READ           => 8;
use constant _NA_SYSSTAT_DISK_KB_WRITTEN        => 9;
use constant _NA_SYSSTAT_TAPE_KB_READ           => 10;
use constant _NA_SYSSTAT_TAPE_KB_WRITTEN        => 11;
use constant _NA_SYSSTAT_CACHE_AGE              => 12;
use constant _NA_SYSSTAT_CACHE_HIT              => 13;
use constant _NA_SYSSTAT_CP_TIME                => 14;
use constant _NA_SYSSTAT_CP_TYPE                => 15;
use constant _NA_SYSSTAT_DISK_UTIL              => 16;
use constant _NA_SYSSTAT_DAFS_IOPS              => 17;
use constant _NA_SYSSTAT_FCP_IOPS               => 18;
use constant _NA_SYSSTAT_ISCSI_IOPS             => 19;
use constant _NA_SYSSTAT_FCP_KB_RECV            => 20;
use constant _NA_SYSSTAT_FCP_KB_SENT            => 21;

use constant FIRST_DATA                 => _NA_SYSSTAT_CPU_UTIL;
use constant LAST_STD_DATA              => _NA_SYSSTAT_FCP_KB_SENT ;

use Getopt::Std;
getopts("SdDf:hHi:nXz");

if ($opt_h) { usage(); }

if ($opt_S) {
   $avg_all_samples = 1;
} else {
   $avg_all_samples = 0;
}

if ($opt_d) {
   $debug = 1;        
} else {
   $debug = 0;
}

if ($opt_D) {
   $data_plus_header = 1;        
} else {
   $data_plus_header = 0;
}

if ($opt_f) {
   $filer = $opt_f;; 
} 

if ($opt_H) {
   $header = 1; 
} else {
   $header = 0;
}

if ($opt_i) {
   $interval = $opt_i; 
} else {
   $interval = 1;
}

if ($opt_X) {
   $extended_stats = 1; 
} else {
   $extended_stats = 0;
}

if ($opt_z) {
   $non_perfstat   = 1; 
} else {
   $non_perfstat   = 0;
}

if ($opt_n) {
   $nice_output = 1;  
   $avg_all_samples = 1; 
} else {
   $nice_output = 0;
}

$samples=0;

@cpu_util, @cp_time, @cp_type;
@nfs_iops, @cifs_iops, @http_iops, @fcp_iops, @iscsi_iops, @dafs_iops;
@net_KB_recv, @net_KB_sent;
@disk_KB_read, @disk_KB_written;
@fcp_KB_recv, @fcp_KB_sent;
@tape_KB_read, @tape_KB_written;
@cache_age, @cache_hit, @disk_util;

$header_line1 = "                 ,CPU  ,NFS   ,CIFS  ,HTTP  ,Total ,   Net,kB/s  , Disk,kB/s  , Tape,kB/s  ,Cache ,Cache,  CP  , CP , Disk ,DAFS ,FCP ,iSCSI , FCP,kB/s, \n";
$header_line2 = "Timestamp        ,     ,      ,      ,      ,      ,    in, out  , read, write, read, write,  age , hit , time , ty, util  ,     ,    ,      ,  in, out, \n";
 
$data[_NA_SYSSTAT_CPU_UTIL] 		= \@cpu_util;
$data[_NA_SYSSTAT_NFS_IOPS]		= \@nfs_iops;
$data[_NA_SYSSTAT_CIFS_IOPS]		= \@cifs_iops;
$data[_NA_SYSSTAT_HTTP_IOPS]		= \@http_iops;
$data[_NA_SYSSTAT_TOTAL_IOPS]		= \@total_iops;
$data[_NA_SYSSTAT_NET_KB_RECV]		= \@net_KB_recv;
$data[_NA_SYSSTAT_NET_KB_SENT]		= \@net_KB_sent;
$data[_NA_SYSSTAT_DISK_KB_READ]		= \@disk_KB_read;
$data[_NA_SYSSTAT_DISK_KB_WRITTEN]	= \@disk_KB_written;
$data[_NA_SYSSTAT_TAPE_KB_READ]		= \@tape_KB_read;
$data[_NA_SYSSTAT_TAPE_KB_WRITTEN]	= \@tape_KB_written;
$data[_NA_SYSSTAT_CACHE_AGE]	        = \@cache_age;
$data[_NA_SYSSTAT_CACHE_HIT]	        = \@cache_hit;
$data[_NA_SYSSTAT_CP_TIME] 		= \@cp_time;
$data[_NA_SYSSTAT_CP_TYPE] 		= \@cp_type;
$data[_NA_SYSSTAT_DISK_UTIL] 		= \@disk_util;
$data[_NA_SYSSTAT_DAFS_IOPS]		= \@dafs_iops;
$data[_NA_SYSSTAT_FCP_IOPS]		= \@fcp_iops;
$data[_NA_SYSSTAT_ISCSI_IOPS]		= \@iscsi_iops;
$data[_NA_SYSSTAT_FCP_KB_RECV]		= \@fcp_KB_recv;
$data[_NA_SYSSTAT_FCP_KB_SENT]		= \@fcp_KB_sent;

# from psf_versions.pl

use constant    _NA_VERS_PERFSTAT_MAJOR => 0;
use constant    _NA_VERS_PERFSTAT_MINOR => 1;
use constant    _NA_VERS_ONTAP_VERSION  => 2;
use constant    _NA_VERS_ONTAP_MAJOR    => 3;
use constant    _NA_VERS_ONTAP_MINOR    => 4;
use constant    _NA_VERS_FILER_MODEL    => 5;
use constant    _NA_VERS_FILER_MAGIC1   => 6;
use constant    _NA_VERS_FILER_MAGIC2   => 7;
use constant    _NA_VERS_PERFSTAT_PLATFORM => 8;
use constant    _NA_VERS_NUM_SAMPLES    => 9;
use constant    _NA_VERS_SECS_PER_SAMPLE => 10;

if ( !$nice_output && $header ) {
   print $header_line1;
   print $header_line2;
   if (!$data_plus_header) {
     exit;
   }
}

$perfstat_file = $ARGV[0];
$version_line = `psf_versions.pl $perfstat_file`;
@versions = split (/,/, $version_line);
$perfstat_major = $versions[_NA_VERS_PERFSTAT_MAJOR];
$perfstat_minor = $versions[_NA_VERS_PERFSTAT_MINOR];
$perfstat_platform = $versions[_NA_VERS_PERFSTAT_PLATFORM];
$ontap_major = $versions[_NA_VERS_ONTAP_MAJOR];
$ontap_minor = $versions[_NA_VERS_ONTAP_MINOR];
if (($ontap_major == 7) &&
    ($ontap_minor > 0)) {
	$dafs_col_exists = 0;
} elsif ($ontap_major > 7) {
	$dafs_col_exists = 0;
} else {
	$dafs_col_exists = 1;
}

$model = $versions[_NA_VERS_FILER_MODEL];
$perfstat_samples = $versions[_NA_VERS_NUM_SAMPLES];
$perfstat_interval = $versions[_NA_VERS_SECS_PER_SAMPLE];

$new_timestamp = `psf_filer_timestamp.pl $perfstat_file`;
chomp $new_timestamp;

open(INPUTA, '<' . $perfstat_file) || die 'Unable to open perfstat file (arg1) .';

$samples=0;
while ($Line = <INPUTA>) { 
   if (($Line =~ / sysstat -[s,x] -c / ) && 
       ($Line =~ / $interval ; priv set/ )) {   # Windows version
      $Line =  <INPUTA>;
      $Line =  <INPUTA>;
      $found_it = 1;
   }
   elsif ( ( $Line =~ /^=-=-/ ) &&
           ( $Line =~ /POSTSTATS/ ) &&
           ( $Line =~ /sysstat_1sec/ ) ) {      # 5.20 Linux version and 
      @line = split (/\s+/, $Line);             # 5.3x all versions
      $curr_filer = $line[2];
      if ( (!$filer) ||
            ($filer =~ /^$curr_filer/) ) {
         $found_it = 1;
         $Line =  <INPUTA>;
         if ($Line =~ /\d+:\d+:\d+/) {
            @line = split (/\s+/, $Line);
            if ($line[0] =~ /Begin/) {
                $mo = $line[2];
                $day = $line[3];
                $time = $line[4];
                $yr = $line[6];
                $mo_num = month_to_numeric ($mo);
            } else {
                if ($line[1] =~ /\d+\/\d+\/\d+/) {
			@date_line = split (/\//, $line[1]);
            		if ($debug) {
                		printf ("Dateline format mm/dd/yyyy : %s \n", $line[1]);
            		}
			$mo_num = $date_line[0];
			$day = $date_line[1];
			$yr = $date_line[2];
			$time = $line[2];
		}
                elsif ($line[1] =~ /\d\d-\d\d-\d\d\d\d/) {
                        @date_line = split (/-/, $line[1]);
                        if ($debug) {
                                printf ("Dateline format dd-mm-yyyy : %s \n", $line[1]);
                        }
                        $mo_num = $date_line[1];
                        $day = $date_line[0];
                        $yr = $date_line[2];
                        $time = $line[2];
                }
                elsif ($line[0] =~ /\d\d-\d\d-\d\d\d\d/) {
                        @date_line = split (/-/, $line[0]);
                        if ($debug) {
                                printf ("Dateline format dd-mm-yyyy : %s \n", $line[0]);
                        }
                        $mo_num = $date_line[1];
                        $day = $date_line[0];
                        $yr = $date_line[2];
                        $time = $line[1];
                }
                elsif ($line[0] =~ /^\d{4}-\d{2}-\d{2}/) {
                        @date_line = split (/-/, $line[0]);
                        if ($debug) {
                                printf ("Dateline format yyyy-mm-dd : %s \n", $line[1]);
                        }
                        $mo_num = $date_line[1];
                        $day = $date_line[2];
                        $yr = $date_line[0];
                        $time = $line[1];
                }
                elsif ($line[0] =~ /\d+\.\d+\.\d+/) {
                        @date_line = split (/\./, $line[0]);
                        if ($debug) {
                                printf ("Dateline format dd.mm.yyyy : %s \n", $line[0]);
                        }
                        $mo_num = $date_line[1];
                        $day = $date_line[0];
                        $yr = $date_line[2];
                        $time = $line[1];
                }
		elsif ($line[1] =~ /\d+\.\d+\.\d+/) {
                        @date_line = split (/\./, $line[1]);
                        if ($debug) {
                                printf ("Dateline format dd.mm.yyyy : %s \n", $line[1]);
                        }
                        $mo_num = $date_line[1];
                        $day = $date_line[0];
                        $yr = $date_line[2];
                        $time = $line[2];
                }
		else {
                	$mo = $line[1];
                	$day = $line[2];
                	$time = $line[3];
                	$yr = $line[5];
                        $mo_num = month_to_numeric ($mo);
                        if ($debug) {
                                printf ("Dateline format mm dd time yyyy : %s \n", $line[0]);
                        }
		}
            }
            # If $time ends with  ",xx", remove that
	    if ($time =~ /\,\d\d/) {
               $time = substr ($time, 0, length($time) - 3);
            } 
            $new_timestamp = sprintf ("%02d\/%02d\/%d %s", $mo_num, $day, $yr, $time);
            if ($debug) {
                printf ("%s \n", $new_timestamp);
            }
         }
      }
      else {
         $found_it = 0;
      }
   }
   elsif ($non_perfstat) {
      $found_it = 1;
      if ($Line =~ /\d+:\d+:\d+/) {
                 # Found timestamp in non-perfstat syssstat stream
             @line = split (/\s+/, $Line);
             $mo = $line[1];
             $day = $line[2];
             $time = $line[3];
             $yr = $line[5];
             $mo_num = month_to_numeric ($mo);
             $new_timestamp = sprintf ("%02d\/%02d\/%d %s", $mo_num, $day, $yr, $time);
             if ($debug) {
                 printf ("%s \n", $new_timestamp);
             }
      }

   }
   if ( $found_it ) {
      while ($Line = <INPUTA>) { 
         @line = split (/\s+/, $Line);
         if ($non_perfstat & 
             ($Line =~ /\d+:\d+:\d+/)) {
                 # Found timestamp in non-perfstat syssstat stream
             $mo = $line[1];
             $day = $line[2];
             $time = $line[3];
             $yr = $line[5];
             $mo_num = month_to_numeric ($mo);
             $new_timestamp = sprintf ("%02d\/%02d\/%d %s", $mo_num, $day, $yr, $time);
             if ($debug) { 
                 printf ("%s \n", $new_timestamp);
             }
         }
         if (($Line =~ /CPU/ ) ||
             ( !$Line ) ||
             ($Line =~ /write/ ))  {
            next;
         }
         if (($Line =~ /^Min/ ) ||
             ($Line =~ /^--/ )  ||
             !($line[8] =~ /\d+/ )  ||
             ($Line =~ /^Summary/ )) {
            last;
         }
         
         # Otherwise it should be a sysstat line
         $samples++; 
         if ( $new_timestamp ) {
            $timestamp[$samples] = $new_timestamp ;
            $new_timestamp = 0;
         }
         else {
            $timestamp[$samples] = bump_timestamp ($timestamp[$samples-1], $interval);
         }
         if ( $line[0] ) { 
            $index = 0;
         }
         else {
            $index = 1;
         }
         $cpu_util[$samples]        = $line[$index++];
         $nfs_iops[$samples]        = $line[$index++];
         $cifs_iops[$samples]       = $line[$index++];
         $http_iops[$samples]       = $line[$index++];
         $total_iops[$samples]      = $line[$index++];
         $net_KB_recv[$samples]     = $line[$index++];
         $net_KB_sent[$samples]     = $line[$index++];
         $disk_KB_read[$samples]    = $line[$index++];
         $disk_KB_written[$samples] = $line[$index++];
         $tape_KB_read[$samples]    = $line[$index++];
         $tape_KB_written[$samples] = $line[$index++];
         $cache_age[$samples]       = $line[$index++];
         $cache_hit[$samples]       = $line[$index++];
         $cp_time[$samples]         = $line[$index++];
         $cp_type[$samples]         = $line[$index++];
         $disk_util[$samples]       = $line[$index++];
	 if ($dafs_col_exists) {
         	$dafs_iops[$samples]       = $line[$index++];
	 }
         $fcp_iops[$samples]        = $line[$index++];
         $iscsi_iops[$samples]      = $line[$index++];
         $fcp_KB_recv[$samples]     = $line[$index++];
         $fcp_KB_sent[$samples]     = $line[$index++];
      } # End while
      # Add one more sample to delineate the end of an iteration
      # this is done to prevent multiple iterations from
      # showing the wrong CP state in graphs when going from 
      # one iteration to another. 
      $samples++;
      $timestamp[$samples] = bump_timestamp ($timestamp[$samples-1], $interval);
      $cpu_util[$samples] = 0;
      $nfs_iops[$samples] = 0;
      $cifs_iops[$samples] = 0;
      $http_iops[$samples] = 0;
      $total_iops[$samples] = 0;
      $net_KB_recv[$samples] = 0;
      $net_KB_sent[$samples] = 0;
      $disk_KB_read[$samples] = 0;
      $disk_KB_written[$samples] = 0;
      $tape_KB_read[$samples] = 0;
      $tape_KB_written[$samples] = 0;
      $cache_age[$samples] = 0;
      $cache_hit[$samples] = 0;
      $cp_time[$samples] = "1%";
      $cp_type[$samples] = "U";
      $disk_util[$samples] = 0;
      $dafs_iops[$samples] = 0;
      $fcp_iops[$samples] = 0;
      $iscsi_iops[$samples] = 0;
      $fcp_KB_recv[$samples] = 0;
      $fcp_KB_sent[$samples] = 0;

      $found_it = 0;
   } # End else
   $prev_2_line = $prev_1_line;
   $prev_1_line = $Line;
}

if ( $debug ) { printf ("Num samples found : %d \n", $samples); }

if ( $samples > 0 ) { 
   if  (!$avg_all_samples) {
       for ( $j=1; $j <= $samples; $j++ ) {
         printf ("%s, ", $timestamp[$j]);
         for ( $i=FIRST_DATA; $i <= LAST_STD_DATA; $i++ ) {
            if (( $i == _NA_SYSSTAT_CPU_UTIL ) ||
                ( $i == _NA_SYSSTAT_DISK_UTIL ) ||
                ( $i == _NA_SYSSTAT_CP_TIME   ) ||
                ( $i == _NA_SYSSTAT_CACHE_HIT )) {
               printf ("%4s,", $data[$i]->[$j]);
            }
            elsif ( $i == _NA_SYSSTAT_CP_TYPE ) {
               printf ("%3s,", $data[$i]->[$j]);
            }
            else {
               printf ("%6d,", $data[$i]->[$j]);
            }
         }
         printf ("\n");
       }
   }
   else {
      print "Not Implemented\n";
      # average_stats();
      # print_stats();
   }
}

# END OF MAIN 

sub month_to_numeric {
   local ($month) = @_ ;
   my $result;

   if ($month =~ /Jan/ ) {
        $result = 1;
   }
   elsif ($month =~ /Feb/ ) {
        $result = 2;
   }
   elsif ($month =~ /Mar/ ) {
        $result = 3;
   }
   elsif ($month =~ /Apr/ ) {
        $result = 4;
   }
   elsif ($month =~ /May/ ) {
        $result = 5;
   }
   elsif ($month =~ /Jun/ ) {
        $result = 6;
   }
   elsif ($month =~ /Jul/ ) {
        $result = 7;
   }
   elsif ($month =~ /Aug/ ) {
        $result = 8;
   }
   elsif ($month =~ /Sep/ ) {
        $result = 9;
   }
   elsif ($month =~ /Oct/ ) {
        $result = 10;
   }
   elsif ($month =~ /Nov/ ) {
        $result = 11;
   }
   elsif ($month =~ /Dec/ ) {
        $result = 12;
   }
   else {
        $result = -1;
   }
   return $result;
}

sub get_timestamp {      # Only used for Windows 
      @date_line = split (/ +/, $prev_2_line);
      $date = $date_line[1];
      @line = split (/ +/, $prev_1_line);
      if ( length($line[0]) ) {
              $time = $line[0];
              $ampm = $line[1];
      }
      else {
              $time = $line[1];
              $ampm = $line[2];
      }
      if ( length($ampm) == 0 ) {
              chop $time;
              chop $time;
              $ampm = chop $time;
              if ( $ampm =~ /p/ ) {
                 $ampm = "PM";
              }
              else {
                 $ampm = "AM";
              }
      }
      if ($debug) { printf ("AM/PM     : %s \n", $ampm); }
      @timeparts = split (/:/, $time);
      $hour = @timeparts[0];
      $min  = @timeparts[1];
      if ( ( $ampm =~ /PM/ ) &&
           ( $hour < 12 ) )  {
         # Add 12 hours to time
         $hour += 12;
      }
      elsif ( ( $ampm =~ /AM/ ) &&
              ( $hour == 12 ) ) {
         $hour = 0;
      }
      $time = sprintf ("%d:%d:%d", $hour, $min, 0);

      if ($debug) { printf ("Base Time : %s \n", $time); }
      $timestamp = sprintf ("%s %s", $date, $time);
      if ($debug) { printf ("Base timestamp : %s \n", $timestamp ); }
      return $timestamp;
}

sub bump_timestamp {
   local ($timestamp, $interval) = @_;

   @timestamp_parts = split (/ /, $timestamp);
   $date = $timestamp_parts[0];
   @dateparts = split (/\//, $date);
   $mo = $dateparts[0];
   $day = $dateparts[1];
   $year = $dateparts[2];
   $time = $timestamp_parts[1];
   @timeparts = split (/:/, $time);
   $hour = $timeparts[0];
   $min  = $timeparts[1];
   $sec  = $timeparts[2];
   if ($debug) { printf ("Date: %s %d %d %d Time : %d %d %d \n",
                           $date, $mo, $day, $year, $hour, $min, $sec ); }
   if ( $sec < (60 - $interval) ) {
      $sec += $interval;
   }
   else {
      $sec = ($sec + $interval) - 60;
      if ( $min < 59 ) {
         $min++;
      }
      else {
         $min = 0;
         if ( $hour < 23 ) {
            $hour++;
         }
         else {
            $hour = 0;
            $day++;      # I am not going to figure out
                         # days in month and leap years.
         }
      }
   }
   $newdate = sprintf ("%02d\/%02d\/%d", $mo, $day, $year);
   $newtime = sprintf ("%02d:%02d:%02d", $hour, $min, $sec);
   if ($debug) { printf ("New Time : %s \n", $newtime); }
   $newtimestamp = sprintf ("%s %s", $newdate, $newtime);
   return $newtimestamp;
}

sub average_statit_stats {
   my $i, $j;

   if ($debug) { print "In average_statit_stats \n"; }
   for ( $i=FIRST_DATA; $i <= LAST_STD_DATA; $i++ ) {
      $data[$i]->[0] = 0;    # oth element used for averages
      for ( $j=1; $j <= $samples; $j++ ) {
         $data[$i]->[0] += $data[$i]->[$j];
      }   
      $data[$i]->[0] /= $samples; 
   }
}

sub usage {
   print <<EOM;
psf_sysstat.pl [-DHh] [-f <filer>]  <perfstat filename>

Extracts the sysstat -i 1 data from perfstat output files. 

    Arguments and switches - 
       -D          Print data as well as header. Only 
                   relevent if -H specified.
       -H          Print a header line only unless -D specified. 
       -h          This message.
       -f          For multi-filer perfstat files specify which one. 
EOM
exit;

}
