#!/usr/bin/env perl 

# Change history
# sept 15 2004    - Set ONTAP version to 6.1 for private builds.
# nov 12, 2004    - Added field for # filer CPUs

use Getopt::Std;
getopts("df:h");

if ($opt_h) { usage(); }

if ($opt_f) {
   $filer = $opt_f;   
} 

if ($opt_d) {
   $debug = 1;        
} else {
   $debug = 0;
}

use constant 	_NA_VERS_PERFSTAT_MAJOR	=> 0;
use constant 	_NA_VERS_PERFSTAT_MINOR	=> 1;
use constant 	_NA_VERS_ONTAP_VERSION 	=> 2;
use constant 	_NA_VERS_ONTAP_MAJOR   	=> 3;
use constant 	_NA_VERS_ONTAP_MINOR   	=> 4;
use constant 	_NA_VERS_FILER_MODEL   	=> 5;
use constant 	_NA_VERS_FILER_MAGIC1  	=> 6;
use constant 	_NA_VERS_FILER_MAGIC2  	=> 7;
use constant 	_NA_VERS_PERFSTAT_PLATFORM => 8;
use constant    _NA_VERS_NUM_SAMPLES    => 9;
use constant    _NA_VERS_SECS_PER_SAMPLE => 10;
use constant    _NA_VERS_SECS_PER_ITERATION => 11;
use constant    _NA_VERS_NUM_FILER_CPUS     => 12;


my %CPUMemToModel;

$CPUMemToModel{1}{1792} = 'FAS2050';
$CPUMemToModel{1}{2048} = 'FAS920';
$CPUMemToModel{1}{3072} = 'FAS940';
$CPUMemToModel{2}{1022} = 'FAS270';
$CPUMemToModel{2}{2048} = 'FAS3050';
$CPUMemToModel{2}{6144} = 'FAS960';
$CPUMemToModel{2}{16384} = 'FAS6030';
$CPUMemToModel{4}{3072} = 'FAS3050';
$CPUMemToModel{4}{8192} = 'FAS980';
$CPUMemToModel{4}{32768} = 'FAS6070';

my %ModelToCPUs ; 
		
$ModelToCPUs{FAS250}		= 1;
$ModelToCPUs{F825}		= 1;
$ModelToCPUs{F810}		= 1;
$ModelToCPUs{FAS920}		= 1;
$ModelToCPUs{FAS940}		= 1;
$ModelToCPUs{FAS2050}		= 1;
$ModelToCPUs{F880}		= 2;
$ModelToCPUs{R200}		= 2;
$ModelToCPUs{FAS270}		= 2;
$ModelToCPUs{FAS3020}		= 2;
$ModelToCPUs{FAS960}		= 2;
$ModelToCPUs{FAS6030}		= 2;
$ModelToCPUs{FAS3050}		= 4;
$ModelToCPUs{FAS3070}		= 4;
$ModelToCPUs{FAS980}		= 4;
$ModelToCPUs{FAS6070}		= 4;
	
# Both FAS3070 and FAS980 have 4 processors and 8192 MB. But 
# since the main purpose is to know the num CPUs don't worry. 

$perfstat_file = $ARGV[0];
open(INPUTA, '<' . $perfstat_file) || die 'Unable to open perfstat file (arg1) .';

while ($Line = <INPUTA>) { 
   $Line =~ s/\r//g;     # Remove DOS carraige returns
   if ( $Line =~ /^\*-+\s+Perfstat/ ) {
      if ($debug) {
         printf ("Recognized new perfstat\n");
      }
      @line = split (/\s+/, $Line);
      $perfstat_version = $line[2];
      if ($debug) {
         printf ("Version : %s \n",  $perfstat_version);
      }
   }
   elsif ( $Line =~ /^SOURCE_HOST_OS/ ) {
      @line = split (/\s+/, $Line);
      $host_OS = $line[1];
      if ( $host_OS =~ /Win/ ) {
         $platform = "Windows";
      }
      else {
         $platform = "Unix";
      }
   }
   elsif ( $Line =~ /^SAMPLES/ ) {
      @line = split (/\s+/, $Line);
      $num_samples = $line[1];
   }
   elsif ( $Line =~ /^SAMPLE_INTERVAL/ ) {
      @line = split (/\s+/, $Line);
      $secs_per_sample = $line[1];
   }
   elsif ( $Line =~ /^TIME/ ) {
      @line = split (/\s+/, $Line);
      $secs_per_iteration = $line[2];
      $len = length $secs_per_iteration;
      if ($secs_per_iteration =~ /\"/) {
            $secs_per_iteration = substr ($secs_per_iteration, 1, $len-1);
            $len--;
      }
      if ($secs_per_iteration =~ /\"/) {
            $secs_per_iteration = substr ($secs_per_iteration, 0, $len-1);
      }
   }
   # Pre-5.30 stuff  
   elsif (($Line =~ /^Perfstat: v/ ) ||
       ($Line =~ /^perfstat.exe/ ))  { 
      if ($Line =~ /^perfstat.exe/ ) {
         $platform = "Windows";
      }
      else {
         $platform = "Unix";
      } 
      chop $Line;
      @line = split (/ +/, $Line);
      if ( $line[1] =~ /Version/ ) {
         $perfstat_version = $line[2];
      }
      else {
         $perfstat_version = $line[1];
      }
      last;
   }
   elsif ($Line =~ /=-=-=.+CONFIG/ ) { 
      last;
   }
   else {
      next;
   }
}
chomp $perfstat_version;
if ( $perfstat_version ) {
   $first_char = substr($perfstat_version,0,1);
   if ( ($first_char =~ /v/ ) ||
        ($first_char =~ /V/ ) )  {
       substr($perfstat_version,0,1) = '';
   }
   @parts = split (/\./, $perfstat_version );
   $perfstat_major = $parts[0];
   $perfstat_minor = $parts[1];
   $perfstat_vers_num = ($perfstat_major * 100) + $perfstat_minor;
   if ($debug) { 
      printf ("perfstat_version_num : %d \n", $perfstat_vers_num); 
   }
}
if (($perfstat_vers_num >= 520) && 
   ($perfstat_vers_num < 530)) {
   while ($Line = <INPUTA>) {
      if ($Line =~ / seconds per sample/) {
         @line = split (/ +/, $Line);
         $num_samples = substr($line[1],1,(length($line[1]) - 1));
         $secs_per_sample = $line[3];
      }
      elsif ($Line =~ /CONFIG/ ) {
         last;
      }
   }
}

$magic_1 = 0;
$magic_2 = 0;
$model = "";
seek INPUTA, 0,0;
while ($Line = <INPUTA>) {
   if   ($Line =~ / sysconfig -av/) {
      @line = split (/\s+/, $Line);
      $curr_filer = $line[2];
      if ( (!$filer) ||
           ($filer =~ /^$curr_filer/) ) {

         while ($Line = <INPUTA>) {
            $Line =~ s/\r//g;     # Remove DOS carraige returns
            if ($Line =~ /Model Name/ ) {
               @line = split (/ +/, $Line);
               $model = $line[3];
               chomp $model;
				$model =~ s/C//;	# Remove Cluster designation
				$model =~ s/^GF/FAS/;	# Remove Gfiler designation
               $filer_num_cpus = $ModelToCPUs{$model};
               last;
            }
         }
      }
   }
   elsif ($Line =~ /POSTSTAT/ ) { last;}
}

# If did not find sysconfig -av, look for sysconfig.out in POSTSTATS.
if (!$model) {
	while ($Line = <INPUTA>) {
		if ($Line =~ /sysconfig.out/) {
			@line = split (/\s+/, $Line);
			$curr_filer = $line[2];
			if ( (!$filer) ||
				($filer =~ /^$curr_filer/) ) {
				while ($Line = <INPUTA>) {
					$Line =~ s/\r//g;     # Remove DOS carraige returns
					if ($Line =~ /Processors:\s+(\d+)/) {
						$filer_num_cpus = $1;
					} elsif ($Line =~ /Memory Size:\s+(\d+)/) {
						$memory_size = $1;
					}
					if ($filer_num_cpus && $memory_size) {
						$model = $CPUMemToModel{$filer_num_cpus}{$memory_size};
						last;
					}
				}
				last;
			}
		}
	}
}

while ($Line = <INPUTA>) {
   if ($Line =~ /^Hostname:/ ) {
      last;
   }
   else {
      next;
   }
}

# I'm not sure what the above while loop is intended to do. But I need to
# restart my scan of the file, at least for Perfstat 6.0 

seek INPUTA, 0,0;
while ($Line = <INPUTA>) {
   if ($Line =~ /NetApp Release / ) {
      chop $Line;
      @line = split (/\s+/, $Line);
      if (@line[0] =~ /NetApp/) {
         $ontap_version = $line[2];
      }
      else {
         $ontap_version = $line[3];
      }
      chop $ontap_version;
      last;
   }
   else {
      next;
   }
}
$ontap_major = 7;
$ontap_minor = 2;
if ( $ontap_version ) {
	if ($ontap_version =~ /^(\d)\.(\d)/ ) {
		# Released version or candidate
		$ontap_major = $1; 
		$ontap_minor = $2;
	}
	elsif ($ontap_version =~ /^R/ ) {
		if ($ontap_version =~ /krakus/ ) { 
			$ontap_major = 6;
			$ontap_minor = 3;
		}
		elsif ($ontap_version =~ /fullsail/ ) {
			$ontap_major = 6;
			$ontap_minor = 4;
		}
		elsif ($ontap_version =~ /scrimshaw/ ) {
			$ontap_major = 6;
			$ontap_minor = 5;
		}
		elsif ($ontap_version =~ /anchorsteam/ ) {
			$ontap_major = 7;
			$ontap_minor = 0;
		}
		elsif ($ontap_version =~ /tsingtao/ ) {
			$ontap_major = 7;
			$ontap_minor = 1;
		}
		elsif ($ontap_version =~ /gordonb/ ) {
			$ontap_major = 7;
			$ontap_minor = 2;
		}
		elsif ($ontap_version =~ /ironcity/ ) {
			$ontap_major = 7;
			$ontap_minor = 3;
		}
	}
}
printf ("%d, %d, %s, %d, %d, %s, %d, %d, %s, %d, %d, %d, %d, \n", 
       $perfstat_major, $perfstat_minor, $ontap_version, $ontap_major, $ontap_minor, $model, $magic_1, $magic_2, $platform, $num_samples, $secs_per_sample, $secs_per_iteration, $filer_num_cpus );

# END OF MAIN 

sub model_to_num_cpus {
   my ($model) =  @_ ;
   my $num_cpus;

   if (($model =~ /FAS980/) ||
		($model =~ /FAS3050/) ||
		($model =~ /FAS6070/)){
			$num_cpus = 4;
   } 
   elsif (($model =~ /FAS3020/) ||
		($model =~ /FAS6070/) ||
		($model =~ /FAS96/) ||
		($model =~ /FAS27/) ||
		($model =~ /F880/)  ||
		($model =~ /R200/)) {
			$num_cpus = 2;
   }
   elsif (($model =~ /FAS2050/) ||
		($model =~ /FAS94/) ||
		($model =~ /FAS92/) ||
		($model =~ /F825/)  ||
		($model =~ /FAS25/)  ||
		($model =~ /F810/)) {
			$num_cpus = 1;
   }
   return $num_cpus;
}

sub usage {
   print <<EOM;
psf_versions.pl [-dh]  <perfstat filename>

Returns version info for perfstat and Data ONTAP. 

    Arguments and switches - 
       -d          Print debug messages.
       -h          This message.

Example : 

$ psf_versions.pl perfstat_file

EOM
exit;

}
