#!/usr/bin/env perl 

# This script is provided "as is" it is
# not warranted but may be supported on a limited basis
# by Network Appliance.
#
# psf_vol.pl
#
#   PerfStatFilter for Volume stats. 
#	   01-30-2007 : Fixed bug where disks on loops with port names other
#			then a,b would not be found. burt 233827.  
#   v0.06  11-08-2004 : Fixed bug where volumes would be combined if ones name was
#                       a substring of another. WOuld also create charts with messed up
#                       time scales. 
#   v0.06  03-19-2004 : Added -f <filer> for multi-filer perfstat files.
#                       Only tested against perfstat 5.20 Linux version. 
#   v0.05  01-20-2004 : Fixed some bugs related to averaging different samples.
#                       There are still some problems related to perfstat files with 
#                       data from multiple filers. 
#   v0.04  12-18-2003 : Removed tests for minimal utilizations or xfers to 
#                       print a volume's stats. This will avoid gaps in 
#                       data when charting.
#   v0.03  12-09-2003 : Changed from specificying #!/usr/local/bin to       
#                        #!/usr/bin/env perl . This even seemed to work on 
#                        my Win2K system. 
#
#   v0.01  11-04-2003 : Initial implementation.
#

# EXPORTED CONSTANTS (Not really, you have to copy them)

use constant   _NA_TIMESTAMP          => 0;
# use constant   _NA_VOL_NAME           => 1;
use constant   _NA_VOL_RG             => 2;
use constant   _NA_VOL_UT             => 3;
use constant   _NA_VOL_XFERS          => 4; # Note this is avg per disk by default
use constant   _NA_VOL_UREADS         => 5;
use constant   _NA_VOL_UREAD_CHAIN    => 6;
use constant   _NA_VOL_UREAD_USECS    => 7;
use constant   _NA_VOL_UWRITES        => 8;
use constant   _NA_VOL_UWRITE_CHAIN   => 9;
use constant   _NA_VOL_UWRITE_USECS   => 10;
use constant   _NA_VOL_CPREADS        => 11;
use constant   _NA_VOL_CPREAD_CHAIN   => 12;
use constant   _NA_VOL_CPREAD_USECS   => 13;
use constant   _NA_VOL_GREADS         => 14;
use constant   _NA_VOL_GREAD_CHAIN    => 15;
use constant   _NA_VOL_GREAD_USECS    => 16;
use constant   _NA_VOL_GWRITES        => 17;
use constant   _NA_VOL_GWRITE_CHAIN   => 18;
use constant   _NA_VOL_GWRITE_USECS   => 19;
use constant   _NA_VOL_SVCTIME        => 20; # Avg disk service time computed as S = U/X
use constant   _NA_VOL_RESPONSE_TIME  => 21; # Average response time of all transfers
use constant   _NA_VOL_Q_LEN          => 22;
use constant   _NA_VOL_Q_BUSY         => 23;
use constant   _NA_VOL_NUM_DISKS      => 24; # Num data disks
use constant   FIRST_DATA             => _NA_VOL_UT;
use constant   LAST_DATA              => _NA_VOL_Q_BUSY;

# From psf_vol_info.pl 

use constant    _NA_VOL_NAME            => 0;
use constant    _NA_VOL_RAID_TYPE       => 1;
use constant    _NA_VOL_RAID_SIZE       => 2;
use constant    _NA_VOL_MIRRORED        => 3;

use Getopt::Std;
getopts("SdDf:hHi:np:tXrRV:");

if ($opt_h) { usage(); }

if ($opt_S) {
   $avg_all_samples = 1;  
} else {
   $avg_all_samples = 0;
}

if ($opt_d) {
   $debug = 1;  
} else {
   $debug = 0;
}

if ($opt_D) {
   $data_plus_header = 1;  
} else {
   $data_plus_header = 0;
}

if ($opt_f) {
   $filer = $opt_f ;  
}

if ($opt_H) {
   $header = 1;  
} else {
   $header = 0;
}

if ($opt_i) {
   $vol_info_file = $opt_i;  
} else {
   $vol_info_file = "";
}

if ($opt_n) {
   $nice_output = 1;  
} else {
   $nice_output = 0;
}

if ($opt_p) {
   $parity_disk_file = $opt_p;  
} else {
   $parity_disk_file = "";  
}

if ($opt_t) {
   $show_totals = 1;  
} else {
   $show_totals = 0;
}

if ($opt_X) {
   $extended_stats = 1;
} else {
   $extended_stats = 0;
}

if ($opt_r) {
   $show_rg = 1;
} else {
   $show_rg = 0;
}

if ($opt_R) {
   $show_rg = 1;
   $rg_only = 1;
} else {
   $rg_only = 0;
}

if ($opt_V) {
   $only_vol = $opt_V;
} 

if ($opt_x && $opt_t) {
   printf ("Extended statistics not available with totals \n");
   exit;
}


$std_header =     "Timestamp,                   Vol,  rg,   \%ut,  xfer, uread,chain,  usec, write,chain,  usec, CPrd,chain,usec,   grd,chain,usec,  gwr,chain,usec";
$ext_header = "Timestamp,                   Vol,  rg,   \%ut,  xfer, uread,chain,  usec, write,chain,  usec, CPrd,chain,usec,   grd,chain,usec, gwr,chain,usec, Sdsk, Rdsk,Qlen,Qbusy,Ndsks,";

$in_disk_stats = 0;
$in_vol_stats = 0;
$rgid = 1;
$volid = 0;
$disk = 0;
$old_vol = "xyzzy";
$old_volid = 1;     
$sample=0;
$not_first_rg = 0;
$not_first_vol = 0;
$found_end_of_disk_stats = 0;

@rg_xfers, @rg_ut, @rg_ureads;
@vol_xfers, @vol_ut;
@vol_ureads, @vol_uread_chain, @vol_uread_usecs;
@vol_writes, @vol_write_chain, @vol_write_usecs;
@vol_cpreads, @vol_cpread_chain, @vol_cpread_usecs;
@vol_greads, @vol_gread_chain, @vol_gread_usecs;
@vol_gwrites, @vol_gwrite_chain, @vol_gwrite_usecs;

if ( !$nice_output && $header ) {
   if ( $extended_stats ) {
      printf ("%s \n", $ext_header);
   }
   else {
      printf ("%s \n", $std_header);
   }
   if (!$data_plus_header) {
     exit;
   }
}

if ($vol_info_file) {
   if (open(VOLINFO, '<' . $vol_info_file) ) {
   	@vol_info = <VOLINFO>;
   	foreach $vol_line (@vol_info) {
		@vol_info_parts = split (/,/, $vol_line);
		$vol_name = $vol_info_parts [_NA_VOL_NAME];
		$raid_type = $vol_info_parts [_NA_VOL_RAID_TYPE];
		$vol_raid_type {$vol_name} = $raid_type; 
   	}
	close (VOLINFO);
   }
}

my %parity_disks;
if ($parity_disk_file) {
   if (open(PARITY, '<' . $parity_disk_file) ) {
   	@tmp_disks = <PARITY>;
	chomp @tmp_disks;
	close (PARITY);
	if (scalar (@tmp_disks)) {	
		foreach $dsk (@tmp_disks) {
			$parity_disks{$dsk} = $dsk;
			if ($debug) {	
				printf ("Parity disk: %s \n",  $parity_disks{$dsk});
			}
		}
		$have_parity_disks = 1;
	}
   }
}

$perfstat_file = $ARGV[0];
$perfstat_file = $ARGV[0];
open(INPUTA, '<' . $perfstat_file) || die 'Unable to open perfstat file (arg1) .';

     init_all_sample_averages ($volid);
     while ($Line = <INPUTA>) { 
        if ( $in_disk_stats ) { 
                 if ( $found_end_of_disk_stats ) { $in_disk_stats = 0; }
                 if ($Line =~ /plex/ ) { 
                    $new_rg = 1; 
                    $in_vol_stats = 1;
                    $found_end_of_disk_stats = 0;
                    if ( $debug ) {print "Found new rg \n"; }
                 }
                 if ( $found_end_of_disk_stats || $new_rg ) { 
                    if ( $not_first_rg ) {   # Process old rg
                       if ($disk) {
                          average_rg_stats($disk, $rgid);
                       }
                       # if (( $show_rg && ( $rg_xfers[$rgid] > 1 )) &&
                       if (( $show_rg ) &&
                          ((!$only_vol) || ( $only_vol eq $old_vol ))) {
                          # $tmp_rg_name = $rg_name[$old_volid][$rgid];
                          $tmp_rg_name = $rg_name[$volid][$rgid];
                          print_rg_sample ($old_vol, $tmp_rg_name, $rgid);
                        # if ( sample = 1 & unix & num samples > 1 then duplicate line for
                        #   right number of times 

                       }
                    }
                    else {
                       $not_first_rg = 1;
                    }
                 }
                 if ( $new_rg || $found_end_of_disk_stats ) {
                    $new_rg = 0;
                    @line = split (/\//, $Line);
                    $vol = $line[1];
                    $plex = $line[2];
                    $rg = $line[3];
                    chop $rg;
                    chop $rg;
                    if ( $debug ) { 
                        print $Line;
                        printf ("Vol : %s rg: %s  \n", $vol, $rg); }
                    if ( $vol eq $old_vol ) {  # New rg, old vol
                       $rgid++;
                       $rg_name[$volid][$rgid] = sprintf ("%s/%s",$plex, $rg);
                       $disk = 0;
                    }
                    else {  # New vol
                       if ( $not_first_vol || $found_end_of_disk_stats ) { # Process old vol 
                          average_vol_stats(($rgid), $volid);
                          if ((!$only_vol) || ( $only_vol eq $old_vol )) {
                             add_to_all_sample_averages ($volid);
                             $only_volid = $volid;
                             if  ((!$avg_all_samples) && (!$rg_only) ){
                               printf ("%s, ", $timestamp );
                               printf ("%s, all, %5.2f, %5.1f, ",
                                 $old_vol, $vol_ut[$volid], $vol_xfers[$volid]);
                               printf ("%5.1f, %4.1f, %5d, ",
                                $vol_ureads[$volid], $vol_uread_chain[$volid], $vol_uread_usecs[$volid]);
                               printf ("%5.1f, %4.1f, %5d, ",
                                $vol_writes[$volid], $vol_write_chain[$volid], $vol_write_usecs[$volid]);
                               printf ("%4.1f, %3.1f, %5d, ",
                                $vol_cpreads[$volid], $vol_cpread_chain[$volid], $vol_cpread_usecs[$volid]);
                               printf ("%4.1f, %3.1f, %4d, ",
                                $vol_greads[$volid], $vol_gread_chain[$volid], $vol_gread_usecs[$volid]);
                               printf ("%4.1f, %3.1f, %4d, ",
                                $vol_gwrites[$volid], $vol_gwrite_chain[$volid], $vol_gwrite_usecs[$volid]);
                               if ( ($extended_stats) && ($vol_xfers[$volid]) ) {
                                  $Sdisk = (($vol_ut[$volid]/100)/$vol_xfers[$volid])*1000;
                                  $Ruread = ($vol_uread_chain[$volid]*$vol_uread_usecs[$volid])/1000;
                                  $Rwrite = ($vol_write_chain[$volid]*$vol_write_usecs[$volid])/1000;
                                  $Rcpread = ($vol_cpread_chain[$volid]*$vol_cpread_usecs[$volid])/1000;
                                  $Rgread = ($vol_gread_chain[$volid]*$vol_gread_usecs[$volid])/1000;
                                  $Rgwrite = ($vol_gwrite_chain[$volid]*$vol_gwrite_usecs[$volid])/1000;
                                  $Rdisk = (($Ruread * $vol_ureads[$volid]) +
                                            ($Rwrite * $vol_writes[$volid]) +
                                            ($Rcpread * $vol_cpreads[$volid]) +
                                            ($Rgread * $vol_greads[$volid]) +
                                            ($Rgwrite * $vol_gwrites[$volid]) ) / $vol_xfers[$volid] ;
                                  $Qlen  = $vol_xfers[$volid] * ( $Rdisk / 1000) ;
                                  if ( $Sdisk ) { 
                                     $Qbusy = $Rdisk / $Sdisk ;
                                  }
                                  printf ("%3.1f, %4.1f, %3.1f, %3.1f, %d, ",
                                       $Sdisk, $Rdisk, $Qlen, $Qbusy, $vol_ndisks[$volid]); 
                               }
                               print ("\n");
                             }
                          }
                       }
                       $old_vol = $vol;
                       $old_rg = $rg;
                       if ( $not_first_vol ) {
                         $old_volid = $volid;
                       }
                       else {
                          $old_volid = 1;
                          $not_first_vol = 1;
                       }
                       $volid++;
                       $volname[$volid] = $vol;
                       $rgid = 1;
                       $rg_name[$volid][$rgid] = sprintf ("%s/%s",$plex, $rg);
                       $disk = 0;
                    }
		    if (!$have_parity_disks) {
                    # Guess that the first disks are parity 
                    	$Line = <INPUTA>; 
		    	$raid_type =  $vol_raid_type {$vol};
		    	if ($debug) {printf ("Vol %s raidtype %s \n", $vol, $raid_type); } 
		    	if ($raid_type =~ /raid_dp/) {
			 	$Line = <INPUTA>; 
		    	}
		    }
                    $disk = 0;
                 }
                 elsif ( $in_vol_stats ) {                 # Should be disk or blank line
                    @disk_line = split (/ +/, $Line);
                    if ( ($disk_line[0] =~ /^\d+[a,b,c,d,e,f,g,h]*.\d+/) || 
                         ($disk_line[0] =~ /^\S+:\d+.\d+/) ) {
			$disk_name = $disk_line[0];
			if (!$have_parity_disks ||
			    ($have_parity_disks &&
			     !$parity_disks{$disk_name} )) { 
                       		$disk++; 
                       		$ut[$disk]          = $disk_line[1];
                       		$xfers[$disk]       = $disk_line[2];
                       		$ureads[$disk]      = $disk_line[3];
                       		$uread_chain[$disk] = $disk_line[4];
                       		$uread_usecs[$disk] = $disk_line[5];
                       		$writes[$disk]      = $disk_line[6];
                       		$write_chain[$disk] = $disk_line[7];
                       		$write_usecs[$disk] = $disk_line[8];
                       		$cpreads[$disk]      = $disk_line[9];
                       		$cpread_chain[$disk] = $disk_line[10];
                       		$cpread_usecs[$disk] = $disk_line[11];
                       		$greads[$disk]       = $disk_line[12];
                       		$gread_chain[$disk]  = $disk_line[13];
                       		$gread_usecs[$disk]  = $disk_line[14];
                       		$gwrites[$disk]      = $disk_line[15];
                       		$gwrite_chain[$disk]  = $disk_line[16];
                       		$gwrite_usecs[$disk]  = $disk_line[17];
			} else {
				if ($have_parity_disks && $debug) {
					printf ("Skipping parity disk %s\n", $disk_name);
				}
			}
                    }
                    else { # End of Disk Statistics
                       if ( $debug ) { print $Line ; } 
                       $found_end_of_disk_stats = 1;
                    } 
                 } # else of new rg/vol or disk 
        } # if $in_disk_stats
        elsif ( ( $Line =~ /^=-=-/ ) &&
                ( $Line =~ /POSTSTATS/ ) && 
                ( $Line =~ /statit/ ) ) {
           @line = split (/\s+/, $Line);
           $curr_statit_filer = $line[2];
           if ( $debug ) { 
              printf ("Current statit filer : %s \n", $curr_statit_filer);
           } 
        }
        elsif ($Line =~ / Disk Statistics.*second/ )  {
           if ( (!$filer) ||
                ($filer eq $curr_statit_filer) ) {
              if ( $debug ) { print "Found new sample \n"; }
              $found_end_of_disk_stats = 0;
              $in_disk_stats = 1;
              $in_vol_stats = 0;
              $disk = 0;
              $volid = 0;
              $rgid = 0;
              $not_first_rg = 0;
              $not_first_vol = 0;
              $sample++;
           }
        }
        elsif ($Line =~ / Start time:/ ) {
           @disk_line = split (/ +/, $Line);
           $mo = $disk_line[4];
           $day = $disk_line[5]; 
           $time = $disk_line[6];
           $yr = $disk_line[8];
           $mo_num = month_to_numeric ($mo);
           $timestamp = sprintf ("%02d\/%02d\/%d %s", $mo_num, $day, $yr, $time);
        }
        elsif ($Line =~ / FILER POSTSTAT =-=-/ ) {
           $in_disk_stats = 0;
        }
      } #while

      if ( $avg_all_samples ) {
        $last_volid = $volid;
        for ( $volid = 1 ; $volid < $last_volid; $volid ++ ) {
         compute_all_sample_averages ($volid);
         # if ($all_ut[$volid] > 4) { 
         if ( 1 )  {  
          if ( !$all_xfers[$volid] ) { 
             $all_xfers[$volid] = 1;  # Avoid divide by zero 
          }
          $Sdisk = (($all_ut[$volid]/100)/$all_xfers[$volid])*1000;
          $Ruread = ($all_uread_chain[$volid]*$all_uread_usecs[$volid])/1000;
          $Rwrite = ($all_write_chain[$volid]*$all_write_usecs[$volid])/1000;
          $Rcpread = ($all_cpread_chain[$volid]*$all_cpread_usecs[$volid])/1000;
          $Rgread = ($all_gread_chain[$volid]*$all_gread_usecs[$volid])/1000;
          $Rgwrite = ($all_gwrite_chain[$volid]*$all_gwrite_usecs[$volid])/1000;
          $Rdisk = (($Ruread * $all_ureads[$volid]) +
                    ($Rwrite * $all_writes[$volid]) +
                    ($Rcpread * $all_cpreads[$volid]) +
                    ($Rgread * $all_greads[$volid]) +
                    ($Rgwrite * $all_gwrites[$volid]) ) / $all_xfers[$volid] ;
          $Qlen  = $all_xfers[$volid] * ( $Rdisk / 1000 );
          if ( $Sdisk ) { 
             $Qbusy = $Rdisk / $Sdisk ;  # Should divide both $Rdisk and $Sdisk by 1000, but 
                                      # that cancels out. 
          }
          else {
             $Qbusy = 0;
          }
          if ( $nice_output ) {
             printf ("Summary disk statistics for vol %s \n", $volname[$volid]);
             printf ("   Avg. Disk Utilization       : \t %.2f \%\n", $all_ut[$volid]);
             printf ("   Avg. Disk Xfers/sec         : \t %.1f \n", $all_xfers[$volid]);
             if ($extended_stats) {
                printf ("   Avg. Disk Svc Time (ms)     : \t %.1f \n", $Sdisk);
             }
             if ($extended_stats) {
                printf ("   Avg. Disk RT    (ms)        : \t %.1f \n", $Rdisk);
             }
             if ($extended_stats) {
                printf ("   Avg. Disk QLen              : \t %.1f \n", $Qlen);
             }
             if ($extended_stats) {
                printf ("   Avg. Disk QLen when Busy    : \t %.1f \n", $Qbusy);
             }
             printf ("   Avg. ureads/sec             : \t %.1f \n", $all_ureads[$volid]);
             printf ("   Avg. uread chain len        : \t %.1f \n", $all_uread_chain[$volid]);
             printf ("   Avg. uread usecs            : \t %d \n", $all_uread_usecs[$volid]);
             printf ("   Avg. writes/sec             : \t %.1f \n", $all_writes[$volid]);
             printf ("   Avg. write chain len        : \t %.1f \n", $all_write_chain[$volid]);
             printf ("   Avg. write usecs            : \t %d \n", $all_write_usecs[$volid]);
             printf ("   Avg. cpreads/sec            : \t %.1f \n", $all_cpreads[$volid]);
             printf ("   Avg. cpread chain len       : \t %.1f \n", $all_cpread_chain[$volid]);
             printf ("   Avg. cpread usecs           : \t %d \n", $all_cpread_usecs[$volid]);
             printf ("   Avg. greads/sec             : \t %.1f \n", $all_greads[$volid]);
             printf ("   Avg. gread chain len        : \t %.1f \n", $all_gread_chain[$volid]);
             printf ("   Avg. gread usecs            : \t %d \n", $all_gread_usecs[$volid]);
             printf ("   Avg. gwrites/sec            : \t %.1f \n", $all_gwrites[$volid]);
             printf ("   Avg. gwrite chain len       : \t %.1f \n", $all_gwrite_chain[$volid]);
             printf ("   Avg. gwrite usecs           : \t %d \n", $all_gwrite_usecs[$volid]);
          }
          else {
             printf ("%s, ", $timestamp );
             printf ("%s, all, %5.2f, %5.1f, ",
                    $volname[$volid], $all_ut[$volid], $all_xfers[$volid]);
             printf ("%5.1f, %4.1f, %5d, ",
                    $all_ureads[$volid], $all_uread_chain[$volid], $all_uread_usecs[$volid]);
             printf ("%5.1f, %4.1f, %5d, ",
                    $all_writes[$volid], $all_write_chain[$volid], $all_write_usecs[$volid]);
             printf ("%4.1f, %3.1f, %5d, ",
                    $all_cpreads[$volid], $all_cpread_chain[$volid], $all_cpread_usecs[$volid]);
             printf ("%4.1f, %3.1f, %4d, ",
                    $all_greads[$volid], $all_gread_chain[$volid], $all_gread_usecs[$volid]);
             printf ("%4.1f, %3.1f, %4d, ",
                    $all_gwrites[$volid], $all_gwrite_chain[$volid], $all_gwrite_usecs[$volid]);
             if ($extended_stats) {
                 printf ("%3.1f, %4.1f, %3.1f, %3.1f, %d, ",
                       $Sdisk, $Rdisk, $Qlen, $Qbusy, $all_ndisks[$volid]) ; 
             }
             printf ("\n");
          }
         }
        }
      }
sub average_rg_stats {
   local ($ndisks, $rgid) = @_ ;
   local ($d);

   # printf ("Num disks in raidgroup %s rg%d %d \n", $vol, $rgid ,$ndisks);
   $rg_ut[$rgid] = 0; $rg_xfers[$rgid] = 0; 
   $rg_ureads[$rgid] = 0; $rg_uread_chain[$rgid] = 0; $rg_uread_usecs[$rgid] = 0;
   $rg_writes[$rgid] = 0; $rg_write_chain[$rgid] = 0; $rg_write_usecs[$rgid] = 0;
   $rg_cpreads[$rgid] = 0; $rg_cpread_chain[$rgid] = 0; $rg_cpread_usecs[$rgid] = 0;
   $rg_greads[$rgid] = 0; $rg_gread_chain[$rgid] = 0; $rg_gread_usecs[$rgid] = 0;
   $rg_gwrites[$rgid] = 0; $rg_gwrite_chain[$rgid] = 0; $rg_gwrite_usecs[$rgid] = 0;
   for ($d = 1; $d <= $ndisks; $d++) {
      $rg_ut[$rgid]           += $ut[$d]; 
      $rg_xfers[$rgid]        += $xfers[$d];
      $rg_ureads[$rgid]       += $ureads[$d];
      $rg_uread_chain[$rgid]  += $uread_chain[$d] * $ureads[$d];
      $rg_uread_usecs[$rgid]  += $uread_usecs[$d] * $ureads[$d];
      $rg_writes[$rgid]       += $writes[$d];
      $rg_write_chain[$rgid]  += $write_chain[$d] * $writes[$d];
      $rg_write_usecs[$rgid]  += $write_usecs[$d] * $writes[$d];
      $rg_cpreads[$rgid]      += $cpreads[$d];
      $rg_cpread_chain[$rgid] += $cpread_chain[$d] * $cpreads[$d];
      $rg_cpread_usecs[$rgid] += $cpread_usecs[$d] * $cpreads[$d];
      $rg_greads[$rgid]       += $greads[$d];
      $rg_gread_chain[$rgid]  += $gread_chain[$d] * $greads[$d];
      $rg_gread_usecs[$rgid]  += $gread_usecs[$d] * $greads[$d];
      $rg_gwrites[$rgid]      += $gwrites[$d];
      $rg_gwrite_chain[$rgid] += $gwrite_chain[$d] * $gwrites[$d];
      $rg_gwrite_usecs[$rgid] += $gwrite_usecs[$d] * $gwrites[$d];
   }

   if ($rg_ureads[$rgid]) {
	$rg_uread_chain[$rgid] /= $rg_ureads[$rgid];
	$rg_uread_usecs[$rgid] /= $rg_ureads[$rgid];
   }
   if ($rg_writes[$rgid]) {
	$rg_write_chain[$rgid] /= $rg_writes[$rgid];
	$rg_write_usecs[$rgid] /= $rg_writes[$rgid];
   }
   if ($rg_cpreads[$rgid]) {
	$rg_cpread_chain[$rgid] /= $rg_cpreads[$rgid];
	$rg_cpread_usecs[$rgid] /= $rg_cpreads[$rgid];
   }
   if ($rg_greads[$rgid]) {
	$rg_gread_chain[$rgid] /= $rg_greads[$rgid];
	$rg_gread_usecs[$rgid] /= $rg_greads[$rgid];
   }
   if ($rg_gwrites[$rgid]) {
	$rg_gwrite_chain[$rgid] /= $rg_gwrites[$rgid];
	$rg_gwrite_usecs[$rgid] /= $rg_gwrites[$rgid];
   }

   $rg_ut[$rgid] /= $ndisks;
   if ( !$show_totals ) { 
	$rg_xfers[$rgid] /= $ndisks; 
	$rg_ureads[$rgid] /= $ndisks;
	$rg_writes[$rgid] /= $ndisks; 
	$rg_cpreads[$rgid] /= $ndisks; 
	$rg_greads[$rgid] /= $ndisks; 
	$rg_gwrites[$rgid] /= $ndisks; 
   }
}

sub print_rg_sample {
   local ($volname, $rgname, $rgid) = @_ ;

   printf ("%s, ", $timestamp );
   printf ("%s, %s, %5.2f, %5.1f, ",
      $volname, $rgname, $rg_ut[$rgid], $rg_xfers[$rgid]);
   printf ("%5.1f, %4.1f, %5d, ",
      $rg_ureads[$rgid], $rg_uread_chain[$rgid], $rg_uread_usecs[$rgid]);
   printf ("%5.1f, %4.1f, %5d, ",
      $rg_writes[$rgid], $rg_write_chain[$rgid], $rg_write_usecs[$rgid]);
   printf ("%4.1f, %3.1f, %5d, ",
      $rg_cpreads[$rgid], $rg_cpread_chain[$rgid], $rg_cpread_usecs[$rgid]);
   printf ("%4.1f, %3.1f, %4d, ",
      $rg_greads[$rgid], $rg_gread_chain[$rgid], $rg_gread_usecs[$rgid]);
   printf ("%4.1f, %3.1f, %4d, ",
      $rg_gwrites[$rgid], $rg_gwrite_chain[$rgid], $rg_gwrite_usecs[$rgid]);
   print ("\n");
}

sub average_vol_stats {
   local ($nrgrps, $volid) = @_ ;
   local ($d);

   $vol_ut[$volid] = 0;
   $vol_xfers[$volid] = 0; 
   $vol_ureads[$volid] = 0; 
   $vol_uread_chain[$volid] = 0; 
   $vol_uread_usecs[$volid] = 0;
   $vol_writes[$volid] = 0; 
   $vol_write_chain[$volid] = 0; 
   $vol_write_usecs[$volid] = 0;
   $vol_cpreads[$volid] = 0; 
   $vol_cpread_chain[$volid] = 0; 
   $vol_cpread_usecs[$volid] = 0;
   $vol_greads[$volid] = 0; 
   $vol_gread_chain[$volid] = 0; 
   $vol_gread_usecs[$volid] = 0;
   $vol_gwrites[$volid] = 0; 
   $vol_gwrite_chain[$volid] = 0; 
   $vol_gwrite_usecs[$volid] = 0;

   for ($d = 0; $d <= $nrgrps; $d++) {
      $vol_ut[$volid]          += $rg_ut[$d]; 
      $vol_xfers[$volid]       += $rg_xfers[$d];
      $vol_ureads[$volid]      += $rg_ureads[$d];
      $vol_uread_chain[$volid] += $rg_uread_chain[$d];
      $vol_uread_usecs[$volid] += $rg_uread_usecs[$d];
      $vol_writes[$volid]      += $rg_writes[$d];
      $vol_write_chain[$volid] += $rg_write_chain[$d]; 
      $vol_write_usecs[$volid] += $rg_write_usecs[$d];
      $vol_cpreads[$volid]     += $rg_cpreads[$d];
      $vol_cpread_chain[$volid] += $rg_cpread_chain[$d];
      $vol_cpread_usecs[$volid] += $rg_cpread_usecs[$d];
      $vol_greads[$volid]       += $rg_greads[$d];
      $vol_gread_chain[$volid]  += $rg_gread_chain[$d];
      $vol_gread_usecs[$volid]  += $rg_gread_usecs[$d];
      $vol_gwrites[$volid]      += $rg_gwrites[$d];
      $vol_gwrite_chain[$volid] += $rg_gwrite_chain[$d];
      $vol_gwrite_usecs[$volid] += $rg_gwrite_usecs[$d];

   }

   $vol_ut[$volid] /= $nrgrps;
   if ( !$show_totals ) { $vol_xfers[$volid] /= $nrgrps; }
   if ( !$show_totals ) { $vol_ureads[$volid] /= $nrgrps; }
   $vol_uread_chain[$volid] /= $nrgrps;
   $vol_uread_usecs[$volid] /= $nrgrps;
   if ( !$show_totals ) { $vol_writes[$volid] /= $nrgrps; }
   $vol_write_chain[$volid] /= $nrgrps;
   $vol_write_usecs[$volid] /= $nrgrps;
   if ( !$show_totals ) { $vol_cpreads[$volid] /= $nrgrps; }
   $vol_cpread_chain[$volid] /= $nrgrps;
   $vol_cpread_usecs[$volid] /= $nrgrps;
   if ( !$show_totals ) { $vol_greads[$volid] /= $nrgrps; }
   $vol_gread_chain[$volid] /= $nrgrps;
   $vol_gread_usecs[$volid] /= $nrgrps;
   if ( !$show_totals ) { $vol_gwrites[$volid] /= $nrgrps; }
   $vol_gwrite_chain[$volid] /= $nrgrps;
   $vol_gwrite_usecs[$volid] /= $nrgrps;
   $vol_ndisks[$volid] = $nrgrps * $disk; # make this better.
}

sub init_all_sample_averages {
   local ($volid) = @_ ;

   $all_ut[$volid] = 0;
   $all_xfers[$volid] = 0;
   $all_ureads[$volid] = 0;
   $all_uread_chain[$volid] = 0;
   $all_uread_usecs[$volid] = 0;
   $all_writes[$volid] = 0;
   $all_write_chain[$volid] = 0;
   $all_write_usecs[$volid] = 0;
   $all_cpreads[$volid] = 0;
   $all_cpread_chain[$volid] = 0;
   $all_cpread_usecs[$volid] = 0;
   $all_greads[$volid] = 0;
   $all_gread_chain[$volid] = 0;
   $all_gread_usecs[$volid] = 0;
   $all_gwrites[$volid] = 0;
   $all_gwrite_chain[$volid] = 0;
   $all_gwrite_usecs[$volid] = 0;
}

sub add_to_all_sample_averages { 
   local ($volid) = @_ ;

   $all_ut[$volid] +=  $vol_ut[$volid];          
   $all_xfers[$volid] +=   $vol_xfers[$volid] ;
   $all_ureads[$volid] +=   $vol_ureads[$volid] ;   
   $all_uread_chain[$volid] +=   $vol_uread_chain[$volid] ;
   $all_uread_usecs[$volid] +=   $vol_uread_usecs[$volid] ;
   $all_writes[$volid] +=   $vol_writes[$volid] ;
   $all_write_chain[$volid] +=   $vol_write_chain[$volid] ;
   $all_write_usecs[$volid] +=   $vol_write_usecs[$volid] ;
   $all_cpreads[$volid] +=   $vol_cpreads[$volid] ;
   $all_cpread_chain[$volid] +=   $vol_cpread_chain[$volid] ;
   $all_cpread_usecs[$volid] +=   $vol_cpread_usecs[$volid] ;
   $all_greads[$volid] +=   $vol_greads[$volid] ;
   $all_gread_chain[$volid] +=   $vol_gread_chain[$volid] ;
   $all_gread_usecs[$volid] +=   $vol_gread_usecs[$volid] ;
   $all_gwrites[$volid]  +=   $vol_gwrites[$volid] ;
   $all_gwrite_chain[$volid] +=   $vol_gwrite_chain[$volid] ;
   $all_gwrite_usecs[$volid] +=   $vol_gwrite_usecs[$volid] ;
   $all_ndisks[$volid] = $vol_ndisks[$volid];
}

sub compute_all_sample_averages { 
   local ($volid) = @_ ;

   $all_ut[$volid]           /= $sample;
   $all_xfers[$volid]        /= $sample;
   $all_ureads[$volid]       /= $sample;
   $all_uread_chain[$volid]  /= $sample;
   $all_uread_usecs[$volid]  /= $sample;
   $all_writes[$volid]       /= $sample;
   $all_write_chain[$volid]  /= $sample;
   $all_write_usecs[$volid]  /= $sample;
   $all_cpreads[$volid]      /= $sample;
   $all_cpread_chain[$volid] /= $sample;
   $all_cpread_usecs[$volid] /= $sample;
   $all_greads[$volid]       /= $sample;
   $all_gread_chain[$volid]  /= $sample;
   $all_gread_usecs[$volid]  /= $sample;
   $all_gwrites[$volid]      /= $sample;
   $all_gwrite_chain[$volid] /= $sample;
   $all_gwrite_usecs[$volid] /= $sample;
}

sub month_to_numeric {
   local ($month) = @_ ;
   my $result;

   if ($month =~ /Jan/ ) {
	$result = 1;
   }
   elsif ($month =~ /Feb/ ) { 
        $result = 2;
   }
   elsif ($month =~ /Mar/ ) { 
        $result = 3;
   }
   elsif ($month =~ /Apr/ ) { 
        $result = 4;
   }
   elsif ($month =~ /May/ ) { 
        $result = 5;
   }
   elsif ($month =~ /Jun/ ) { 
        $result = 6;
   }
   elsif ($month =~ /Jul/ ) { 
        $result = 7;
   }
   elsif ($month =~ /Aug/ ) { 
        $result = 8;
   }
   elsif ($month =~ /Sep/ ) { 
        $result = 9;
   }
   elsif ($month =~ /Oct/ ) { 
        $result = 10;
   }
   elsif ($month =~ /Nov/ ) { 
        $result = 11;
   }
   elsif ($month =~ /Dec/ ) { 
        $result = 12;
   }
   else {
        $result = -1;
   }
   return $result;
}

sub usage {
   print <<EOM;
psf_vol.pl [-DhHnStrX] [-f <filer>] [-V <vol>]  <perfstat filename>

Summarizes disk statistics by volume. 

    Arguments and switches - 
       -D          Print data as well as header. Only relevant if -H specified.
       -h          This message.
       -H          Print headers only unless -D specified. 
       -n          Print in 'nice' output. Rather than printing in long
                   lines of values, print one line per value with a 
                   descriptive label.
       -f <filer>  For multi-filer perfstat files, indicates which filer. 
       -S          Average together all samples. Don't display individual samples. 
       -t          Print per volume (or raidgroup) totals for relevant counts 
                   rather than per disk averages. Relevant counts are the 
                   different request rate counts, not chain lengths or 
                   response times. 
       -r          Print per raidgroup statistics as well as per volume. 
       -X          Print additional extended values. These include:
                     Sdisk - Average disk service time computed as U/X 
                             where U is disk utilization and 
                                   X is disk transfers per second. 
                             Displayed in msecs.
                     Rdisk - Average disk response time. Computed as weighted
                             (by throughput) average of :  
                             Ruread, Rwrite, Rcpread, Rgread, Rgwrite - Average
                             disk response time for the given operations
                             computed as usecs*chain for each. 
                             Displayed in msecs.
                     Qlen - Average disk queue depth. Does not include 
                            host based queue depth, only queue depth on the disks.
                            Computed as Q = XR, where X is xfers/sec, and R is 
                            Rdisk as described above converted to seconds. 
                     Busy_Qlen - Average disk queue depth when the disk is busy. 
                            Computed as Qb = XR/U = XR/XS = R/S. 
       -V <vol>   Only process the volume specified. 
EOM
exit;

}
