#!/usr/bin/env perl 

# This script is provided "as is" it is
# not warranted but may be supported on a limited basis
# by Network Appliance.
#
# psf_wafl.pl
#
#   PerfStatFilter for certain WAFL stats .
#
#   v0.07  04-19-2005 : Added the output of active_map & summ_map blocks written 
#   v0.06  07-19-2004 : Added the output of active_map blocks (read) per CP i,
#                        snap summary blcks read per CP, total KB written per CP
#   v0.06  07-19-2004 : Added the output of sync and async active map blocks. 
#   v0.05  03-19-2004 : Added -f <filer> for multi-filer perfstat files.
#                       Only tested against perfstat 5.20 Linux version. 
#                       The only reliable way to get good data for such files
#                       is to use this option. 
#   v0.04  03-02-2004 : Print header before opening file.                  
#   v0.03  12-09-2003 : Changed from specificying #!/usr/local/bin to       
#                        #!/usr/bin/env perl . This even seemed to work on 
#                        my Win2K system. 
#   v0.01  11-04-2003 : Initial implementation.
#

# EXPORTED CONSTANTS (Not really, you have to copy them)

use constant FIRST_DATA                     => 1;
use constant _NA_TIMESTAMP                  => 0;
use constant _NA_WAFL_READS                 => 1;
use constant _NA_WAFL_READ_RESTARTS         => 2;
use constant _NA_WAFL_WRITES                => 3;
use constant _NA_WAFL_WRITE_RESTARTS        => 4;
use constant _NA_WAFL_RAIDIO_READ_DONE      => 5;
use constant _NA_WAFL_RAIDIO_WRITE_DONE     => 6;
use constant _NA_WAFL_BLKS_READ_OR_RA       => 7;
use constant _NA_WAFL_BLKS_WRITTEN          => 8;
use constant _NA_WAFL_BLKS_RA_SPECULATIVE   => 9;
use constant _NA_WAFL_BLKS_READ_L0          => 10;
use constant _NA_WAFL_BLKS_READ_INDIRECT    => 11;
use constant _NA_WAFL_BLKS_READ_OTHER       => 12;
use constant _NA_WAFL_BLKS_WRITTEN_L0       => 13;
use constant _NA_WAFL_BLKS_WRITTEN_INDIRECT => 14;
use constant _NA_WAFL_BLKS_WRITTEN_OTHER    => 15;
use constant _NA_WAFL_PW_OVERLIMIT          => 16;
use constant _NA_WAFL_PW_ASYNC_READ         => 17;
use constant _NA_WAFL_ACTIVE_MAP_RA         => 18;
use constant _NA_WAFL_ACTIVE_MAP_RD         => 19;
use constant _NA_WAFL_ACTIVE_MAP_RA_PER_CP  => 20;
use constant _NA_WAFL_ACTIVE_MAP_RD_PER_CP  => 21;
use constant _NA_WAFL_SNAP_SUMMARY_RD_PER_CP  => 22;
use constant _NA_WAFL_KB_WRITTEN_PER_CP     => 23;
use constant _NA_WAFL_ACTIVE_MAP_WRITTEN    => 24;
use constant _NA_WAFL_SUMMARY_MAP_WRITTEN   => 25;
use constant LAST_DATA                      => 25;


use Getopt::Std;
getopts("dDf:hHn");

if ($opt_h) { usage(); }

if ($opt_d) {
   $debug = 1;
} else {
   $debug = 0;
}

if ($opt_D) {
   $data_plus_header = 1;
} else {
   $data_plus_header = 0;
}

if ($opt_f) {
   $filer = $opt_f;
} 

if ($opt_H) {
   $header = 1;
} else {
   $header = 0;
}

if ($opt_n) {
   $nice_output = 1;
} else {
   $nice_output = 0;
}

$wafl_reads, $wafl_read_restarts, $wafl_writes, $wafl_write_restarts;
$raidio_read_done, $raidio_write_done;
$blks_read_or_ra, $blks_written, $ra_speculative;
$l0_blocks_read_or_ra, $indirect_blocks_read, $other_blocks_read,
$l0_blocks_written, $indirect_blocks_written, $other_blocks_written,
$over_limit, $async_read, $active_map_async, $active_map_sync, $active_map_async_per_cp, $active_map_sync_per_cp;
$snap_summ_read_per_cp, $kb_written_per_cp, $active_map_written, $snap_summ_written;

$data[_NA_WAFL_READS]                   = \$wafl_reads;
$data[_NA_WAFL_READ_RESTARTS]           = \$wafl_read_restarts;
$data[_NA_WAFL_WRITES]                  = \$wafl_writes;
$data[_NA_WAFL_WRITE_RESTARTS]          = \$wafl_write_restarts;
$data[_NA_WAFL_RAIDIO_READ_DONE]        = \$raidio_read_done;
$data[_NA_WAFL_RAIDIO_WRITE_DONE]       = \$raidio_write_done;
$data[_NA_WAFL_BLKS_READ_OR_RA]         = \$blks_read_or_ra;
$data[_NA_WAFL_BLKS_WRITTEN]            = \$blks_written;
$data[_NA_WAFL_BLKS_RA_SPECULATIVE]     = \$ra_speculative;
$data[_NA_WAFL_BLKS_READ_L0]            = \$l0_blocks_read_or_ra;
$data[_NA_WAFL_BLKS_READ_INDIRECT]      = \$indirect_blocks_read;
$data[_NA_WAFL_BLKS_READ_OTHER]         = \$other_blocks_read;
$data[_NA_WAFL_BLKS_WRITTEN_L0]         = \$l0_blocks_written;
$data[_NA_WAFL_BLKS_WRITTEN_INDIRECT]   = \$indirect_blocks_written;
$data[_NA_WAFL_BLKS_WRITTEN_OTHER]      = \$other_blocks_written;
$data[_NA_WAFL_PW_OVERLIMIT]            = \$over_limit;
$data[_NA_WAFL_PW_ASYNC_READ]           = \$async_read;
$data[_NA_WAFL_ACTIVE_MAP_RA] 	        = \$active_map_async;
$data[_NA_WAFL_ACTIVE_MAP_RD] 	        = \$active_map_sync;
$data[_NA_WAFL_ACTIVE_MAP_RA_PER_CP] 	= \$active_map_async_per_cp;
$data[_NA_WAFL_ACTIVE_MAP_RD_PER_CP] 	= \$active_map_sync_per_cp;
$data[_NA_WAFL_SNAP_SUMMARY_RD_PER_CP] 	= \$snap_summ_read_per_cp;
$data[_NA_WAFL_KB_WRITTEN_PER_CP] 	= \$kb_written_per_cp;
$data[_NA_WAFL_ACTIVE_MAP_WRITTEN] 	= \$active_map_written;
$data[_NA_WAFL_SUMMARY_MAP_WRITTEN] 	= \$snap_summ_written;

$header_line = "Timestamp,      WAFL_Rd,Rd_Susp,WAFL_Wr,Wr_Susp,RAID_Rd,RAID_Wr,Blks_Rd,Blks_Wr,SpecuRd,RdL0Blk,RdIndBlk,RdO_Blk,WrL0Blk,WrIndBlk,WrO_Blk,SyncRfW,AsyncRfW,ActiveMapAsync, ActiveMapSync, ActiveMapAsyncPerCP, ActiveMapSyncPerCP, SnapSummRdPerCP, KBWrPerCP, WrActvMap, WrSumm, ";

if ( !$nice_output && $header ) {
    printf ("%s \n", $header_line);
    if ( !$data_plus_header ) {
        exit;
    }
}

$perfstat_file = $ARGV[0];
open(INPUTA, '<' . $perfstat_file) || die 'Unable to open perfstat file.';

$seconds = 0;
$async_read = 0;
$over_limit = 0;
$found_wafl_stats = 0;

$timestamp = `psf_filer_timestamp.pl $perfstat_file`;
chomp $timestamp;

# Get to POSTSTATs 
while ($Line = <INPUTA>) {
   if ($Line =~ /POSTSTAT/ ) {  
      last ;
   }
}

while ($Line = <INPUTA>) {
   if ( ( $Line =~ /^=-=-/ ) &&
           ( $Line =~ /POSTSTATS/ ) &&
           ( $Line =~ /wafl_susp/ ) ) {
      @line = split (/\s+/, $Line);
      $curr_filer = $line[2];
      if ( $debug ) {
              printf ("Current statit filer : %s \n", $curr_filer);
      }
   }
   elsif ($found_wafl_stats && 
          ($Line =~ /POSTSTATS/)) {
      last;
   }
   elsif ( ($Line =~ /WAFL statistics over / ) &&
           ( (!$filer) ||
              ($filer =~ /^$curr_filer/) ) ) {
      if ( $debug ) { print "Found wafl_susp \n"; }
      $found_wafl_stats = 1;
      $num_cps = 0;
      @line = split (/ +/, $Line);
      $seconds = $line[3];
      if ( $debug ) {
         printf ("seconds: %d \n",
                 $seconds );
     }
      $found_blks_read = 0;
      while ($Line = <INPUTA>) { 
               if ( ($Line =~ /^blks_read/ ) &&
                   (!$found_blks_read) ){
                  @line = split (/\s+/, $Line);
                  $blks_read = $line[2];
                  $found_blks_read = 1;   # needed because wafl stats now 
                                          # had 2 lines labeled blks_read
                  if ( $debug ) {
                      printf ("blks_read: %d \n",
                                     $blks_read );
                  }
                  next;
               }
               elsif ($Line =~ /^blks_written/ ){
                  @line = split (/\s+/, $Line);
                  $blks_written = $line[2];
                  next;
               }
               elsif ($Line =~ /^ra\.read\s+/ ){
                  @line = split (/\s+/, $Line);
                  $ra_read = $line[2];
                  if ( $debug ) {
                      printf ("ra.read: %d \n",
                                     $ra_read );
                  }
                  next;
               }
               elsif ($Line =~ /^ra\.speculative/ ){
                  @line = split (/\s+/, $Line);
                  $ra_speculative = $line[2];
                  next;
               }
               elsif ($Line =~ /^blocks read by type/ ){
                  $Line = <INPUTA>;   # level line
                  $Line = <INPUTA>;   # type line
                  $other_blocks_read  = 0;
                  while ($Line = <INPUTA>) {
                     chop $Line;
                     if ( length($Line) < 3  )  {
                         last;
                     }
                     elsif ($Line =~ /normal file/ ) {
                         @line = split (/\s+/, $Line);
                         $l0_blocks_read = $line[3];
                         $indirect_blocks_read = $line[4] + 
                                                 $line[5] +
                                                 $line[6] ;
                         if ( $debug ) {
                            printf ("L0 blocks read: %d   indirect blocks read: %d \n",
                                     $l0_blocks_read , $indirect_blocks_read );
                         }
                         next;
                     }
                     elsif ($Line =~ /active bitmap/ ) {
                         @line = split (/\s+/, $Line);
                         $active_map_sync   += $line[3] +
                                               $line[4] +
                                                 $line[5] +
                                                 $line[6] ;
                         next;
                     }
                     elsif ($Line =~ /snap summary/ ) {
                         @line = split (/\s+/, $Line);
                         $snap_summ_read_per_cp += $line[3] +
                                                    $line[4] +
                                                    $line[5] +
                                                    $line[6] ;
                         next;
                     }
                     else {
                         next;
                     }
                  }
               }
               elsif ($Line =~ /^blocks read ahead by type/ ){
                 $Line = <INPUTA>;   # level line
                 $Line = <INPUTA>;   # type line
                  $other_blocks_read_ahead  = 0;
                  while ($Line = <INPUTA>) {
                     chop $Line;
                     if ( length($Line) < 3 )  {
                         last;
                     }
                     elsif ($Line =~ /normal file/ ) { 
                         @line = split (/\s+/, $Line);
                         $l0_blocks_read_ahead = $line[3];
                         $indirect_blocks_read_ahead = $line[4] + 
                                                 $line[5] +
                                                 $line[6] ;
                         if ( $debug ) {
                            printf ("L0 blocks read ahead: %d   indirect blocks read ahead: %d \n",
                                     $l0_blocks_read_ahead , $indirect_blocks_read_ahead );
                         }
                         next;
                     }
                     elsif ($Line =~ /active bitmap/ ) {
                         @line = split (/\s+/, $Line);
                         $active_map_async   += $line[3] +
                                               $line[4] +
                                                 $line[5] +
                                                 $line[6] ;
                         next;
                     }
                     else {
                         @line = split (/\s+/, $Line);
                         $other_blocks_read_ahead += $line[3] +
                                                     $line[4] +
                                                     $line[5] +
                                                     $line[6] ;
                         next;
                     }
                 }
              }
              elsif ($Line =~ /^blocks written by type/ ){
                  $Line = <INPUTA>;   # level line
                  $Line = <INPUTA>;   # type line
                  $other_blocks_written  = 0;
                  while ($Line = <INPUTA>) {
                     chop $Line;
                     if ( length($Line) < 3 )  {
                         last;
                     }
                     elsif ($Line =~ /normal file/ ) {
                         @line = split (/\s+/, $Line);
                         $l0_blocks_written = $line[3];
                         $indirect_blocks_written = $line[4] +
                                                 $line[5] +
                                                 $line[6] ;
                         if ( $debug ) {
                            printf ("L0 blocks written: %d   indirect blocks written: %d \n",
                                     $l0_blocks_written , $indirect_blocks_written  );
                         }
                         next;
                     }
                     elsif ($Line =~ /active bitmap/ ) {
                         @line = split (/\s+/, $Line);
                         $active_map_written   += $line[3] +
                                               $line[4] +
                                                 $line[5] +
                                                 $line[6] ;
                         next;
                     }
                     elsif ($Line =~ /snap summary/ ) {
                         @line = split (/\s+/, $Line);
                         $snap_summ_written += $line[3] +
                                                    $line[4] +
                                                    $line[5] +
                                                    $line[6] ;
                         next;
		     }
                     else {
                         @line = split (/\s+/, $Line);
                         $other_blocks_written += $line[3] +
                                                     $line[4] +
                                                     $line[5] +
                                                     $line[6] ;
                         next;
                     }
                 }
             }

               elsif ($Line =~ /^pw\.over_limit/ ){
                  @line = split (/\s+/, $Line);
                  $over_limit = $line[2];
                  next;
               }
               elsif ($Line =~ /^pw\.async_read/ ){
                  @line = split (/\s+/, $Line);
                  $async_read = $line[2];
                  next;
               }
               elsif ($Line =~ /^cp_from/) {
                  @line = split (/\s+/, $Line);
                  $num_cps += $line[2];
                  next;
               }
               elsif ($Line =~ /WAFL_READ\s+=/ ) {
                  @line = split (/\s+/, $Line);
                  $wafl_reads = $line[3];
                  $wafl_read_restarts = $line[4];
                  next;
               }
               elsif ($Line =~ /WAFL_WRITE\s+=/ ) {
                  @line = split (/\s+/, $Line);
                  $wafl_writes = $line[3];
                  $wafl_write_restarts = $line[4];
                  next;
               }
               elsif ($Line =~ /RAIDIO_READ_DONE\s+=/ ) {
                  @line = split (/\s+/, $Line);
                  $raidio_read_done = $line[3];
                  next;
               }
               elsif ($Line =~ /RAIDIO_WRITE_DONE\s+=/ ) {
                  @line = split (/\s+/, $Line);
                  $raidio_write_done = $line[3];
                  last;
               }
      } 
   }
}
if ( $found_wafl_stats ) {
   # Change everything to per second rates 
   $wafl_reads /= $seconds;
   $wafl_writes /= $seconds;
   $wafl_read_restarts /= $seconds;
   $wafl_write_restarts /= $seconds;
   $raidio_read_done /= $seconds;
   $raidio_write_done /= $seconds;
   $blks_read_or_ra = ($blks_read + $ra_read)/$seconds;
   $ra_speculative /= $seconds;
   $kb_written_per_cp = 4 * ($blks_written / $num_cps);
   $blks_written /= $seconds;
   
   $l0_blocks_read_or_ra = ($l0_blocks_read_ahead + $l0_blocks_read)/$seconds;
   $indirect_blocks_read = ($indirect_blocks_read_ahead + $indirect_blocks_read)/$seconds ;
   $other_blocks_read = ($other_blocks_read_ahead + $other_blocks_read)/$seconds ;
   $active_map_async_per_cp = $active_map_async / $num_cps;
   $active_map_sync_per_cp = $active_map_sync / $num_cps;
   $snap_summ_read_per_cp /= $num_cps;
   $active_map_async /= $seconds;
   $active_map_sync /= $seconds;
   $l0_blocks_written /= $seconds;
   $indirect_blocks_written /= $seconds;
   $active_map_written /= $seconds;
   $snap_summ_written /= $seconds;
   $other_blocks_written /= $seconds;
   $async_read /= $seconds;
   $over_limit /= $seconds;

   if ( $nice_output ) {
      printf ("Measurement Interval                    : \t %.1f secs \n",  $seconds );
      printf ("WAFL_READs/sec                          : \t %d \n", $wafl_reads);
      printf ("WAFL_READ_RESTARTS/sec                  : \t %d \n", $wafl_read_restarts);
      printf ("WAFL_WRITEs/sec                         : \t %d \n", $wafl_writes);
      printf ("WAFL_WRITE_RESTARTS/sec                 : \t %d \n", $wafl_write_restarts);
      printf ("RAIDIO_READS_DONE/sec                   : \t %d \n", $raidio_read_done);
      printf ("RAIDIO_WRITES_DONE/sec                  : \t %d \n", $raidio_write_done);
      printf ("blocks read (or ra)/sec                 : \t %d \n", $blks_read_or_ra );
      printf ("blocks written/sec                      : \t %d \n", $blks_written  );
      printf ("blocks ra speculative/sec               : \t %d \n", $ra_speculative );
      printf ("blocks read (or ra)/sec L0              : \t %d \n", $l0_blocks_read_or_ra );
      printf ("blocks read (or ra)/sec indirect        : \t %d \n", $indirect_blocks_read );
      printf ("blocks read (or ra)/sec other           : \t %d \n", $other_blocks_read );
      printf ("blocks written/sec L0                   : \t %d \n", $l0_blocks_written );
      printf ("blocks written/sec indirect             : \t %d \n", $indirect_blocks_written );
      printf ("blocks written/sec other                : \t %d \n", $other_blocks_written );
      printf ("blocks read/sec sync for partial write  : \t %d \n", $over_limit );
      printf ("blocks read/sec async for partial write : \t %d \n", $async_read );
   }
   else {
      printf ("%s, ", $timestamp);
      for ( $i=FIRST_DATA; $i <= LAST_DATA; $i++ ) {
         printf ("%d,    ", ${$data[$i]} );
      }
      printf ("\n");
   }
}

sub usage {
   print <<EOM;
psf_wafl.pl [-dDHhn]  <perfstat filename>

Summarizes the wafl stats data from perfstat outpt files.

    Arguments and switches -
       -d          Print debug messages.
       -D          Print data as well as a header. Only relevent if -H specified. 
       -H          Print a header line only unless -D specified. 
       -h          This message.
       -n          Print Nice output, a value per line labeled with what it is.
EOM
exit;

}

