#!/usr/bin/env perl 

# This script is provided "as is" it is
# not warranted but may be supported on a limited basis
# by Network Appliance.
#
# psf_wafl_susp.pl
#
#   PerfStatFilter for certain WAFL suspends .
#
#   v0.01  05-05-2004 : Initial implementation.
#

# EXPORTED CONSTANTS (Not really, you have to copy them)

use constant FIRST_DATA                     => 1;
use constant _NA_TIMESTAMP                  => 0;
use constant _NA_WAFL_MSG                   => 1;
use constant _NA_WAFL_MSG_COUNT             => 2;
use constant _NA_WAFL_MSG_RATE              => 3;
use constant _NA_WAFL_SUSPEND_REASON        => 4;
use constant _NA_WAFL_SUSPEND_COUNT         => 5;
use constant _NA_WAFL_SUSPEND_RATE          => 6;
use constant _NA_WAFL_SUSPEND_PERCENTAGE    => 7;
use constant LAST_DATA                      => 7;


use Getopt::Std;
getopts("dDf:hHn");

if ($opt_h) { usage(); }

if ($opt_d) {
   $debug = 1;
} else {
   $debug = 0;
}

if ($opt_D) {
   $data_plus_header = 1;
} else {
   $data_plus_header = 0;
}

if ($opt_f) {
   $filer = $opt_f;
} 

if ($opt_H) {
   $header = 1;
} else {
   $header = 0;
}

if ($opt_n) {
   $nice_output = 1;
} else {
   $nice_output = 0;
}

$header_line = "Timestamp,      WAFL_MSG,   COUNT, RATE, SUSPEND_REASON, SUSP_COUNT, SUSP_RATE, SUSP_PERCENT, ";

if ( !$nice_output && $header ) {
    printf ("%s \n", $header_line);
    if ( !$data_plus_header ) {
        exit;
    }
}
%wafl_op_susp_list;
$perfstat_file = $ARGV[0];
open(INPUTA, '<' . $perfstat_file) || die 'Unable to open perfstat file.';

$seconds = 0;
$found_wafl_stats = 0;
$timestamp = `psf_filer_timestamp.pl $perfstat_file`;
chomp $timestamp;

# Get to POSTSTATs 
while ($Line = <INPUTA>) {
   if ($Line =~ /POSTSTAT/ ) {  
      last ;
   }
}

while ($Line = <INPUTA>) {
   if ( ( $Line =~ /^=-=-/ ) &&
           ( $Line =~ /POSTSTATS/ ) &&
           ( $Line =~ /wafl_susp/ ) ) {
      @line = split (/\s+/, $Line);
      $curr_filer = $line[2];
      if ( $debug ) {
              printf ("Current statit filer : %s \n", $curr_filer);
      }
   }
   elsif ( ($Line =~ /WAFL statistics over / ) &&
           ( (!$filer) ||
              ($filer =~ /^$curr_filer/) ) ) {
      if ( $debug ) { print "Found wafl_susp \n"; }
      $found_wafl_stats = 1;
      @line = split (/\s+/, $Line);
      $seconds = $line[3];
      $processing_susp_reason = 0;
      while ($Line = <INPUTA>) { 
               if ($Line =~ /^Suspend because of / ){
                  @line = split (/\s+/, $Line);
                  $susp_reason = $line[3];
                  chomp $susp_reason; 
                  $processing_susp_reason = 1;
                  if ( $debug ) {
                       printf ("Found SUSPEND reason : %s \n", $susp_reason);
                  }
                  next;
               }
               elsif ($processing_susp_reason){
                  chop $Line;
                  @line = split (/\s+/, $Line);
                  if ( $line[2] ) {
                     $susp_count = $line[1];
                     $susp_op    = $line[2];
                     ($wafl_op_susp_list {$susp_op})->{$susp_reason} = $susp_count;  
                  }
                  else {
                     $processing_susp_reason = 0;
                     last;
                  }
                  next;
               }
      }
   }
   elsif ( ($Line =~ /^New messages/ )  &&
           ( (!$filer) ||
              ($filer =~ /^$curr_filer/) ) ) {
      while ($Line = <INPUTA>) {
         if ($Line =~ /====/) {
            $all_done = 1;
            last;
         }
         @line = split (/\s+/, $Line);
         $tmp_wafl_msg = $line[1];
         $tmp_wafl_count = $line[3];
         $wafl_msg_count {$tmp_wafl_msg} = $tmp_wafl_count;
         if ( $debug ) {
            printf ("Found WAFL_MSG      : %s, %d  \n", 
                     $tmp_wafl_msg, $tmp_wafl_count);
         }
      }
      next;
   }
}
if ( $found_wafl_stats ) {

   if ( $nice_output ) {
   }
   else {
      foreach $wafl_msg ( keys %wafl_msg_count ) {
         $msg_count = $wafl_msg_count {$wafl_msg};
	if (!$msg_count) { next; }
         $msg_rate = $msg_count / $seconds;
         if ( $wafl_op_susp_list{$wafl_msg} ) { 
            $susp_list_ref = $wafl_op_susp_list{$wafl_msg};
            foreach $susp_reason ( keys %$susp_list_ref ) {
               $susp_count = $susp_list_ref->{$susp_reason};
               $susp_rate = $susp_count / $seconds;
               $susp_percent = $susp_count / $msg_count;
               printf ("%s, %s, %d, %.1f, %s, %d, %.1f, %.2f, \n",
                     $timestamp, $wafl_msg, $msg_count, $msg_rate, 
                     $susp_reason, $susp_count, $susp_rate, $susp_percent);
            }
         }
         else { 
              printf ("%s, %s, %d, %.1f, %s, %d, %.1f, %.2f, \n",
                     $timestamp, $wafl_msg, $msg_count, $msg_rate,
                     "NONE", 0, 0, 0 );
         }
      }
   }
}

sub usage {
   print <<EOM;
psf_wafl.pl [-dDHhn]  <perfstat filename>

Summarizes the wafl stats data from perfstat outpt files.

    Arguments and switches -
       -d          Print debug messages.
       -D          Print data as well as a header. Only relevent if -H specified. 
       -H          Print a header line only unless -D specified. 
       -h          This message.
       -n          Print Nice output, a value per line labeled with what it is.
EOM
exit;

}

